<?php
namespace Andre\WebtreesAddOn\Arbre;
/* =================================
*     V 1.0 for "arbre" V 1-7-2 for Webtrees 1.7.2
*   André DUMAS August 2015
*   http://www.adumas.1s.fr/wt
*
*  V-1-1  for arbre V-1-7-2-1
*   André DUMAS Sptember 2015
*
*  V-1-1-1
*
*	V-1-2 for Arbre V-1-7-6-1  & WT 1.7.6  25 JUN 2016
*
*	V-1-3 for Arbre V-1-7-8-1 20 NOV 2016
* 	V-1-4  for Arbre V-1-7-8-2 DEC 2016 with Fan chart
*	V-1-4  for Arbre V-1-7-9-1 JAN 2017 with Fan chart
*	V-1-5  for Arbre V-1-7-9-3 JAN 2017 with booklets
*	V-1-6  for Arbre V-1-7-9-4 FEB 2017 
*	V-1-6  for Arbre V-1-7-9-4b MAR 2017 
*	V-1-7  for Arbre V-1-7-9-6 AUG 2017 
*/
use Fisharebest\Webtrees\Controller\PageController;
use Fisharebest\Webtrees\Controller\BaseController;
use Fisharebest\Webtrees\Controller\IndividualController;
use Fisharebest\Webtrees\Database;
use Fisharebest\Webtrees\Filter;
use Fisharebest\Webtrees\Functions\Functions;
use Fisharebest\Webtrees\Functions\FunctionsPrint;
use Fisharebest\Webtrees\GedcomRecord;
use Fisharebest\Webtrees\Tree;
use Fisharebest\Webtrees\Family;
use Fisharebest\Webtrees\I18N;
use Fisharebest\Webtrees\Auth;
use Fisharebest\Webtrees\File;
use Fisharebest\Webtrees\Theme;
use Fisharebest\Webtrees\Individual;
use Andre\WebtreesAddOn\Arbre\Template\PdfTemplate;
use Fisharebest\Webtrees\Session;
use Fisharebest\Webtrees\Log;
use Andre\WebtreesAddOn\Arbre\Template\img_Class;


use PDO;
use TCPDF_STATIC;

class ArbreClass extends ArbreModule {
       
    const ArbreClass_CUSTOM_VERSION  = '1-7 2017-08-12  AD';
	
    private static  $monArbre ;
	
	
	
	public static function getMonArbre() {
        return self::$monArbre;
    }
	
// = = = = = = = = = = = = = = = =
/* creates a table with all the sosa numbers over 23 generations
 *
 * the id in position 1 is the root person. The other positions are filled according to the following algorithm
 * if an individual is at position $i then individual $i's father will occupy position ($i*2) and $i's mother
 * will occupy ($i*2)+1
 */


/* = = = = = = = = = = = = = = = =
 * 
 */
protected function ancestry_sosa_array2($ged_id, $rootid, $maxgen)
{
	global $WT_TREE;
	$arbre =array(array());
	if ($maxgen == 0) $maxgen = 5;
	$treesize = pow(2, ($maxgen));
	$tree_array = array();
	$L2 = log10(2);
	$tree_array[0] = "";
	$tree_array[1] = $rootid;
	$sosa = 1;
	$gen = 1;
	
	$arbre[] = array(
	'id' => $rootid,
	'ged_id' => $ged_id,
	'sosa' => $sosa,
	'gen' => $gen
	);
	
	//fill in the array
	for ($i = 1; $i < ($treesize / 2); $i++) {
		$tree_array[($i * 2) ] = false; // -- father
		$tree_array[($i * 2) + 1] = false; // -- mother
		if (!empty($tree_array[$i]) && ($tree_array[$i] != '?')) {
			$person = Individual::getInstance($tree_array[$i], $WT_TREE);
			$family = $person->getPrimaryChildFamily(); 
			if ($family) {
			  $gen = log10($i * 2) / $L2;
			  $gen = (int)$gen + 1;	
				if ($family->getHusband()) {
					$tree_array[$i * 2] = $family->getHusband()->getXref();
					$personid = $i * 2;
					
						$arbre[] = array(
									'id' => $family->getHusband()->getXref(),
									'ged_id' => $ged_id,
									'sosa' => $i * 2,
									'gen' => $gen
									);
				}	else {
						//pere inconnu
						$tree_array[$i * 2] = '?';
					$arbre[] = array(
									'id' => '?',
									'ged_id' => $ged_id,
									'sosa' => $i * 2,
									'gen' => $gen
									);
				}
				if ($family->getWife()) {
					$tree_array[$i * 2 + 1] = $family->getWife()->getXref();
					$personid = $i * 2 + 1;
				
						$arbre[] = array(
									'id' => $family->getWife()->getXref(),
									'ged_id' => $ged_id,
									'sosa' => $i * 2 + 1,
									'gen' => $gen
									);
					
					
				} else {
					//mere inconnu
					$tree_array[$i * 2 + 1] = '?';
					$arbre[] = array(
									'id' => '?',
									'ged_id' => $ged_id,
									'sosa' => $i * 2 + 1,
									'gen' => $gen
									);
				}
			} //if family
			else { // pas d asc
			    $gen = log10($i * 2) / $L2;
				$gen = (int)$gen + 1;
					//pere inconnu
					$tree_array[$i * 2] = '?';
					$arbre[] = array(
									'id' => '?',
									'ged_id' => $ged_id,
									'sosa' => $i * 2,
									'gen' => $gen
									);
						//mere inconnue
						$tree_array[$i * 2 + 1] = '?';
					$arbre[] = array(
									'id' => '?',
									'ged_id' => $ged_id,
									'sosa' => $i * 2 + 1,
									'gen' => $gen
									);
				
			}
		} //if treeid
		else {
			if ($i % 2 == 0){
				 $gen = log10($i * 2) / $L2;
				$gen = (int)$gen + 1;
					//pere inconnu
					$tree_array[$i * 2] = '?';
					$arbre[] = array(
									'id' => '?',
									'ged_id' => $ged_id,
									'sosa' => $i * 2,
									'gen' => $gen
									);
						$tree_array[$i * 2 + 1] = '?';
					$arbre[] = array(
									'id' => '?',
									'ged_id' => $ged_id,
									'sosa' => $i * 2 + 1,
									'gen' => $gen
									);				
						
			} else {
				//mere inconnue
				 $gen = log10($i * 2) / $L2;
				$gen = (int)$gen + 1;
					//pere inconnu
					$tree_array[$i * 2] = '?';
					$arbre[] = array(
									'id' => '?',
									'ged_id' => $ged_id,
									'sosa' => $i * 2,
									'gen' => $gen
									);
						$tree_array[$i * 2 + 1] = '?';
					$arbre[] = array(
									'id' => '?',
									'ged_id' => $ged_id,
									'sosa' => $i * 2 + 1,
									'gen' => $gen
									);
			}
			
		}
	} //for
	
	return $arbre;
}

	
		//
	protected function createTableOfGenerations($max, $pid) {
		global $controller,  $WT_TREE;
		
		$maxgen =  $max; 
		
		$show_full = "1";
		$ged_id = $WT_TREE->getTreeId(); // indice du gedcom courant
		$gidd = $WT_TREE->getName(); // nom du gedcom courant
		$arbre = array(array());
			
		$arbre = $this->ancestry_sosa_array2($ged_id, $pid, $maxgen);
		
		/*
		* On crée un tableau de type array(array(array()))
		* le premier indice est la generation
		*/
		$gen = 1;	
		foreach ($arbre as $generation) {
			if (!empty($generation)) {
					$gen = $generation['gen'];
						 $asc[$gen][] = $generation; 		
			}		
		}	
        self::$monArbre = $asc;	
		$data = serialize($asc);
		$this->pushToCache($data, 'main');
	return $asc;	
	}


/*
*	->data_arbre 
*	array match names-Lifespan with id 
*	from monArbre 
*	save in data_arbre
*/
	protected function formatDataForFan(){
		global $WT_TREE;
		/* data is an array */
		
		$gen = 1;
		for ($j = 1; $j = count(self::$monArbre) - 1; $j++){
			foreach (self::$monArbre[$j] as $record){
				$person = Individual::getInstance($record['id'],$WT_TREE);
				$guy[$j][] = $person->getFullName();
				$guy[$j][] = $person->getLifeSpan();
			}
		}
		
		$this->pushToCache(serialize($guy), 'data');
	}


protected function getTabHeader($pid) {
	global $WT_TREE;
				$login = '';
				$html ='<div id="entete"><table><tr><td>';	
				$html .= '<div style="padding-bottom:10px;">';
				if (Auth::check()){
					$login = 'log_in';			
					$html .= '<span><a id="arbre-pdf" style="margin-left:25px;" href="#" title="' . I18N::translate('Create a pdf') . '">';
				} else {
					$html .= '<span>';
					$html .= '<a id="" style="margin-left:25px;"  title="' . I18N::translate('Create a pdf if you are a member: subscribe for an account')  . '">';
				}
				$html .= '	<i class="icon-mime-application-pdf"></i></a></span>'.
				'</td>';
				$html .= '<td style="text-align:center;">' .
						 $this->selectSpecificTrees() .
						 '</td>';
				
				$html .= '<td style="text-align:center;">' . ArbreClass::selectBooklet() . '
						 </td>
						 <td style="text-align:center;">
						 
						 </td>
						 <td id="fct1" style="
											border:1px solid black;
											width:20%;
											">
						 <img class="' . $login .'" src="' . WT_MODULES_DIR . $this->getName(). '/images/Fan-chart1-01.png " 
										style="width:100%;"
										title="' . I18N::translate("Click to create a fan chart: pdf, 10 generations  max") . '">
						 </td>
						 </tr></table>
			</div>
			
			<div class="fcpopup popup_block"  >				
				<p>' . I18N::translate('Create a chart and export it in a pdf file') . '</p>' .	
				 $this->getDefaultPieColors() .
				 $this->selectTypeOfChart() .
				 $this->selectNbreGen() . 
				 $this->selectFormatPage() . 
				 '<table><tr><td><input type="submit" id="fcOK" value="OK"/>' . 
				 '<td style="width:80%;"></td><td>     ' .
				 '<input type="submit" id="fcCancel" value="' . I18N::translate('Cancel') . '"/></td></tr></table>' . '				 
			</div>
			<br>
			
			';
	return $html;
	}
	
	private  function selectTypeOfChart(){
		$html = '<fieldset><legend>' . I18N::translate('Which chart?')  .'</legend><p id="fanconf" style="display:none;">' . $this->getCbMultiPage() . '
		<span><label>Type: </label><select id="typeofchart">
			<option value="fan180"> 180° </option>
			<option value="fan270"> 270° </option>
			<option value="fan360"> 360° </option>
			</select>
		</span>
			' . $this->chooseSetOfColors() . '
		
		</p>';
	return $html;	
	} 

	
	private function chooseSetOfColors(){
		return 
				'<span class="setofcolors">
				<label>' . I18N::translate('Set of colors: ') . '</label>
			<select id="colorofchart">
			<option value="0">' . I18N::translate('Choose your set of colors') . ' </option>
			<option value="1">' . I18N::translate('Default configuration') . ' </option>
			<option value="2">' . I18N::translate('My configuration') . ' </option>
			<option value="3">' . I18N::translate(' Classic..') . ' </option>
			<option value="4">' . I18N::translate('Black/white: save ink') . ' </option>
			</select>
			</span>	';
	}
	
	/*
	*
	*
	*/
	
	private function selectFan(){
		$html = '
			<select id="selectFan">
				<option value="0">' . I18N::translate('Type of fan') . '</option>
				<option value="1">' . I18N::translate('180° Half circle') . '</option>
				<option value="2">' . I18N::translate('270° 3/4 circle') . '</option>
				<option value="3">' . I18N::translate('360° full circle') . '</option>
			</select>	
		';
		return $html;
	}
	
	
	public static function selectBooklet(){
		if (Auth::check()){
		$html = '
			<select id="booklet">
				<option value="0">' . I18N::translate('Type of booklet') . '</option>
				<option value="10">' . I18N::translate('Diptych: 2 sheets  A4/P') . '</option>
				<option value="11">' . I18N::translate('Diptych: 2 sheets A4/L') . '</option>
				<option value="12">' . I18N::translate('Diptych: 2 sheets A3/L') . '</option>
				<option value="1">' . I18N::translate('Triptych : 3 sheets A4/P') . '</option>
				<option value="2">' . I18N::translate('Triptych : 3 sheets A3/L A4/P A3/L') . '</option>
				<option value="3">' . I18N::translate('Triptych : 3 sheets A4/L A4/P A4/L') . '</option>
				<option value="4">' . I18N::translate('Triptych : 3 sheets Letter/P') . '</option>
				<option value="5">' . I18N::translate('Triptych : 3 sheets Lg/L Letter/P Lg/L') . '</option>
			</select>	
		';
		} else {
		$html = '
			<select id="booklet">
				<option value="0">' . I18N::translate('Type of booklet for members') . '</option>
				
			</select>	
		';	
		}
		return $html;
	}

	public static function selectSpecificTrees(){
		$html = '
			<select id="specificTrees">
			<option value="0">' . I18N::translate('Specific trees') . '</option>
				<option value="1">' . I18N::translate('Agnatic tree') . '</option>
				<option value="2">' . I18N::translate('Cognatic tree') . '</option>
			</select>	
		';
		return $html;
	}

	
//////avec des tableaux/////////////////
/*
*   Meilleur rendu en PDF
*
*  param: $maxgen nombre max de génération, defaut 5 si maxgen=''
*  
*	-> arbre html with table
*/
protected function getTableAscsss($max, $pid) {// sens classique, commence par le ligne du bas  bottom:xx
	global $controller, $WT_TREE;
		
	if ($max == '')   {$maxgen = $this->NumberOfGenerations;} else {$maxgen = $max;}
	
	$largeurMini = 940; // px  voir CSS
		
	$root = $pid;   
	$person = $this->getPerson($root);
	$this->arbre = $this->createTableOfGenerations($maxgen, $root);

	$html = ''; 
	$gen = $maxgen;
	
		$html .= '<div>
		<p><span class="titreArbre" >' .I18N::translate("Family tree : ") . $this->getPerson($root)->getFullName().
							'   '.
							$this->getPerson($pid)->getLifeSpan().
							'</span></p></div>
		<br><div>';
	 
/*
*
* Début tableau, chaque boucle for détermine une ligne, soit une génération
*
*/	
	$colspan = 1;
	$html .= '<table cellspacing="3" cellpadding="3">';
	$info = I18N::translate('Click inside the cell to get more details');
	
	
for ($i = $gen; $i>0; $i--) {	
			
			$cell = pow(2, $i -1) ; // nombre de cases
			$largeur = (int)($largeurMini)/$cell;	 
			$colspan = pow(2, $gen-$i);
			
				$html .= '<tr>';
			 
	foreach ($this->arbre[$i] as $generation) { 
		// on cherche la generation la plus grande
		$sosa = $generation['sosa'];
		 // on attaque une autre generation, nouvelle ligne: on recalcule
		 
			if ($generation['id'] != '?') { // ancetre connu
				 $person = $this->getPerson($generation['id']);
				 $pid = $generation['id'];
				 $color = $this->getBackgroundColor($person);
				 $plus = '<a href="#" data-width="500" data-rel="popup" class="poplight"><span class="iconz icon-zoomin '.$i.'"  
					title="' . I18N::translate('More detals about this id. ') . '"></span>
					<span class ="pid" style="display:none;">'.$pid.'</span></a>';
				 $label = $this->getLabel($person);
			} else { //ancetre inconnu
				$color = 'white';
				 $label = '	<p><a href=""> Inconnu </a>'.'?</p>';
				 $plus ='';
			}	 
					//une case
					if ($i<=4) {
				$html .= '<td title="'.$info.'" colspan="'.$colspan.'" style="max-width:'.$largeur.'px;background-color:'.$color.'" class="sosa sosa'.$sosa.'">'.$plus.'<span class="sosaNbre">'.
							$sosa.'</span>:'.
							$label.'</td>';
					} 
					elseif ($i <= 7) {
				$html .= '<td title="'.$info.'" colspan="'.$colspan.'" style="max-width:'.$largeur.'px;font-size:50%;background-color:'.$color.'" class="sosa sosa'.$sosa.'">'.$plus.'<span class="sosaNbre">'.
							$sosa.'</span>:'.
							$label.'</td>';

	} else {
				$html .= '<td colspan="'.$colspan.'" style="max-width:'.$largeur.'px;font-size:15%;background-color:'.$color.'" class="sosa sosa'.$sosa.'">'.plus.'<span class="sosaNbre">'.
							$sosa.'</span>:'.
							$label.'</td>';		
					}
			
					} //foreach		
		    $html .= '</tr>'; //on ferme la ligne correspondant à une generation
			$html .= '<tr><td class="separation" style="border:0 solid white;"></td></tr>'; //ligne de separation
			
} //for				
			$html .= '</table>';	 //fin du tableau					
		
			$html .= '</div>';
			
	return $html;
}	

public function printPDF($root, $gen, $format) {
		global $controller;
		
		$max  = $gen; 					
		$out = '';			
			$out .=  '<body>'.
				$this->getTableAscsss($max, $root).
				'</body>
				';
			$cache = $this->pushToCache($out);	
		return true;		
	}

	/*
	*		Arbre cache
	*
	*/
	
	
	protected function addCache(){
		$dir = $this->getSetting('ARBRE_CACHE_DIR');
		
		$errorDir = false;
		$errorFile = false;
		if ( $dir != null){
			$this->emptyCache();
			if ( ($f = $this->getSetting('ARBRE_CACHE_FILE')) != null){		
				$file = WT_DATA_DIR .  $dir . DIRECTORY_SEPARATOR . $f;
				$cofile = WT_DATA_DIR .  $dir . DIRECTORY_SEPARATOR . 'cogna_' . $f;
				$agfile = WT_DATA_DIR .  $dir . DIRECTORY_SEPARATOR . 'agna_' . $f;
				$datafile = WT_DATA_DIR .  $dir . DIRECTORY_SEPARATOR . 'data_' . $f;
				$fanfile = WT_DATA_DIR .  $dir . DIRECTORY_SEPARATOR . 'fan_arbre.png' ;
				if (!file_exists(WT_DATA_DIR . $dir)) {
					@mkdir(WT_DATA_DIR . $dir);
				}
				if (!file_exists($file)) {
					$handle = fopen($file, 'w+');
					fclose($handle);
				}
				if (!file_exists($agfile)) {
					$handle = fopen($agfile, 'w+');
					fclose($handle);
				}
				if (!file_exists($cofile)) {
					$handle = fopen($cofile, 'w+');
					fclose($handle);
				}
				if (!file_exists($datafile)) {
					$handle = fopen($datafile, 'w+');
					fclose($handle);
				}
				if (!file_exists($fanfile)) {
					$handle = fopen($fanfile, 'w+');
					fclose($handle);
				}
			} else {$errorFile = true;}
		Log::addConfigurationLog('Arbre: addCache dir OK');	
		} else { $errorDir = true;}	
		if ( ($errorDir == true) || ($errorFile == true) ){
		/* add Message */
		$message = I18N::translate('There is a problem with the cache...!');
		self::addMessage(1,'warning', '', $message);
		Log::addConfigurationLog('Arbre: Problem with installation: no cache dir see to the migration process and ARBRE_CACHE_DIR');
	}		
		
	}
	
	/*
	*	param $type: main/cogna/agna/data data->array
	*/
	protected  function pushToCache($page, $type = 'main'){ 
		switch ($type){
			case 'main':
			$dir = $this->getSetting('ARBRE_CACHE_DIR');
			$file = $this->getSetting('ARBRE_CACHE_FILE');
			if ($dir && $file){
				$cache = WT_DATA_DIR . $this->getSetting('ARBRE_CACHE_DIR') . DIRECTORY_SEPARATOR  . $this->getSetting('ARBRE_CACHE_FILE');
			} else {
				$cache = WT_DATA_DIR . 'arbre_cache/arbre.txt';
			}
			break;
			default:
			$dir = $this->getSetting('ARBRE_CACHE_DIR');
			$file = $this->getSetting('ARBRE_CACHE_FILE');
			if ($dir && $file){
				$cache = WT_DATA_DIR . $this->getSetting('ARBRE_CACHE_DIR') . DIRECTORY_SEPARATOR . $type . '_' . $this->getSetting('ARBRE_CACHE_FILE');
			} else {
				$cache = WT_DATA_DIR . 'arbre_cache/arbre.txt';
			}
			break;
		}	

		if (file_exists($cache)){
			$handle = fopen($cache,'w+');
			if (!fwrite($handle, $page)){
				/* add Message */
				$message = I18N::translate('There is a problem with the cache...!');
				self::addMessage(1,'warning', '', $message);
			}
			fclose($handle);
				return $cache;
		} else {
			/* add Message */
			$message = I18N::translate('There is a problem with the cache...!');
			self::addMessage(1,'warning', '', $message);
		}
	return $cache;	
	}
	
	public function getFromCache($type){
			
		if ($type == 'fan'){
			$cache_filename = WT_DATA_DIR . $this->getSetting('ARBRE_CACHE_DIR') . DIRECTORY_SEPARATOR . $type . '_arbre.png';
		} else {	
		$cache_filename = WT_DATA_DIR . $this->getSetting('ARBRE_CACHE_DIR') . DIRECTORY_SEPARATOR . $type . '_' . $this->getSetting('ARBRE_CACHE_FILE');
		}
		if (file_exists($cache_filename)){
			return file_get_contents($cache_filename);
		} else {
			return false;
		}
	}

	protected function emptyCache(){
		if (is_dir(WT_DATA_DIR . $this->getSetting('ARBRE_CACHE_DIR'))){
			foreach (glob(WT_DATA_DIR . $this->getSetting('ARBRE_CACHE_DIR') . '/*.*')  as $file ){
				if(in_array(strtolower($this->getFileExt($file)), array("txt", "jpg","jpeg","png","gif"))){
					File::delete($file);
				}
			}
		}
	}
	
	public function getFileExt($filename){					 
	return $ext = pathinfo($filename, PATHINFO_EXTENSION);
	}
		/*
	*	$type info danger warning success
	*		jQ associé:
	*	$(".alert .close").on("click", function(){
	*					$(this).alert("close");
	*				});
	*
	*
	*/
	protected static function addMessage($id, $type, $hidden, $message = '') {
		$style = $hidden ? ' style=""' : ''; /* hidden : fade in */
		/* button or not button: it depends on... */
		$btClose = '<button type="button" class="close" aria-label="' . I18N::translate('close') . '">' .
			'<span aria-hidden="true">&times;</span>' .
			'</button>';
		$nobt ='';	
		
		switch ($type) {
			case 'warning':
				$bgkcolor = 'lightred';	
				$color = 'blue';
				$button = $btClose;
			break;	
			case 'wait' :
			$type = 'warning';
				$bgkcolor = 'lightred';	
				$color = 'lightblue';
				$button = $nobt;
			break;	
			case 'danger': 
				$bgkcolor = 'red';	
				$color = 'black';
				$button = $btClose;
			break;
			default:
				$bgkcolor = 'lightgrey';
				$color = 'black';
				$button = $btClose;
		}
		$bgk = 'style="
			width:500px;
			margin:auto;
			background-color:' .$bgkcolor .';
			color:' . $color .'
			border-radius:5px;
			text-align:center;
			text-weight:bold;
			border: 5px solid #fff;
			-moz-box-shadow: 8px 8px 12px #aaa; 
			-webkit-box-shadow: 8px 8px 12px #aaa; 
			box-shadow: 8px 8px 12px #555;
		"';
		
		return
			'<div  id="' . $id . '" class="alert alert-' . $type . ' alert-dismissible"' . $style . ' '.$bgk . '>' .
			$button .
			'<span class="message">' . $message . '</span>' .
			'</div>';
	}

	/**
	 * Show a waiting message while generating the PDF file
	 * This function is called from the Page Template
	 * 
	 * @return string
	 */
	public function getPdfWaitingMessage() {
		$bgkcolor = 'lightred'; $color = 'blue';
		$bgk = 'style="
			width:500px;
			margin:auto;
			background-color:' .$bgkcolor .';
			color:' . $color .'
			border-radius:5px;
			text-align:center;
			text-weight:bold;
			border: 5px solid #fff;
			-moz-box-shadow: 8px 8px 12px #aaa; 
			-webkit-box-shadow: 8px 8px 12px #aaa; 
			box-shadow: 8px 8px 12px #555;
		"';
		$type = 'warning';
		return
			'<div class="class="alert alert-' . $type . ' alert-dismissible"  ' . $bgk .'>' . I18N::translate('Creating PDF file. This process may take a while. Please wait...') . '</div>';
	}
///////////////////////////
	
	private function getLabel($person) {
		if ($person->canShow()){
	$html = '<p>
		<a class="arbreLien" href="'.$person->getHtmlUrl().'">'.
						$person->getFullName().'
					</a>'.' '.$person->getLifeSpan()
			.'</p>';
		} else {
			$html = 'Détails privés';
		}	
	return $html;
	}
	
		protected function getPerson($pid) {
		global $WT_TREE;
		return Individual::getInstance($pid, $WT_TREE);
	}

	private function getBackgroundColor($person) {
		switch ($person->getSex()) {
		case 'M':
			$color = 'lightblue';
		break;
		case 'F':
			$color = 'lightpink';
		break;
		case 'U':
			$color = 'lightgrey';
		break;
		default:
		    $color = 'white';
		}
	return $color;	
	}
	
	private function getXref($sosa, $gen) {
		foreach ($this->arbre[$gen-1] as $people) {
			if ($people['sosa'] == $sosa) {
					$pid = $people['id'];
					return $pid;
			}
		}
	}
	
	

	/*
	*	format agna/cogna trees
	*	$type = 'A' agna / 'C' cogna
	*/
	protected function specificTree($mothers, $type, $root){
		global $WT_TREE;
		$nb_col = count($mothers);
		if ($nb_col == 0){$nb_col = 1;}
		$largeur = 940;
		$nb_lignes = array();
		$ltab = array();
		$col = 0;
		$gen = 1;
		$nb_lignes_max = 0;	
		$tpdf = array();
		$html = '';
		$html .= '<table cellspacing="3" cellpadding="3">';
		
		$largcell = (int)($largeur/$nb_col);
		$styletd = 'align="center" valign="middle" 
					style="
							text-align:center;
							font-weight:bold;
							font-size:80%;
							border:1px solid black;
							border-radius:3px;
							padding:3px;
							width:' . $largcell . 'px; 
							'; 
		
		foreach ($mothers as $fam => $children){
			$l = 0;
			$pfam = Family::getInstance($fam, $WT_TREE);
			$ltab[$l][$col] 		= '<td ' . $styletd .'background-color:lightgrey;"><span class="pdf"><br></span><span> Gen:' . $gen .  '</span></td>';
			$tpdf[$l][$col]			= 'Gen:   ' . $gen;
			$l++;
			if ($pfam->getHusband()){
			$ltab[$l][$col] 		= '<td ' . $styletd .'background-color:' . $this->getBackgroundColor($pfam->getHusband() ) . ';">'. $this->getLabel($pfam->getHusband()) .  '</td>';
			$tpdf[$l][$col]			=  $pfam->getHusband()-> getXref();
			} else {
				$ltab[$l][$col] 		= '<td ' . $styletd .'background-color:lightblue;"><a href=""><span class="pdf"><br></span>' . ' ? ' . '</a></td>';
				$tpdf[$l][$col] 		= '';
			}
			$l++;
			$ltab[$l][$col] 	= '<td ' . $styletd .'background-color:lightgrey;"><span class="pdf"><br></span><span> x </span></td>';
			$tpdf[$l][$col]		= ' x ';
			$l++;
			if ($pfam->getWife()){
			$ltab[$l][$col] 	= '<td ' . $styletd .'background-color:' . $this->getBackgroundColor($pfam->getWife() ) . ';">'. $this->getLabel($pfam->getWife() ) .  '</td>';
			$tpdf[$l][$col]		= $pfam->getWife()->getXref();
			} else {
				$ltab[$l][$col] 	= '<td ' . $styletd .'background-color:lightpink;"><a href="">' . ' ? ' . '</a></td>';
				$tpdf[$l][$col]		='';
			}
			$l++;
			$ltab[$l][$col] 	= '<td ' . $styletd .'background-color:lightgrey;"><span class="pdf"><br></span>  ↓ </td>';
			$tpdf[$l][$col]		= '  ↓  ';
			$l++;				
			$nb_lignes[$col] = 5 + count($children);
			if ((count($children) + $l)  > $nb_lignes_max){$nb_lignes_max = count($children) + $l; }

				foreach ($children as $c => $child){
					$ltab[$l][$col] 	= '<td ' . $styletd .'background-color:' . $this->getBackgroundColor($child) . ';">'. $this->getLabel($child) .  '</td>';
					$tpdf[$l][$col]		= $child->getXref();
					$l++;
				} 
		$gen++;
		$col++;
		}
		$specificTree = array(	'type' => $type,
								'NbreLn' => $nb_lignes_max,
								'NbreCol' => $nb_col,
								'rows' => $nb_lignes,
								'root' => $root,
								'data'=>$tpdf);
		//save for peinting
			$this->pushToCache(serialize($specificTree), $type);
		//Display	
		$colmax = $gen++ ; 
		$col = 0;
		/* output table $html */
		for ($ligne = 0; $ligne <= $nb_lignes_max; $ligne++){
			$html .= '<tr>';			
				for ($i = 0; $i < $colmax - 1; $i++){ 
					if ($ligne < $nb_lignes[$i]){
						$html .= $ltab[$ligne][$i];
					} else {/* last row */
							$html .= '<td style="background-color:grey;"></td>';
					}	
				}
				//if ($col <= $colmax - 2){ $col++;}
				
			$html .= '</tr>';	
		}		
		$html .= '</table>';
		return $html;
	}

	/*
	*   popup fan chart Template1
	*
	*/
		private function selectFormatPage(){			
		$t = new TCPDF_STATIC;
		$formats = $t::$page_formats;
		$html = '<fieldset><legend>' . I18N::translate('Change Page format') . '</legend>
					<select id="formatpage">
					<option value="0">' . I18N::translate('Choose an other format..') . '</option>
					';	
						
		foreach ($formats as $ptype => $dim){
			$html .= '<option value="' . $ptype . '" >' . $ptype . '</option>'; 
		}
		$html .= '</select></fieldset>';
	return $html;	
	}
	

	
	
	private function selectNbreGen(){
		$html = '<fieldset><legend class="legendYN">' . I18N::translate('Number of generations/format : ') . '</legend>' .
				'<select id="nbregens" >' .
					'<option value="0" selected="selected"> 4/A4 </option>' .
					'<option value="1"> 5/A4 </option>' .
					'<option value="2"> 6/A4 </option>' .
					'<option value="3"> 7/A3 </option>' .
					'<option value="4"> 8/A3 </option>' .
					'<option value="5"> 9/A2 </option>' .
					'<option value="6"> 10/A1 </option>' .
					'<option value="7"> 11/A0 </option>' .
					'<option value="8"> 12/A0 </option>' .
					
				'</select>
				<span class="setofcolors">
					' .  $this->chooseSetOfColors(). '
				</span>
				</fieldset>';
		return $html;		
	}
	
	/*
	*	checkbox to choose multipage printing
	*
	*/
	private function getCbMultiPage(){
		return '
			<span><label> Multi pages
				<input type="checkbox" class="cbmp" />
			</label></span>
		';
	}
	
		/*
		*
		*
		*/
		protected function getColorPick(){
			
			$html = '<table id="confcolors">
						<tr>
							<td colspan="2"><span>' . I18N::translate('Choose colors for trees and/or fan charts') . '</span>
							</td>
						</tr>
						<tr>						
							<td colspan="2">
								<label><img src="' .img_Class::getImage('male') . '" style="width:15px;border:1px solid black;">
								<input  class="colorPicker cBgdMale" name="cBgdMale" />
								<input  class="colorPicker cTxtMale" name="cTxtMale"/>
								</label>
								<label><img src="' .img_Class::getImage('female') . '" style="width:15px;border:1px solid black;">
								<input  class="colorPicker cBgdFemale" name="cBgdFemale"/>
								<input  class="colorPicker cTxtFemale" name="cTxtFemale" value="T"/>
								</label>
								<label><img src="' .img_Class::getImage('unknown') . '" style="width:15px;border:1px solid black;">
								<input  class="colorPicker cBgdUnknown" name="cBgdUnknown"/>
								<input  class="colorPicker cTxtUnknown" name="cTxtUnknown"/>
								</label>								
							</td>							
						</tr>
						<tr>
							<td colspan="2">
								<table id="colorView"><tr>
									<td class="tdmale">
										<span>
											'. I18N::translate('male') . '
										</span>
									</td>
									<td class="tdfemale">
										<span>
											'. I18N::translate('female') . '
										</span>
									</td>
									<td class="tdunknown">
										<span>
											'. I18N::translate('unknown') . '
										</span>
									</td>
								</tr>
								<tr><td colspan="2">
									<input type="submit" id="bgdsubmit"  value="' . I18N::translate("Send")  .'">
								</td></tr>
								</table>
							</td>
						</tr>
						<tr>
							<table>
						<tr>
							<td colspan="2"><span>' . I18N::translate('Background images for fan charts and booklets') . '</span>
							</td>
						</tr>
						<tr class="wout">
							<td>
								<p><label title="One image"> Fan chart: <input type="radio" class="rfan" checked="checked" value=""></label></p>
								<p><label title="Two images"> Booklet 2P: <input type="radio" class="rbookllet" value=""></label></p>
								<p><label title="Two images: central and lateral if  3 pages"> Booklet 3P: <input type="radio" class="rbookllet" value=""></label></p>
							</td>
							<td>
							
							</td>
						</tr>
						<tr class="bgdimage">
							<td>' .
								$this->selectBackgroundImage() . '
							</td>
							<td>
								<img id="imgbgd" src="' . WT_MODULES_DIR . $this->getName() . '/images/bgd/' . '" alt="background image"/>
							</td>
							</tr></table>
						</tr>
						</table>
						';
			 
			return $html;		 	
		}
		
		
		/*
		* 	called from adminTemplate
		*	define backgrounds for trees/booklets...
		*/
		protected function selectBackground(){
			
			$html = '<table>
						<tr>
							<td colspan="2"><span><h3>' . I18N::translate('Background images for fan charts and booklets') . '</h3></span>
							</td>
						</tr>
						<tr class="wout">
							<td>
								<p><label style="width:180px;" title="One image"> Fan chart: </label></p>
							</td>
							<td class="fc">
								<p>default: <input type="radio" class="confbgk" name="confbgkfc" value="1"/>
									my conf: <input type="radio" class="confbgk" name="confbgkfc" value="2" checked="checked"/></p>
								' .
								$this->selectBackgroundImage() . '
							</td>
							</tr>							
							<tr>
							<td>
							<p><label style="width:180px;" title="' . I18N::translate('Same image on the two pages') .'"> Booklet 2P: </label></p>
							</td>
							<td class="b2p">
								<p>default: <input type="radio" class="confbgk" name="confbgk2p" value="1"/>
									my conf: <input type="radio" class="confbgk" name="confbgk2p" value="2" checked="checked" /></p>
								' .
								$this->selectBackgroundImage() . '
							</td>
							</tr>							
							<tr>
							<td>
							<p><label style="width:180px;" title="' . I18N::translate('Booklet 3 pages: image on the central page') .'"> Booklet 3P Center: </label></p>
							</td>
							<td class="b3pl">
								<p>default: <input type="radio" class="confbgk" name="confbgk3pl" value="1"/>
									my conf: <input type="radio" class="confbgk" name="confbgk3pl" value="2" checked="checked" /></p>
								' .
								$this->selectBackgroundImage() . '
							</td>
						</tr>
						<tr>
							<td>
							<p><label style="width:180px;" title="' . I18N::translate('Booklet 3 pages: image on the lateral page') .'"> Booklet 3P lateral: </label></p>
							</td>
							<td>
								<p>default: <input type="radio" class="confbgk" name="confbgk3p" value="1"/>
									my conf: <input type="radio" class="confbgk" name="confbgk3p" value="2" checked="checked" /></p>
								' .
								$this->selectBackgroundImage() . '
							</td>
						</tr>
						</table>
						</tr>
						</table>
						';
			 
			return $html;		 	
		}

	/*
	*	options defined for colors -> ln 302
	*/	
		
	protected function getDefaultPieColors(){
		$html ='<table id="defaultPieCols" style="width:95%;">
			<tr>
				<td style="width:20%;">
					<span>' .
						I18N::translate('Base conf :') 
					. '</span>
				</td>		
				<td>';
				$tmp = unserialize($this->getSetting('FAN_CHART_COLORS_M'));
				$html .= '
					<label><img src="' .img_Class::getImage('male') . '" style="width:15px;border:1px solid black;">
						<input  type="text" class="cmsBgdMale"  value="' . I18N::translate('Male') . '" readonly
								style="
									width:50%;
									text-align:center;
									background-color:rgb(' . 
										$tmp['bgd'][0] . ',' .
										$tmp['bgd'][1] . ',' .
										$tmp['bgd'][2] 
										. ');
									color:rgb(' . 
										$tmp['txt'][0] . ',' .
										$tmp['txt'][1] . ',' .
										$tmp['txt'][2] 
								. ');"
						/>
								</label>
				</td>
				<td>';
				$tmp = unserialize($this->getSetting('FAN_CHART_COLORS_F'));
				$html .= '
					<label><img src="' .img_Class::getImage('female') . '" style="width:15px;border:1px solid black;">
						<input  type="text" class="cmsBgdFemale"  value="' . I18N::translate('Female') . '" readonly
								style="
									width:50%;
									text-align:center;
									background-color:rgb(' . 
										$tmp['bgd'][0] . ',' .
										$tmp['bgd'][1] . ',' .
										$tmp['bgd'][2] 
										. ');
									color:rgb(' . 
										$tmp['txt'][0] . ',' .
										$tmp['txt'][1] . ',' .
										$tmp['txt'][2] 
								. ');"
						/>					
					</label>
					</td>
					<td>';
				$tmp = unserialize($this->getSetting('FAN_CHART_COLORS_U'));
				$html .= '
					<label><img src="' .img_Class::getImage('unknown') . '" style="width:15px;border:1px solid black;">
						<input  type="text" class="cmsBgdUnknown"  value="' . I18N::translate('Unknown') . '" readonly
								style="
									width:50%;
									text-align:center;
									background-color:rgb(' . 
										$tmp['bgd'][0] . ',' .
										$tmp['bgd'][1] . ',' .
										$tmp['bgd'][2] 
										. ');
									color:rgb(' . 
										$tmp['txt'][0] . ',' .
										$tmp['txt'][1] . ',' .
										$tmp['txt'][2] 
								. ');"
						/>					
					</label>
					</td>
					<td></td>
						</tr>
		
			<tr>
				<td style="width:20%;">
					<span>' .
						I18N::translate('Your conf :') 
					. '</span>
				</td>		
				<td>';
				$tmp = unserialize(Auth::user()->getPreference('FAN_CHART_COLORS_M'));
				$html .= '
					<label><img src="' .img_Class::getImage('male') . '" style="width:15px;border:1px solid black;">
						<input  type="text" class="cmsBgdMale"  value="' . I18N::translate('Male') . '" readonly
								style="
									width:50%;
									text-align:center;
									background-color:rgb(' . 
										$tmp['bgd'][0] . ',' .
										$tmp['bgd'][1] . ',' .
										$tmp['bgd'][2] 
										. ');
									color:rgb(' . 
										$tmp['txt'][0] . ',' .
										$tmp['txt'][1] . ',' .
										$tmp['txt'][2] 
								. ');"
						/>
								</label>
				</td>
				<td>';
				$tmp = unserialize(Auth::user()->getPreference('FAN_CHART_COLORS_F'));
				$html .= '
					<label><img src="' .img_Class::getImage('female') . '" style="width:15px;border:1px solid black;">
						<input  type="text" class="cmsBgdFemale"  value="' . I18N::translate('Female') . '" readonly
								style="
									width:50%;
									text-align:center;
									background-color:rgb(' . 
										$tmp['bgd'][0] . ',' .
										$tmp['bgd'][1] . ',' .
										$tmp['bgd'][2] 
										. ');
									color:rgb(' . 
										$tmp['txt'][0] . ',' .
										$tmp['txt'][1] . ',' .
										$tmp['txt'][2] 
								. ');"
						/>					
					</label>
					</td>
					<td>';
				$tmp = unserialize(Auth::user()->getPreference('FAN_CHART_COLORS_U'));
				$html .= '
					<label><img src="' .img_Class::getImage('unknown') . '" style="width:15px;border:1px solid black;">
						<input  type="text" class="cmsBgdUnknown"  value="' . I18N::translate('Unknown') . '" readonly
								style="
									width:50%;
									text-align:center;
									background-color:rgb(' . 
										$tmp['bgd'][0] . ',' .
										$tmp['bgd'][1] . ',' .
										$tmp['bgd'][2] 
										. ');
									color:rgb(' . 
										$tmp['txt'][0] . ',' .
										$tmp['txt'][1] . ',' .
										$tmp['txt'][2] 
								. ');"
						/>					
					</label>
					</td>
					<td>
						
									<a href="module.php?mod=arbre&mod_action=admin_config" target="_blank">
										<input style="padding:2px 5px 2px 5px;" type="submit" value="'.I18N::translate('Edit') .'" />
									</a>
									
						</tr>
					</table>		
		';
	return $html;	
	}	

	private function selectBackgroundImage(){
		$dir = WT_MODULES_DIR . $this->getName() .'/images/bgd';
		$images = self::getBgdImages($dir);
		$index = 2;
		$html = '
			<select class="bgdimages">
				<option value="0">' . I18N::translate('Choose an image..') . '</option>
				<option value="1">' . I18N::translate('no background') . '</option>';
				foreach ($images as $image){
				$html .= '<option value="' .
						$index . ' ">' .
						$image .'</option>';
						$index++;
				}
			$html .= '</select>	';
	return $html;	
	}	

	private static function getBgdImages($dir){
			$files = array();
			$dh  = opendir($dir);
			while (false !== ($fileName = readdir($dh))) {
				$ext = substr($fileName, strrpos($fileName, '.') + 1);
				if(in_array(strtolower($ext), array("jpg","jpeg","png","gif")))
					$files[] = $fileName;
			}
	return $files;		
	}

	
	/**
	 * Use javascript to include the stylesheet(s) in the header
	 *
	 * @param type $css
	 * @return javascript
	 */
	protected function includeCss($css) {
		return
			'<script>
				var newSheet=document.createElement("link");
				newSheet.setAttribute("href","' . $css . '");
				newSheet.setAttribute("type","text/css");
				newSheet.setAttribute("rel","stylesheet");
				newSheet.setAttribute("media","all");
				document.getElementsByTagName("head")[0].appendChild(newSheet);
			</script>';
	}

	
	/*
	*  Change colors background/text 
	*	in the admin page 
	*	for trees/fan charts
	*	called from adminTempate
	* 	$what TM/TF/TU   FM/FF/FU
	*	$conf 1/2  default/person
	*	return colors array
	*/
	protected function getAdminStyle($what, $conf){
		 //change radio default <-> personal conf
			
				// $conf default = 1; personal = 2
				$type = $what[0]; // Fan / Tree
				if ($type == 'F'){
					$prefix = 'FAN_CHART_COLORS_';
				} else {
					$prefix = 'TREE_COLORS_';
				}
				$gender = $what[1]; //M/F
				
					if ($conf == 1){
							$tab =  unserialize($this->getSetting($prefix . $gender));
					} else {
							$tab =  unserialize(Auth::user()->getPreference($prefix . $gender));
					}
					
			
		return $tab;	
	}
	
	public static function delAccents($text){
    $p=array(
        '/[ÀÁÂÃÄÅ]/u',
		'/[àáâãäå]/u',
        '/[ÈÉÊË]/u',
		'/[éèêë]/u',
        '/[ÌÍÎÏ]/u',
		'/[ìíîï]/u',
        '/[ÒÓÔÕÖØ]/u',
		'/[òóôõöø]/u',
        '/[ÙÚÛÜ]/u',
		'/[ùúûü]/u',
        '/[ýÿ]/u',
        '/[Ç]/u',
		'/[ç]/u',
        '/[Ñ]/u',
		'/[ñ]/u',
		'/[ ]/u',
        '/[^a-zA-Z0-9-]/'
    );
	if (($text == ' ? ') || ($text == ' ?  ? ')) {return $text;}
    $s = preg_replace($p,array('A', 'a', 'E', 'e', 'I', 'i', 'O', 'o', 'U', 'u','y', 'C', 'c', 'N', 'n', ' '),$text);
    //return preg_replace('/-{2,}/','-',$s);
	return $s;
}
	
}
