<?php
/**
 * webtrees: online genealogy
 * Copyright (C) 2015 webtrees development team
 * Copyright (C) 2015 JustCarmen
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
namespace Andre\WebtreesAddOn\Arbre\Schema;

use Fisharebest\Webtrees\Database;
use Fisharebest\Webtrees\I18N;
use Fisharebest\Webtrees\Schema\MigrationInterface;
use Fisharebest\Webtrees\Tree;
use Fisharebest\Webtrees\Log;



/**
 * Upgrade the database schema from version 1 to version 2.
 */
class Migration1 implements MigrationInterface {

	/** {@inheritDoc} */
	public function upgrade() {		
	//Default values: one data / sex to -> user's Preference limited to 255 cars
	//Initialise colors for fan chart: ARBRE_FAN_COLORS
	
	$lightblue = array(173, 216, 230);
	$lightpink = array(254, 182, 193);
	$lightgrey = array(211, 211, 211);
	$white = array(255, 255, 255);
	$beigeclair = array(245, 245, 220);
	$greyp = array(206, 206, 206);
	$blue_duck = array(4, 139, 154);
	$honey = array(218, 179, 10);
	$silver = array(206, 206, 206);
	$black = array(0, 0, 0);
	
	
	$fan_background_image  = 'bgd_filigrane_WT.png';
	$fan_colors = array('M' => array('bgd'=>$blue_duck, 'txt'=>$white ),
						'F' => array('bgd'=>$honey, 'txt'=>array(0, 0, 255) ),
						'U' => array('bgd'=>$silver, 'txt'=>$black )
						);
	//colors for trees with rectangles
	
	$tree_background_image  = 'bgd_filigrane_WT.png';
	$booklet_background_center = 'bgd_filigrane_WT';
	$booklet_background_lat = 'bgd_filigrane_WT';
	
	$rec_colors = array('M' => array('bgd'=>$blue_duck, 'txt'=>$white ),
						'F' => array('bgd'=>$honey, 'txt'=>array(0, 0, 255) ),
						'U' => array('bgd'=>$silver, 'txt'=>$black )
						);
Log::addConfigurationLog('Arbre: FAN TREE colors  values IN');

	//fist clean module_setting`
	Database::prepare(
				"DELETE FROM `##module_setting`	WHERE " . 
				"(module_name = 'arbre'" .
				" AND (" .
					"(setting_name LIKE '%FAN_%') " .
					" OR (setting_name LIKE '%TREE_%')" .
					" OR  (setting_name LIKE '%BOOKLET_%')" . 
					") ) ")->execute();
	Log::addConfigurationLog('Arbre: clean  values');


	$sql = 	"INSERT INTO `##module_setting` (module_name, setting_name, setting_value) VALUES " .		
			"('arbre','FAN_CHART_COLORS_M','" . serialize($fan_colors['M']) ."')," .
				"('arbre', 'TREE_COLORS_M', '" . serialize($rec_colors['M']) ."')," .
				"('arbre','FAN_CHART_COLORS_F' , '" . serialize($fan_colors['F']) ."')," .
				"('arbre', 'TREE_COLORS_F', '" . serialize($rec_colors['F']) ."')," .
				"('arbre','FAN_CHART_COLORS_U' , '" . serialize($fan_colors['U']) ."')," .
				"('arbre', 'TREE_COLORS_U', '" . serialize($rec_colors['U']) ."')
			";
				
	Log::addConfigurationLog('Arbre:  IN global :'. $sql);
	
	Database::prepare($sql)->execute();	
	Log::addConfigurationLog('Arbre: FAN TREE colors  values');
	
	$sql = "INSERT INTO `##module_setting` (module_name, setting_name, setting_value) VALUES  " .
				"('arbre','FAN_CHART_BGD' , '" . $fan_background_image ."')," .
				"('arbre','TREE_BGD' , '" . $tree_background_image ."')," .
				"('arbre','BOOKLET_BGD_CENTER' , '" . $booklet_background_center ."')," .
				"('arbre','BOOKLET_BGD_LAT' , '" . $booklet_background_lat ."');";				
			Database::prepare($sql)->execute();
			Log::addConfigurationLog('Arbre: default colors values');
	
	
		// fin creation 
	}
}

?>