<?php
/*
 * webtrees: online genealogy
 * Copyright (C) 2016 webtrees development team
 * Copyright (C) 2016 JustCarmen
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
namespace Andre\WebtreesAddOn\Arbre\Template;

/*
*	V-1-0 for arbre V-1-7-9-6 12 AUG 2017 ADMIN
* 	site http://www.adumas.1s.fr/wt     http://adumas.shost.ca/    http://webtrees.adumas.shost.ca/
*
*/

use Fisharebest\Webtrees\Auth;
use Fisharebest\Webtrees\Controller\PageController;
use Fisharebest\Webtrees\Controller\BaseController;
use Fisharebest\Webtrees\Controller\AjaxController;
use Fisharebest\Webtrees\Filter;
use Fisharebest\Webtrees\Functions\FunctionsEdit;
use Fisharebest\Webtrees\I18N;
use Fisharebest\Webtrees\Individual;
use Fisharebest\Webtrees\Tree;
use Fisharebest\Webtrees\Session;
use Andre\WebtreesAddOn\Arbre\ArbreClass;
use Andre\WebtreesAddOn\Arbre\Template\img_Class;

use Fisharebest\Webtrees\Site;
use Andre\WebtreesAddOn\Arbre\myUpdateFunctions;



class AdminTemplate extends ArbreClass {
	const THIS_CUSTOM_VERSION = '1-1';
	const THIS_CUSTOM_VERSION_DATE = '22-09-2017';

	protected function pageContent() {
		$controller = new PageController;
		return
			$this->pageHeader($controller) .
			$this->pageBody($controller);
	}

	private function pageHeader(PageController $controller) {
		$controller
			->restrictAccess(Auth::isAdmin())
			->setPageTitle(I18N::translate('Arbre admin'))
			->pageHeader()
			->addInlineJavascript('
						var moduleName = "' . $this->getName() .'";
						')
			->addExternalJavascript(
						WT_MODULES_DIR . $this->getName() .'/js/admin.js')
			->addExternalJavascript(
						WT_MODULES_DIR . $this->getName() . '/colorPicker/colorPicker.js')
			;

		// add javascript and styleseheet
		//$this->includeJs($controller, 'admin');
		$css = WT_MODULES_DIR . $this->getName() . '/colorPicker/colorPicker.css';
		echo $this->includeCss($css);
		$css1 = WT_MODULES_DIR . $this->getName() . '/css/admin.css';
		echo $this->includeCss($css1);
		
	}

	private function pageBody(PageController $controller) {
		
		if ( null == ($conf = Filter::get('conf'))){
			$conf = 2;
		}
		
		if (null !== ($what = Filter::get('what')) ){
			$type = $what[0];
			$gender = $what[1];
		} else {
			$what = 'ALL';
		}		
		echo $this->getBandeau($controller);
		?>
		<!-- ADMIN PAGE CONTENT -->

								<p class="small text-muted">
									<?php echo /* I18N: Help text for creating Arbre conf */ I18N::translate('Use this page  below to define colors for trees and fans. ')
									. ' ' .
									I18N::translate('Default conf is pre-defined but NOT your own one (-_+). ')
									. '  ' .
									I18N::translate('Define it with the RGB values and save it for each one. ')
									?>
								</p>
							</div>
						<div >
						
			<div id="admincolors">
			<p><i><?php echo I18N::translate('Clicking on "Send" you save only one option, repeat the operation to change more than one option') ?> </i></p>
			<fieldset><legend>			
					<h2><?php echo I18N::translate('Fan chart') ?></h2></legend>	
					<div id="fan">
						<div class="idFM">
							<div class="im">
								<p></p>
								<label><img src="
									<?php echo img_Class::getImage('male') ?>
									"
									style="width:25px;border:1px solid black;">
								</label>	
							</div>
							<?php   
									
										$style = $this->getAdminStyle('FM', $conf);
									
									$bstyle = $style['bgd'];
									$tstyle = $style['txt'];
									$brgb = 'rgb('. $bstyle[0]. ',  ' . $bstyle[1] . ', ' . $bstyle[2] . ')';
									
									$trgb = $tstyle[0]. ',  ' . $tstyle[1] . ', ' . $tstyle[2];
							?>
							<div class="idgenre"
								style="
									background-color: <?php echo $brgb; ?> ;
									color:rgb( <?php echo $trgb ?> );
								">
								<p></p>
								<span><?php echo I18N::translate('male') ?></span>
							</div>
							<div class="bcgd">
								<fieldset><legend>Background</legend>
									<label>R: <input class="rgbR" value=" <?php echo $bstyle[0]; ?> "/></label>
									<label>G: <input class="rgbG" value="<?php echo $bstyle[1]; ?>"/></label>
									<label>B: <input class="rgbB" value="<?php echo $bstyle[2]; ?>"/></label>
								</fieldset>
							</div>
							<div class="txt">
								<fieldset><legend>Names</legend>
								<label>R: <input class="rgbR" value="<?php echo $tstyle[0]; ?>"/></label>
								<label>G: <input class="rgbG" value="<?php echo $tstyle[1]; ?>"/></label>
								<label>B: <input class="rgbB" value="<?php echo $tstyle[2]; ?>"/></label>
							
								</fieldset>
							</div>
							<div class="send">
							<?php if ($conf == 2){ ?>
								default: <input type="radio" name="confFM" value="1"/>
								my conf: <input type="radio" name="confFM" value="2" checked="checked" />
							<?php } else { ?>
								default: <input type="radio" name="confFM" value="1" checked="checked"/>
								my conf: <input type="radio" name="confFM" value="2" />
							<?php } ?>
								<input type="submit" class="editcolorsFM"/>
							</div>
							
						</div>
						
						<div class="idFF">
							<div class="im">
								<p></p>
								<label><img src="
									<?php echo img_Class::getImage('female') ?>
									"
									style="width:25px;border:1px solid black;">
								</label>	
							</div>
							<?php   
										$style = $this->getAdminStyle('FF', $conf);
									
									$bstyle = $style['bgd'];
									$tstyle = $style['txt'];
									$brgb = 'rgb('. $bstyle[0]. ',  ' . $bstyle[1] . ', ' . $bstyle[2] . ')';
									$trgb = $tstyle[0]. ',  ' . $tstyle[1] . ', ' . $tstyle[2];
							?>
							<div class="idgenre"
										style="
										background-color: <?php echo $brgb; ?> ;
										color:rgb( <?php echo $trgb ?> );
										"
							>
								<p></p>
								<span><?php echo I18N::translate('female') ?></span>
							</div>
							<div class="bcgd">
								<fieldset><legend>Background</legend>
									<label>R: <input class="rgbR" value="<?php echo $bstyle[0]; ?>"/></label>
									<label>G: <input class="rgbG" value="<?php echo $bstyle[1]; ?>"/></label>
									<label>B: <input class="rgbB" value="<?php echo $bstyle[2]; ?>"/></label>
								</fieldset>
							</div>
							<div class="txt">
								<fieldset><legend>Names</legend>
								<label>R: <input class="rgbR" value="<?php echo $tstyle[0]; ?>"/></label>
								<label>G: <input class="rgbG" value="<?php echo $tstyle[1]; ?>"/></label>
								<label>B: <input class="rgbB" value="<?php echo $tstyle[2]; ?>"/></label>
								</fieldset>
							</div>
							<div class="send">
								<?php if ($conf == 2){ ?>
								default: <input type="radio" name="confFF" value="1"/>
								my conf: <input type="radio" name="confFF" value="2" checked="checked" />
							<?php } else { ?>
								default: <input type="radio" name="confFF" value="1" checked="checked"/>
								my conf: <input type="radio" name="confFF" value="2" />
							<?php } ?>
								 <input type="submit" class="editcolorsFF"/>
							</div>
						</div>
						
						<div class="idFU">
							<div class="im">
								<p></p>
								<label><img src="
									<?php echo img_Class::getImage('unknown') ?>
									"
									style="width:25px;border:1px solid black;">
								</label>	
							</div>
							<?php   $style  = $this->getAdminStyle('FU', $conf);
									$bstyle = $style['bgd'];
									$tstyle = $style['txt'];
									$brgb = 'rgb('. $bstyle[0]. ',  ' . $bstyle[1] . ', ' . $bstyle[2] . ')';
									$trgb = $tstyle[0]. ',  ' . $tstyle[1] . ', ' . $tstyle[2];
							?>
							<div class="idgenre" 
										style="
										background-color: <?php echo $brgb; ?> ;
										color:rgb( <?php echo $trgb ?> );
										"
							>
								<p></p>
								<span><?php echo I18N::translate('unknown') ?></span>
							</div>
							<div class="bcgd">
								<fieldset><legend>Background</legend>
									<label>R: <input class="rgbR" value="<?php echo $bstyle[0]; ?>"/></label>
									<label>G: <input class="rgbG" value="<?php echo $bstyle[1]; ?>"/></label>
									<label>B: <input class="rgbB" value="<?php echo $bstyle[2]; ?>"/></label>
								</fieldset>
							</div>
							<div class="txt">
								<fieldset><legend>Names</legend>
								<label>R: <input class="rgbR" value="<?php echo $tstyle[0]; ?>"/></label>
								<label>G: <input class="rgbG" value="<?php echo $tstyle[1]; ?>"/></label>
								<label>B: <input class="rgbB" value="<?php echo $tstyle[2]; ?>"/></label>
								</fieldset>
							</div>
							<div class="send">
								<?php if ($conf == 2){ ?>
								default: <input type="radio" name="confFU" value="1"/>
								my conf: <input type="radio" name="confFU" value="2" checked="checked" />
							<?php } else { ?>
								default: <input type="radio" name="confFU" value="1" checked="checked"/>
								my conf: <input type="radio" name="confFU" value="2" />
							<?php } ?>
								 <input type="submit" class="editcolorsFU"/>
							</div>
						</div>


						</div>
			</fieldset>		
			<br>
			<fieldset><legend>	
					<h2><?php echo I18N::translate('Tree') ?></h2></legend>
					<div id="tree">
						<div class="idTM">
							<div class="im">
								<p></p>
								<label><img src="
									<?php echo img_Class::getImage('male') ?>
									"
									style="width:25px;border:1px solid black;">
								</label>	
							</div>
							<?php   $style  = $this->getAdminStyle('TM', $conf);
									$bstyle = $style['bgd'];
									$tstyle = $style['txt'];
									$brgb = 'rgb('. $bstyle[0]. ',  ' . $bstyle[1] . ', ' . $bstyle[2] . ')';
									$trgb = $tstyle[0]. ',  ' . $tstyle[1] . ', ' . $tstyle[2];
							?>
							<div class="idgenre" 
										style="
										background-color: <?php echo $brgb; ?> ;
										color:rgb( <?php echo $trgb ?> );
										"
							>
								<p></p>
								<span><?php echo I18N::translate('male') ?></span>
							</div>
							<div class="bcgd">
								<fieldset><legend>Background</legend>
									<label>R: <input class="rgbR" value="<?php echo $bstyle[0]; ?>"/></label>
									<label>G: <input class="rgbG" value="<?php echo $bstyle[1]; ?>"/></label>
									<label>B: <input class="rgbB" value="<?php echo $bstyle[2]; ?>"/></label>
								</fieldset>
							</div>
							<div class="txt">
								<fieldset><legend>Names</legend>
								<label>R: <input class="rgbR" value="<?php echo $tstyle[0]; ?>"/></label>
								<label>G: <input class="rgbG" value="<?php echo $tstyle[1]; ?>"/></label>
								<label>B: <input class="rgbB" value="<?php echo $tstyle[2]; ?>"/></label>
								</fieldset>
							</div>
							<div class="send">
								<?php if ($conf == 2){ ?>
								default: <input type="radio" name="confTM" value="1"/>
								my conf: <input type="radio" name="confTM" value="2" checked="checked" />
							<?php } else { ?>
								default: <input type="radio" name="confTM" value="1" checked="checked"/>
								my conf: <input type="radio" name="confTM" value="2" />
							<?php } ?>
								 <input type="submit" class="editcolorsTM"/>
							</div>
						</div>
						
						<div class="idTF">
							<div class="im">
								<p></p>
								<label><img src="
									<?php echo img_Class::getImage('female') ?>
									"
									style="width:25px;border:1px solid black;">
								</label>	
							</div>
							<?php   $style  = $this->getAdminStyle('TF', $conf);
									$bstyle = $style['bgd'];
									$tstyle = $style['txt'];
									$brgb = 'rgb('. $bstyle[0]. ',  ' . $bstyle[1] . ', ' . $bstyle[2] . ')';
									$trgb = $tstyle[0]. ',  ' . $tstyle[1] . ', ' . $tstyle[2];
							?>
							<div class="idgenre" 
										style="
										background-color: <?php echo $brgb; ?> ;
										color:rgb( <?php echo $trgb ?> );
										"
							>
								<p></p>
								<span><?php echo I18N::translate('female') ?></span>
							</div>
							<div class="bcgd">
								<fieldset><legend>Background</legend>
									<label>R: <input class="rgbR" value="<?php echo $bstyle[0]; ?>"/></label>
									<label>G: <input class="rgbG" value="<?php echo $bstyle[1]; ?>"/></label>
									<label>B: <input class="rgbB" value="<?php echo $bstyle[2]; ?>"/></label>
								</fieldset>
							</div>
							<div class="txt">
								<fieldset><legend>Names</legend>
								<label>R: <input class="rgbR" value="<?php echo $tstyle[0]; ?>"/></label>
								<label>G: <input class="rgbG" value="<?php echo $tstyle[1]; ?>"/></label>
								<label>B: <input class="rgbB" value="<?php echo $tstyle[2]; ?>"/></label>
								</fieldset>
							</div>
							<div class="send">
								<?php if ($conf == 2){ ?>
								default: <input type="radio" name="confTF" value="1"/>
								my conf: <input type="radio" name="confTF" value="2" checked="checked" />
							<?php } else { ?>
								default: <input type="radio" name="confTF" value="1" checked="checked"/>
								my conf: <input type="radio" name="confTF" value="2" />
							<?php } ?>
								 <input type="submit" class="editcolorsTF"/>
							</div>
						</div>

						<div class="idTU">
							<div class="im">
								<p></p>
								<label><img src="
									<?php echo img_Class::getImage('unknown') ?>
									"
									style="width:25px;border:1px solid black;">
								</label>	
							</div>
							<?php   $style  = $this->getAdminStyle('TU', $conf);
									$bstyle = $style['bgd'];
									$tstyle = $style['txt'];
									$brgb = 'rgb('. $bstyle[0]. ',  ' . $bstyle[1] . ', ' . $bstyle[2] . ')';
									$trgb = $tstyle[0]. ',  ' . $tstyle[1] . ', ' . $tstyle[2];
							?>
							<div class="idgenre" 
										style="
										background-color: <?php echo $brgb; ?> ;
										color:rgb( <?php echo $trgb ?> );
										"
							>
								<p></p>
								<span><?php echo I18N::translate('unknown') ?></span>
							</div>
							<div class="bcgd">
								<fieldset><legend>Background</legend>
									<label>R: <input class="rgbR" value="<?php echo $bstyle[0]; ?>"/></label>
									<label>G: <input class="rgbG" value="<?php echo $bstyle[1]; ?>"/></label>
									<label>B: <input class="rgbB" value="<?php echo $bstyle[2]; ?>"/></label>
								</fieldset>
							</div>
							<div class="txt">
								<fieldset><legend>Names</legend>
								<label>R: <input class="rgbR" value="<?php echo $tstyle[0]; ?>"/></label>
								<label>G: <input class="rgbG" value="<?php echo $tstyle[1]; ?>"/></label>
								<label>B: <input class="rgbB" value="<?php echo $tstyle[2]; ?>"/></label>
								</fieldset>
							</div>
							<div class="send">
								<?php if ($conf == 2){ ?>
								default: <input type="radio" name="confTU" value="1"/>
								my conf: <input type="radio" name="confTU" value="2" checked="checked" />
							<?php } else { ?>
								default: <input type="radio" name="confTU" value="1" checked="checked"/>
								my conf: <input type="radio" name="confTU" value="2" />
							<?php } ?>
								 <input type="submit" class="editcolorsTU"/>
							</div>
						</div>


						</div>
			</fieldset>			
		</div> <!-- end  admincolors     -->
		
		<!--   admin background -->
			<div class="bkg">	
				
				<?php echo $this->selectBackground() ?>
			</div>
		<?php	
	}
	
		
		/////////////////// Header ///////////////////

	private function getBandeau($controller){
		$linkdoc = Site::getPreference('MY_VERSIONS_DOC_BASE_URL') . $this->getName() . '.html';
		$linklog = Site::getPreference('MY_VERSIONS_DOC_BASE_URL') . $this->getName() . '_log.txt';
		$linkupd = 'module.php?mod=' . $this->getName() . '&mod_action=check_version';
		
 		//update version?
		myUpdateFunctions::checkNewModsVersions();
		$spref = Site::getPreference('LATEST_MY_MODS_VERSION');
		
		if ($spref !== null){
			$new_version = unserialize($spref)[$this->getName()]['version'];
		} else {		
			$new_version = self::CUSTOM_VERSION;
		}

		$html = '<ol class="breadcrumb small">
			<li><a href="admin.php">'. I18N::translate('Control panel') . '</a></li>
			<li><a href="admin_modules.php">' . I18N::translate('Module administration') . '</a></li>
			<li class="active">' . $controller->getPageTitle() . '</li>
		</ol>';
		$html .= '<div style="
						border:2px solid black;
						border-radius:3px;
						padding:10px;">';
					
		$html .= '	<table style="width:95%;">
			<tr>
			<td style="width:65%; text-align:center;">
			<h3>' . $this->getTitle() . '  V-' . self::CUSTOM_VERSION . '   ' . '<span style="font-size:60%;">(' . 
							self::CUSTOM_VERSION_DATE . ')</span></h3>
			</td>
			<td style="text-align:center;"><span>
				<a href="' . $linkdoc . '">' .  I18N::translate('Documentation') . '
				</a></span></td>
			<td style="text-align:center;"><span>
			<a href="' . $linklog . '">' .  I18N::translate('Change.log') . '
			</a></span></td>
			<td style="text-align:center;" >
			<a href="' . $linkupd . '" title="' . I18N::translate('Checking is done every week, but you can force it now')  . '"><button 
												style="background-color:lightgrey;"
												>' . I18N::translate('Check new version now!') . '</button></a>
			</td>
			</tr>';
			
		$html .= '<tr><td style="text-align:center;" colspan="3">';
		//new version?
		$html .= myUpdateFunctions::lookForNewVersion($this->getName(), self::CUSTOM_VERSION, $new_version ) ;
		$html .= '</td></tr>';
		
		$html .= '</table></div><br>';

	return $html;
	}
	
	public function forceUpdate(){
		myUpdateFunctions::checkNewModsVersions($now = true);
		header('Location:module.php?mod=' . $this->getName() .'&mod_action=admin_config');
	}

}
