<?php
/**
 * webtrees: online genealogy
 * Copyright (C) 2015 webtrees development team
 * Copyright (C) 2015 JustCarmen
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *---------------------------------------------------------------------
 *   V 1.0.1 for "arbre" V-1-7-2 for Webtrees 1.7.2
*   André DUMAS August 2015
*   http://www.adumas.1s.fr/wt
*
*	V-1-0 for Arbre V-1-7-9-3  WT 1.7.9   2017
*		Pie-sectors
*		printed on 4 pages
 */
namespace Andre\WebtreesAddOn\Arbre\Template;

use Fisharebest\Webtrees\File;
use Fisharebest\Webtrees\Filter;
use Fisharebest\Webtrees\I18N;
use Fisharebest\Webtrees\Auth;
use Fisharebest\Webtrees\Individual;
use Andre\WebtreesAddOn\Arbre\ArbreClass;
use Andre\WebtreesAddOn\Arbre\ArbreModule;

use Andre\WebtreesAddOn\Arbre\Template\PdfPageController;
use Andre\WebtreesAddOn\Arbre\Template\myPDFTools;

use TCPDF;
use TCPDF_STATIC;
/*
*	To widen the fan chart, we divide it and  distribute it on 2/4 pages
*	fan-180 -> 2 pages
*	fan 270/360 -> 4 pages
*/
class PdfNewTreeTemplatePieMP2_1 extends ArbreModule {
	
	const PdfTemplate_CUSTOM_VERSION  = '2-0';
	const PdfTemplate_CUSTOM_VERSION_DATE  = '2017-09-20';
	const PdfTemplate_CUSTOM_VERSION_AUTHOR  = 'André DUMAS FRANCE';
	const WIDTH_A4 = 1841.890;
	
	
	
	// default : half circle
	var $startAngle; // 0
	var $stopAngle ; /// 180
	var $DRAW_COLOR_LIGHT;
	var $DRAW_COLOR_DARK;
	//colors and backgrounds
	var $fan_colors;
	var $bgd;
	var $centres;
		
	public function pageBody($format, $type, $colorSet , $ftype, $multipages ) {
		global $WT_TREE;
		
		$this->initColors($colorSet);
		$path_bgd = WT_MODULES_DIR . $this->getName() .'images/bgd/';
		
	 $this->DRAW_COLOR_LIGHT = array(255, 255, 255);
	 $this->DRAW_COLOR_DARK = array(90, 90, 90);
	
		switch ($ftype) {
			case 'fan180' : //2 pages
				$this->startAngle = 0; // 0
				$this->stopAngle  = 180; /// 180
			break;
			case 'fan270' : //4 pages
				$this->startAngle = -45; // 270°
				$this->stopAngle  = 225; 
			break;
			case 'fan360' :
				$this->startAngle = -90; // 360°
				$this->stopAngle  = 270; 
			break;
			default:
			break;
		}
		
		// on lit le tableau de valeurs
			
			$temp = WT_DATA_DIR . $this->getSetting('ARBRE_CACHE_DIR');
			
				$f = DIRECTORY_SEPARATOR . 'arbre.txt' ;
							
				$cache = $temp . $f;
				
				//Commence à 1 TODO
				$data = unserialize(file_get_contents($cache));
				// nbre de generations
				$gen = count($data);

				////
			$person = Individual::getInstance($data[1][0]["id"], $WT_TREE);		
			$titre = I18N::translate('Fan chart ') . '  ' . $person->getAllNames()[$person->getPrimaryName()]['fullNN']  ;
			
			$header = '<div style="width:100%;height:5%;">'.$WT_TREE->getTitle(). '     :     ' . $titre .'</div>';	
			$footer = '<div style="width:100%;">'
						. 'Module "arbre" for Webtrees @AD 2015-2016-2017'.'</div>';
						
			$header2 = '<div style="width:100%;height:5%;">'.' '. '     :     ' . '  ' .'</div>';	
			$footer2 = '<div style="width:100%;;text-align:right;">'
						. 'Module "arbre" for Webtrees @AD 2015-2016-2017'.'</div>';			
// ---------------------------------------------------------

if (!is_array($format)){
$PDF_PAGE_FORMAT = $format;
} else {
	$PDF_PAGE_FORMAT = 'A4';
}

$PDF_UNIT = 'pt';
$PDF_PAGE_ORIENTATION = 'P'; //L pour paysage

/////////////
$pdf = new PdfPageController($PDF_PAGE_FORMAT, PDF_PAGE_ORIENTATION);
$tool = new myPDFTools();
// set font
$size = 4;
$pdf->SetFont('dejavu sans', 'B', $size);
//-------------------------------------------------------------

// set auto page breaks
$pdf->SetAutoPageBreak(FALSE, PDF_MARGIN_BOTTOM);

//---------------------------------------------------------------
// add a page
$pdf->AddPage();  

// add page 2
$pdf->AddPage(); 

if ($ftype != 'fan180'){
	// add page 3
	$pdf->AddPage();

	// add page 4
	$pdf->AddPage();   
}
// set JPEG quality

$twidth  = $pdf->getPageWidth();
$theight = $pdf->getPageHeight(); //pdf->getPageHeight()
/////////////
//Cercle:
//Centre $xc, $yc
// X Y axis
$window = $this->stopAngle - $this->startAngle;

	/*
	* 	4 pages max 
	* 	ocenters on each page
	*   format  array( 'page' => array($cx, $cy))
	*/
	$this->centres = array(	'1' => array('x'=>$twidth*0.97, 'y' => $theight*0.97), 
						'2' => array('x' => $twidth*0.03, 'y' => $theight*0.97),
						'3' => array('x' => $twidth*0.97, 'y' => $theight*0.03),
						'4' => array('x' => $twidth*0.03, 'y' => $theight*0.03)
						);

		//default init

	$r = $twidth*0.95  / $gen;
	
///////////////
// background-image 
$img = $path_bgd . $this->bgd;
$this->setBackgroundImage($pdf, $img);

$pdf->SetPage(1);
$pdf->header = $header;
$pdf->Header();

//------------Boucle

$start = $this->startAngle;

$maxCharLn = 20; // nbr max chars/line

$size = $r / $maxCharLn; // 15 chars max
$pdf->SetFont('dejavu sans', 'B', $size);



//centres
//$xc1 = $xc;


for ($j = $gen; $j > 0; $j--){ 
	$ids = pow(2, $j - 1);
	$inc = $window / $ids;
	$r1 = $j * $r;
	$deb = $this->startAngle;
	$fin = 0;
	

	for ($i = 0; $i < $ids  ; $i++){
		$fin = $deb + $inc - 0.01;
		//******** 
				$person = Individual::getInstance($data[$j][$i]["id"], $WT_TREE);
			if ($person){
				$lifeInt = $tool->getMyLifeSpan($person);
				switch ($sex = $person->getSex()) {
				case 'M':
					$tcolor = $this->fan_colors['M']['txt'];;
					$color = $this->fan_colors['M']['bgd'];
					$pdf->SetTextColorArray($tcolor);					
					$pdf->SetColorArray('fill',$color);
				break;	
				case 'F':
					$tcolor = $this->fan_colors['F']['txt'];;
					$color = $this->fan_colors['F']['bgd'];
					$pdf->SetTextColorArray($tcolor);
					$pdf->SetColorArray('fill',$color);
				break;	
				default:
					$tcolor = $this->fan_colors['U']['txt'];;
					$color = $this->fan_colors['U']['bgd'];
					$pdf->SetTextColorArray($tcolor);
					$pdf->SetColorArray('fill',$color);
				break;	
			}
				$text = $tool->getMyShortName($person, $maxCharLn);
			} else {
				$text = ' ?  ? ';
				$lifeInt = '( ? - ? )';
				$tcolor = $this->fan_colors['U']['txt'];;
				$color = $this->fan_colors['U']['bgd'];
				$pdf->SetTextColorArray($tcolor);
				$pdf->SetColorArray('fill',$color);
			}
						
		//****************
		//which page?  -----------
		if ($i < ($ids/2))
		{
				if ( ($deb >= 0) && ( $deb < 90) ){ 
					$page = 1;										
				} else { 
					$page = 3;
				}
		} 
		elseif ($i >= ($ids/2)) 
		{ 
				if ( ($deb >= 90) && ( $deb < 180) ){ 
					$page = 2;
				} else {
					$page = 4;
				}	
		}
		//--------------------
			$pdf->SetPage($page);
			$pdf->SetTextColorArray($tcolor);
			$pdf->SetColorArray('fill',$color);
			$this->setLineColor($pdf, $colorSet);
				$xc = $this->centres[$page]['x'];
				$yc = $this->centres[$page]['y'];
		
		//origine
		if ( ($i == 0) && ( $j == 1) )
		{			
			$pdf->SetPage(1);
			$pdf->SetTextColorArray($tcolor);
				$pdf->SetColorArray('fill',$color);
				$this->setLineColor($pdf, $colorSet);
			
			$pdf->PieSector($this->centres['1']['x'], $this->centres['1']['y'], $r1, 0, 90, 'FD', true, 180);
			$pdf->SetPage(2);
			$pdf->SetTextColorArray($tcolor);
				$pdf->SetColorArray('fill',$color);
				$this->setLineColor($pdf, $colorSet);
			$pdf->PieSector($this->centres['2']['x'], $this->centres['2']['y'], $r1, 90, 180, 'FD', true, 180);
			if ($ftype != 'fan180'){
				$page = 4;
				$pdf->SetPage($page);
				$pdf->SetTextColorArray($tcolor);
				$pdf->SetColorArray('fill',$color);
				$this->setLineColor($pdf, $colorSet);
				$pdf->PieSector($this->centres[$page]['x'], $this->centres[$page]['y'], $r1, 180, $this->stopAngle, 'FD', true, 180);
				$page = 3;
				$pdf->SetPage($page);
				$pdf->SetTextColorArray($tcolor);
				$pdf->SetColorArray('fill',$color);
				$this->setLineColor($pdf, $colorSet);
				$pdf->PieSector($this->centres[$page]['x'], $this->centres[$page]['y'], $r1, $this->startAngle, 0, 'FD', true, 180);
			}	
		} 
		else {
			if (($deb < 0) && ($fin > 0))
			{
					$page = 3;
					$pdf->SetPage($page);
					$pdf->SetTextColorArray($tcolor);
				$pdf->SetColorArray('fill',$color);
				$this->setLineColor($pdf, $colorSet);
				$pdf->PieSector($this->centres[$page]['x'], $this->centres[$page]['y'], $r1, $deb, 0, 'FD', true, 180);
				
				$page = 1;
				$pdf->SetPage($page);
				$pdf->SetTextColorArray($tcolor);
				$pdf->SetColorArray('fill',$color);
				$this->setLineColor($pdf, $colorSet);				
				$pdf->PieSector($this->centres[$page]['x'], $this->centres[$page]['y'], $r1, 0, $fin, 'FD', true, 180);
				
			} 
			elseif (($deb < 180) && ($fin > 180)){
				$page = 2;
				$pdf->SetPage($page);
				$pdf->SetTextColorArray($tcolor);
				$pdf->SetColorArray('fill',$color);
				$this->setLineColor($pdf, $colorSet);
				$pdf->PieSector($this->centres[$page]['x'], $this->centres[$page]['y'], $r1, $deb, 180, 'FD', true, 180);
				
				$page = 4;
				$pdf->SetPage($page);
				$pdf->SetTextColorArray($tcolor);
				$pdf->SetColorArray('fill',$color);
				$this->setLineColor($pdf, $colorSet);
				$pdf->PieSector($this->centres[$page]['x'], $this->centres[$page]['y'], $r1, 180, $fin, 'FD', true, 180);
				
			} 
			else {
				$pdf->PieSector($this->centres[$page]['x'], $this->centres[$page]['y'], $r1, $deb, $fin, 'FD', true, 180);
			}
		}
		//
		
	//font size
	$size = $r*0.9/$maxCharLn;
		if ($r1*$inc*3.1415/180 < 4*$size){$size *= 0.9;}
	$pdf->SetFontSize($size);
			
	switch ($j) {
		case '1':			
		$person = Individual::getInstance($data[1][0]["id"], $WT_TREE);
		$lifeInt = $tool->getMyLifeSpan($person);
		$media = $person->findHighlightedMedia();
		$link = WT_BASE_URL . 'individual.php?pid=' . $data[1][0]["id"] . '&ged=' . $WT_TREE->getName();
		
		if (($media != null) && ($media->fileExists('thumb')) ) {
			$imgType = strtoupper($media->extension());
			if ($imgType == 'PNG' || $imgType == 'JPG' || $imgType == 'JPEG'){
				$imgPath = $media->getServerFilename('thumb');
			} 
		} 
			//Title	
				switch ($sex ) {
						case 'M':
							$pdf->SetTextColor(0, 0, 255);
							$pdf->SetFillColor(176, 213, 230);
							if (($media == null) || !($media->fileExists('thumb')) ){
								$imgType = 'PNG';
								$imgData = (img_Class::getImage('male'));
							}
						break;	
						case 'F':
							$pdf->SetTextColor(255, 0, 0);
							$pdf->SetFillColor(254, 182, 193);
							if (($media == null) || !($media->fileExists('thumb'))  ){
								$imgData = (img_Class::getImage('female'));
								$imgType = 'PNG';
							}
						break;	
						default:
							$pdf->SetTextColor(0, 0, 0);
							$imgData = (img_Class::getImage('unknown'));
							break;	
				}
			
			
		$imgW = ($theight / 8);
		$page = 1;
		$pdf->SetPage($page);
		if (($media == null) || !($media->fileExists('thumb')) ){
			$html = '<img  style="width:500pt;"
				src="'.$imgData . '" />';

			$pdf->writeHTMLCell(	$w= $imgW,
										$h = $imgW,
										$x= $twidth*0.08,
										$y = $theight*0.08,
										$html,
										$border = 0,
										$ln = 0,
										$fill = false,
										$reseth = true,
										$align ='',
										$autopadding = true 
								) ;		
		} else {
		$pdf->Image($imgPath, $twidth*0.08, $theight*0.08, $imgW, '', $imgType, '', 'T', false, 300, '', false, false, 1, false, false, false);	
		}

	$page = 2;
	$pdf->SetPage($page);	
	
		$tsize = $theight / 40;
		$pdf->SetFont('dejavu sans', 'B', $tsize);
		$name = $person->getAllNames()[$person->getPrimaryName()]['fullNN'] ;
		$xn = $this->centres['2']['x'] - strlen($name)* $tsize / 3;
			$text = $name . '  ' . $lifeInt;
			$l = $pdf->getStringWidth($text);
			$d = $twidth*0.9;
			$size = 1.8*$size*$d/$l;
			$pdf->setFontSize($size);
			
			$pdf->SetX($twidth*0.25);
			$pdf->SetY($theight*0.1);
			$pdf->setCellPadding(15);
			$pdf->setCellMargins($left= $twidth*0.20);						
			$pdf->Cell($twidth*0.65, 0, $text, 1, 1, 'C', 1,  $link, 4);
		
		$pdf->SetFont('dejavu sans', 'B', $size);
		$pdf->setCellMargins($left= 0);
		$pdf->setCellPadding(0);
		
		//First gen
		$text = explode(' ', ($tool->getMyShortName2($person, $maxCharLn*2)));
		
		$inc = $window;
		
		$size = 15;
		$pdf->SetFont('dejavu sans', 'B', $size); 
		if ( ($sex == 'M')){ //dark				 
				 $pdf->SetColorArray('text',$this->fan_colors['M']['txt']);
		} else {
			$pdf->SetColorArray('text',$this->fan_colors['F']['txt']);
		}
		
			$pdf->SetFontSize($size);	
			$this->arcWriteGen1($pdf, ArbreClass::delAccents($text[1]), $this->centres['1']['x'], $this->centres['1']['y'], $r, $size, $window );
			$pdf->SetFontSize($size*0.5);		
			$lfn = $pdf->GetStringWidth( $text[0]) ;	
			$lli = $pdf->GetStringWidth($lifeInt) ;
												
			$delta = (1.8*$r - $lfn)/2; 
			$deltai = ($r - $lli)/2; 
			
			//ftype
			
			$this->printTextCharbyChar($pdf, ArbreClass::delAccents($text[0]), $this->centres['1']['x'] - $r*0.9 + $delta, $this->centres['1']['y'] - 0.5 * $r, $size);
			$this->printTextCharbyChar($pdf, $lifeInt, $this->centres['1']['x'] -  1.5*$lli/2, $this->centres['1']['y'] - 0.3 * $r, $size);

			$pdf->SetFont('dejavu sans', 'B', $size); 
	 
		break;
		case '2':
			if ($person){
				//$text = ($tool->getMyShortName($person, $maxCharLn));	
				$t = ($tool->getMyTabNames($person, $maxCharLn));
			} else {
				$t = array(' ? ', ' ? ');
			}
			$text = ArbreClass::delAccents($t[1]) . '. '. ArbreClass::delAccents($t[0]);
			$this->arcWrite1($pdf, $text, $xc, $yc, $r1, $maxCharLn, $inc, $i);

			if ( ($ftype =='fan270') ){				
				$h = 0.55;				
				$inc2_270 = 90;
				
				if ($i == 0){
					$deb2_270 = 0;
					$page = 1;				
				} else {
					$deb2_270 = 90;
					$page = 2;
				}
				
				$pdf->setPage($page);	
				$this->writeRotLine($pdf, $this->centres[$page]['x'], $this->centres[$page]['y'], $deb2_270, $inc2_270, $r, $r1, $lifeInt, $h);	
				
			} elseif ( ($ftype =='fan360') ){
				$li = explode('-', $lifeInt);
				$h = 0.55;				
				$inc2_360 = 90;
				switch ($i){ 
					case '0':
					$page = 3;	
					$st = $li[0];
					$l = $pdf->getStringWidth($st);
					$deb2_360 = -4.5*$l*180/($r1*3.14156);
					$pdf->setPage($page);	
					$this->writeRotLine($pdf, $this->centres[$page]['x'], $this->centres[$page]['y'], $deb2_360, $inc2_360, $r, $r1, $st, $h);
					
					
					$deb2_360 = 0;
					$page = 1;
					$st = $li[1];
					$l = $pdf->getStringWidth($st);
					$inc2_360 = 2.5*$l*180/($r1*3.14156);
					$pdf->setPage($page);	
					$this->writeRotLine($pdf, $this->centres[$page]['x'], $this->centres[$page]['y'], $deb2_360, $inc2_360, $r, $r1, $st, $h);
					break;
					case '1':
						$page = 2;	
						$st = $li[0];
						$l = $pdf->getStringWidth($st);
						$deb2_360 = 180 - 4.5*$l*180/($r1*3.14156);				
						$pdf->setPage($page);	
						$this->writeRotLine($pdf, $this->centres[$page]['x'], $this->centres[$page]['y'], $deb2_360, $inc2_360, $r, $r1, $st, $h);
					
						$deb2_360 = 180;
						$page = 4;
						$st = $li[1];
						$l = $pdf->getStringWidth($st);
						$inc2_360 = 2.5*$l*180/($r1*3.14156);
						$pdf->setPage($page);	
						$this->writeRotLine($this->centres[$page]['x'], $this->centres[$page]['y'], $deb2_360, $inc2_360, $r, $r1, $st, $h);
					break;
				
				}

			} else {
				$h = 0.55;
				$this->writeRotLine($pdf, $xc, $yc, $deb, $inc, $r, $r1, $lifeInt, $h);
			}
		break;
		
		case   	'3':
		case   	'4':
		case 	'5':
		
			if ($person){
				$text = ($tool->getMyTabNames($person, $maxCharLn*6));
			} else {
				$text = array(' ? ', ' ? ');
			}
			
			$this->arcWrite1($pdf, ArbreClass::delAccents($text[0]), $xc, $yc, $r1, $maxCharLn, $inc, $i);
			
			$hli = 0.75;
			$fontli = 0.6;
			$hfn = 0.45;
			$fontfn = 0.9;
			
			//cross page?
			$ends = $deb + $inc;
			if ( ($deb < 0) && ($ends > 0) ){ // page 3 -> 1
			
				$p = -$deb / $inc;
				if ($p < 0.35){// go on page 1
					$page = 1;
					$pdf->setPage($page);
					$this->writeRotLine($pdf, $this->centres[$page]['x'], $this->centres[$page]['y'], 0, $inc + $deb, $r, $r1, $text[1], $hfn, $fontfn);					
					$this->writeRotLine($pdf, $this->centres[$page]['x'], $this->centres[$page]['y'], 0, $inc + $deb, $r, $r1, $lifeInt, $hli, $fontli);
					
				} elseif ($p > 0.65){
					$page = 3;
					$pdf->setPage($page);
					$this->writeRotLine($pdf, $this->centres[$page]['x'], $this->centres[$page]['y'], $deb, -$deb, $r, $r1, $text[1], $hfn, $fontfn);
					$this->writeRotLine($pdf, $this->centres[$page]['x'], $this->centres[$page]['y'], $deb, -$deb, $r, $r1, $lifeInt, $hli, $fontli);
				} else {
					$l = (int)(strlen($text[1])/2);
					$n3 = substr($text[1], 0, $l);
					$n1 = substr($text[1], $l + 1, strlen($text[1]));
					$li = explode('-', $lifeInt);
					
					$page = 3;
					$pdf->setPage($page);
					$this->writeRotLine($pdf, $this->centres[$page]['x'], $this->centres[$page]['y'], $deb, -$deb, $r, $r1, $$n3, $hfn, $fontfn);
					$this->writeRotLine($pdf, $this->centres[$page]['x'], $this->centres[$page]['y'], $deb, -$deb, $r, $r1, $li[0], $hli, $fontli);										
				}
			} elseif ( ($deb < 180) && ($ends > 180) ){// page 2 -> 4
				$p = (180 - $deb) / $inc;
				if ($p < 0.35){// go on page 1
					$page = 4;
					$pdf->setPage($page);
					$this->writeRotLine($pdf, $this->centres[$page]['x'], $this->centres[$page]['y'], 180, $inc + $deb - 180, $r, $r1, $text[1], $hfn, $fontfn);
					$this->writeRotLine($pdf, $this->centres[$page]['x'], $this->centres[$page]['y'], 180, $inc + $deb - 180, $r, $r1, $lifeInt, $hli, $fontli);
					
				} elseif ($p > 0.65){
					$page = 2;
					$pdf->setPage($page);
					$this->writeRotLine($pdf, $this->centres[$page]['x'], $this->centres[$page]['y'], $deb, 180 - $deb, $r, $r1, $text[1], $hfn, $fontfn);
					$this->writeRotLine($pdf, $this->centres[$page]['x'], $this->centres[$page]['y'], $deb, 180 - $deb, $r, $r1, $lifeInt, $hli, $fontli);
				} else {
					$l = (int)(strlen($text[1])/2);
					$n3 = substr($text[1], 0, $l);
					$n1 = substr($text[1], $l + 1, strlen($text[1]));
					$li = explode('-', $lifeInt);
					
					$page = 2;
					$pdf->setPage($page);
					$this->writeRotLine($pdf, $this->centres[$page]['x'], $this->centres[$page]['y'], $deb, -$deb, $r, $r1, $$n3, $hfn, $fontfn);
					$this->writeRotLine($pdf, $this->centres[$page]['x'], $this->centres[$page]['y'], $deb, -$deb, $r, $r1, $li[0], $hli, $fontli);										
				}
				
			} else {
			$this->writeRotLine($pdf, $xc, $yc, $deb, $inc, $r, $r1, $text[1], $hfn, $fontfn);
			$this->writeRotLine($pdf, $xc, $yc, $deb, $inc, $r, $r1, $lifeInt, $hli, $fontli);
			}				
		break;
		
		case '6':
		case '7':
			
			if ($person){
				$name = utf8_encode($tool->getMyShortName($person, $maxCharLn*4));
			} else {
				$name = utf8_encode(' ?  ? ');
			}
			
			$ends = $deb + $inc;
			if ( !( ($deb < 0) && ($ends > 0) ) && !( ($deb < 180) && ($ends > 180) )  )   {
				$this->writeRadialData($pdf, $name, $lifeInt,$r1,  $r, $deb, $inc, $xc, $yc, $maxCharLn );
			} elseif ( ($j == 6) && ($i == 5) ){ //page 1
				$page = 1;
						$pdf->setPage($page);
						$xc = $this->centres[$page]['x'];
						$yc = $this->centres[$page]['y'];
						$inc1 = $inc + $deb;
						$deb1 = 0;
						$this->writeRadialData($pdf, $name, $lifeInt,$r1,  $r, $deb1, $inc1, $xc, $yc, $maxCharLn );
			} elseif ( ($j == 6) && ( $i == 26)  ){ //page 2
				$page = 2;
						$pdf->setPage($page);
						$xc = $this->centres[$page]['x'];
						$yc = $this->centres[$page]['y'];
						$inc1 = 180 - $deb;
						$deb1 = $deb;
						$this->writeRadialData($pdf, $name, $lifeInt,$r1,  $r, $deb1, $inc1, $xc, $yc, $maxCharLn );
			} elseif ( ($j == 7) && ($i == 10) ){ //page 3
				$page = 3;
						$pdf->setPage($page);
						$xc = $this->centres[$page]['x'];
						$yc = $this->centres[$page]['y'];
						$inc1 = -$deb;
						$deb1 = $deb;
						$this->writeRadialData($pdf, $name, $lifeInt,$r1,  $r, $deb1, $inc1, $xc, $yc, $maxCharLn );
			} elseif ( ($j == 7) && ( $i == 53)  ){ //page 4
				$page = 4;
						$pdf->setPage($page);
						$xc = $this->centres[$page]['x'];
						$yc = $this->centres[$page]['y'];
						$inc1 = $inc + $deb - 180;
						$deb1 = 180;
						$this->writeRadialData($pdf, $name, $lifeInt,$r1,  $r, $deb1, $inc1, $xc, $yc, $maxCharLn );
			}

			break;
		default: //  > 7
		
			$pdf->SetFontSize($size*0.8);
			if ($person){
				$name = utf8_encode($tool->getMyShortName($person, $maxCharLn*4));
			} else {
				$name = utf8_encode(' ?  ? ');
			}
			
			$ends = $deb + $inc;
			if ( !( ($deb < 0) && ($ends > 0) ) && !( ($deb < 180) && ($ends > 180) )  )   {
				$this->writeRadialData($pdf, $name, $lifeInt,$r1,  $r, $deb, $inc, $xc, $yc, $maxCharLn );
			} elseif ( ($deb < 0) && ($ends > 0) ) { //pages 3 - 1
				$p = -$deb/$inc;
					if ($p > 0.6){ //page 3
						$page = 3;
						$pdf->setPage($page);
						$xc = $this->centres[$page]['x'];
						$yc = $this->centres[$page]['y'];
						$inc1 = -$deb;
						$deb1 = $deb;
						$this->writeRadialData($pdf, $name, $lifeInt,$r1,  $r, $deb1, $inc1, $xc, $yc, $maxCharLn );
					} elseif ($p < 0.4){ // page 1
						$page = 1;
						$pdf->setPage($page);
						$xc = $this->centres[$page]['x'];
						$yc = $this->centres[$page]['y'];
						$inc1 = $inc + $deb;
						$deb1 = 0;
						$this->writeRadialData($pdf, $name, $lifeInt,$r1,  $r, $deb1, $inc1, $xc, $yc, $maxCharLn );
					} else {
						$this->writeRadialData($pdf, $name, $lifeInt,$r1,  $r, $deb, $inc, $xc, $yc, $maxCharLn );
					}
				
			} elseif ( ($deb < 180) && ($ends > 180) )  { //pages 2 - 4
				$p = -$deb/$inc;
					if ($p > 0.6){ //page 2
						$page = 2;
						$pdf->setPage($page);
						$xc = $this->centres[$page]['x'];
						$yc = $this->centres[$page]['y'];
						$inc1 = 180 - $deb;
						$deb1 = $deb;
						$this->writeRadialData($pdf, $name, $lifeInt,$r1,  $r, $deb1, $inc1, $xc, $yc, $maxCharLn );
					} elseif ($p < 0.4){ // page 1
						$page = 4;
						$pdf->setPage($page);
						$xc = $this->centres[$page]['x'];
						$yc = $this->centres[$page]['y'];
						$inc1 = $inc + $deb - 180 ;
						$deb1 = 180;
						$this->writeRadialData($pdf, $name, $lifeInt,$r1,  $r, $deb1, $inc1, $xc, $yc, $maxCharLn );
					} else {
						$this->writeRadialData($pdf, $name, $lifeInt,$r1,  $r, $deb, $inc, $xc, $yc, $maxCharLn );
					}
			}
		break;
	}
	
	
	$deb +=  $inc;
	
	}

}
//-------------
$page = 1;
$pdf->SetPage($page);
if ($ftype == 'fan180'){
$pdf->footer = $footer;
$pdf->Footer();
}

$page = 2;
$pdf->SetPage($page);
$pdf->header = $header2;
$pdf->Header();
if ($ftype == 'fan180'){
$pdf->footer = $footer2;
$pdf->Footer();
}

if ($ftype != 'fan180'){
	$page = 3;
	$pdf->SetPage($page);
	$pdf->footer = $footer;
	$pdf->Footer();
	$page = 4;
	$pdf->SetPage($page);
	$pdf->footer = $footer2;
	$pdf->Footer();
}			
			$pdf->Output('Fan-chart' . '.pdf', 'I');
									 
	}
	
/*
	private function arcWrite(PdfPageController $pdf, $text, $cx, $cy, $r, $size, $angle){
		$r *= 0.9;
		$arc = 0.8 * 3.1415 * ($r - $size)*$angle/180;
		 $nbrLettres = strlen($text);
  $size = $r / 5;
  $pdf->SetFontSize($size);
  $len = $pdf->GetStringWidth($text) ;	
	if ($len > 0.8*$arc){
		$sizet = $size*0.8*$arc/$len;
		$pdf->SetFontSize($sizet);
		$len *= $sizet / $size;
	} 
	
	$step = $angle*0.8/($nbrLettres);
	if ($step > 10){
		 $step = 10;
	}
	if ($len < 0.6*$arc){
		$step *= 1.3;	
	}  
	  
	  $phy = (180 - ($step * $nbrLettres)) / 2;
	  
  for ($i = 0; $i < $nbrLettres; $i++){
    $angle = $phy + $i * $step*1.2;
    $angleRAD = $angle * 3.1415/180;
	
    $x = $cx - cos($angleRAD)*$r;
    $y = $cy - sin($angleRAD)*$r;
		$pdf->startTransform();
			$pdf->Rotate(90 - $angle, $x, $y);
			$pdf->Text($x, $y, $text[$i]);
		$pdf->stopTransform();
	}

	return true;	
	}
*/
	

/* import */

	private function arcWrite(PdfPageController $pdf, $text, $cx, $cy, $r, $size, $angle){
		$text = ArbreClass::delAccents($text);
		$r *= 0.9;
		$arc = 0.8 * 3.1415 * ($r - $size)*$angle/180;
		 $nbrLettres = strlen($text);
		$size = $r / 5;
		$pdf->SetFontSize($size);
		$len = $pdf->GetStringWidth($text) ;	
	
	if ($len > 0.8*$arc){
		$sizet = $size*0.8*$arc/$len;
		$pdf->SetFontSize($sizet);
		$len *= $sizet / $size;
	} 
	
	$step = $angle*0.8/($nbrLettres);
	if ($step > 10){
		 $step = 10;
	}
	if ($len < 0.6*$arc){
		$step *= 1.3;	
	}  
	  
	  $phy = (180 - ($step * $nbrLettres)) / 2;
	  
		for ($i = 0; $i < $nbrLettres; $i++){
			$angle = $phy + $i * $step*1.2;
			$angleRAD = $angle * 3.1415/180;
			
			$x = $cx - cos($angleRAD)*$r;
			$y = $cy - sin($angleRAD)*$r;
				$pdf->startTransform();
					$pdf->Rotate(90 - $angle, $x, $y);
					$pdf->Text($x, $y, $text[$i]);
				$pdf->stopTransform();
		}

	return true;	
	}


/*  ------  */	

	private function arcWrite1(PdfPageController $pdf, $text, $cx, $cy, $r, $maxChars, $inc, $id){
		//$text = utf8_decode($text);
		
		$nbrLettres = strlen($text);
		$l_arc = 3.1415*$r*$inc/180;
		$cross41 = false; $cross23 = false;
		$size = $pdf->GetFontSize();
		
		$lname = 	$pdf->GetStringWidth($text) ;
		
		if ($lname < 0.6*$l_arc){
			$size *= 2;
			$pdf->SetFontSize($size);
		}
		
		$step = $inc*0.8/($nbrLettres*1.8);
		if ($step > 10){
			$step = 10;
		}
	  
	  $step *= 1.3;
	  $phy = $this->startAngle + $id*$inc + ($inc - ($step * $nbrLettres)) / 2;
	  $endw = $phy + $step*$nbrLettres;
	  $r -= 2*$size;
	  	
	if (($phy < 0) && ($endw < 0) ) {
		$page = 3;
		$pdf->setPage($page);	
	} elseif (($phy < 0) && ($endw > 0) ) {
		$page = 3; $cross41 = true;
		$pdf->setPage($page);		
	} elseif ($phy > 180){
		$page = 4;
		$pdf->setPage($page);
		$cx = $this->centres[$page]['x'];
	} elseif (($phy > 0) && ($endw < 90) ){
		$page = 1;
		$pdf->setPage($page);
	} elseif (($phy > 90) && ($endw < 180) ) {
		$page = 2;
		$pdf->setPage($page);
	} elseif (($phy > 90) && ($endw > 180) ) { 
		$page = 2; $cross23 = true;
		$pdf->setPage($page);
	} else {
		$cross = true;
	}
	$cx = $this->centres[$page]['x'];
	$cy = $this->centres[$page]['y'];
  
	  
	for ($i = 0; $i < $nbrLettres; $i++){
		$angle = $phy + $i * $step;
		$angleRAD = $angle * 3.1415/180;
		
		if ($cross41  && ($angle > -$step) ){	
			$page = 1; 
		$pdf->setPage($page);
		$cx = $this->centres[$page]['x'];
		$cy = $this->centres[$page]['y'];
		$angle += $step;
		$angleRAD = $angle*3.1415/180;
		}
		
		if ($cross23  && ($angle > (180 - $step))){	
			$page = 4; 
		$pdf->setPage($page);
		$cx = $this->centres[$page]['x'];
		$cy = $this->centres[$page]['y'];
		$angle += $step;
		$angleRAD = $angle*3.1415/180;
		}
		
		$pdf->SetFontSize($size);
		
			$x = $cx - cos($angleRAD)*$r;
			$y = $cy - sin($angleRAD)*$r;
			$pdf->startTransform();
				$pdf->Rotate(90 - $angle, $x, $y);
				$pdf->Text($x, $y, $text[$i]);
			$pdf->stopTransform();
	}

	return true;	
	}

	/*
	*
	*	$tpos position in %r of the line < 1
	*	$fsize size in fractio of init size font
	*	char / char change pie sector
	*/
	private function writeRotLine(PdfPageController $pdf, $xc, $yc, $deb, $inc, $r, $r1, $text, $tpos = 0.5, $fsize = 0.8){
		$lifeInt = utf8_decode($text);

			$size = $pdf->getFontSizePt();
			$pdf->SetFontSize($size*$fsize);	
			$larc = ($r1 - $r*$tpos)*$inc*3.1415/180;
			
			$ltxt = $pdf->GetStringWidth($lifeInt) ;		
			$dt = ($larc - $ltxt)/2;
			
			$x = $xc - ($r1 - $r*$tpos)*cos(($deb)*3.1415/180) + 1*$dt*cos((90 - $deb - $inc/4)*3.1415/180);
			$y = $yc - ($r1 - $r*$tpos)*sin(($deb)*3.1415/180) - 1*$dt*sin((90 - $deb - $inc/4)*3.1415/180);
			
			//-----------
			$pdf->StartTransform();
				$pdf->Rotate( 90 - $deb - $inc/2,$x ,$y);
				$pdf->Text($x, $y, $lifeInt);
			$pdf->StopTransform();
	}
	

		private function deg2rad($deg){return $deg*3.14156/180;}
	/*
	*	$text array($name $fname, $lifeInt)
	*
	*/
	
	private function writeRadialData(PdfPageController $pdf, $name, $lifeInt,$r1 ,$r, $deb, $inc, $xc, $yc, $maxChars ){
		
		$alpha = 1;
		$name = utf8_decode($name);
		
		$rot = $inc/2 + $deb;
		$rotrad = $this->deg2rad($rot);
		
		// font
		$l_arc = $r*$inc*3.1415/180;

		// behind gen 7  or part of sector gen 5 - 6
		if ($inc < 3){  // 3 - 180/64
			$alpha = 0;
			$name .= ' ' . $lifeInt;
			
				if (strlen($name) > $maxChars ){
					$size = $pdf->GetFontSize()*$maxChars/strlen($name);
					$pdf->SetFontSize($size);
				}
		}
		
		$ln = 	$pdf->GetStringWidth($name) ;	
		$lfi = 	$pdf->GetStringWidth($lifeInt) ;											
				
		//centrage
		$delta = ($r - $ln)/2;
		$deltai = ($r - $lfi)/2;
		$size = $pdf->GetFontSize();	

		
			if ($rot < 90){		//page 1  - 3	
			
				// which pages?
			
				//point haut gauche->$deb	
				$x = $xc - $r1*cos($rotrad);
				$y = $yc - $r1*sin($rotrad);
				//offset  name $size
				$xn = $x + $alpha*$size*cos(3.1415/2 - $rotrad) + $delta*cos($rotrad);
				$yn = $y - $alpha*$size*sin(3.1415/2 - $rotrad) + $delta*sin($rotrad);
				
				$dli = $l_arc/4; //$size
				$xli =  $x - $alpha*$dli*1*cos(3.1415/2 - $rotrad) + $deltai*cos($rotrad);
				$yli =  $y + $alpha*$dli*1*sin(3.1415/2 - $rotrad) + $deltai*sin($rotrad);
		
				$rot =  -$rot;
				
				if ($name != ''){
				// Start Transformation
				$pdf->StartTransform();		
				$pdf->Rotate($rot, $xn, $yn);
				$pdf->Text($xn, $yn , $name);
				$pdf->StopTransform();
				}
				if ( ($alpha > 0) && ($lifeInt != '') ){
					$pdf->StartTransform();
					$pdf->Rotate( $rot, $xli, $yli);
					$pdf->Text($xli, $yli , $lifeInt);
					// Stop Transformation
					$pdf->StopTransform();
				}
			} 			
			else {   //page 2  -  4
		$x = $xc - ($r1 - $r)*cos($rotrad);
		$y = $yc - ($r1 - $r)*sin($rotrad);
				//offset  name $size
				$xn = $x + (-$alpha*$size*1.5)*cos(3.1415/2 - $rotrad) + $delta*cos(3.1415 - $rotrad);
				$yn = $y - (-$alpha*$size*1.5)*sin(3.1415/2 - $rotrad) - $delta*sin(3.1415 - $rotrad);
				
				$xli =  $x + ($alpha*$size*1.3)*1*cos(3.1415/2 - $rotrad) + $deltai*cos(3.1415 - $rotrad);
				$yli =  $y - ($alpha*$size*1.3)*1*sin(3.1415/2 - $rotrad) - $deltai*sin(3.1415 - $rotrad);
					
				$rot =  180- $rot + $inc/2;
				
			// Start Transformation
				$pdf->StartTransform();		
				$pdf->Rotate($rot, $xn, $yn);
				$pdf->Text($xn, $yn , $name);
				$pdf->StopTransform();
				if ($alpha > 0){
				$pdf->StartTransform();
				$pdf->Rotate( $rot, $xli, $yli);
				$pdf->Text($xli, $yli , $lifeInt);
				// Stop Transformation
				$pdf->StopTransform();
				}
				
				}
			
	return true;	
	}
	
	/*
	*	Init colors for sectors and text
	*
	*/
	private function initColors($colorSet){	
		$sexs = array('M', 'F', 'U');
		$this->fan_colors = array();
		foreach ($sexs as $sex){
			switch ($colorSet){
				case '1'://default			
					$this->fan_colors[$sex] = unserialize($this->getSetting('FAN_CHART_COLORS_' . $sex));
				break;
				case '2':
				if (Auth::user()->getPreference('FAN_CHART_COLORS_' . $sex ) ){
					$this->fan_colors[$sex] = unserialize(Auth::user()->getPreference('FAN_CHART_COLORS_' . $sex ));
				} else {
					$this->fan_colors[$sex] = unserialize($this->getSetting('FAN_CHART_COLORS_' . $sex));
				}
				break;
				case '3':
						//classic				
					 $TEXT_COLOR_LIGHT_M = array(0, 0, 255);
					 $FILL_COLOR_LIGHT_M = array(173, 216, 230); //lightblue
					 $TEXT_COLOR_LIGHT_F = array(0, 0, 255);
					 $FILL_COLOR_LIGHT_F = array(254, 182, 193); //lightpink
					
					 $TEXT_COLOR_LIGHT_U = array(0, 0, 0);
					 $FILL_COLOR_LIGHT_U = array(180, 180, 180);
					 
					$this->fan_colors['M'] = array( 'bgd' => $FILL_COLOR_LIGHT_M, 'txt' => $TEXT_COLOR_LIGHT_M); 
					$this->fan_colors['F'] = array( 'bgd' => $FILL_COLOR_LIGHT_F, 'txt' => $TEXT_COLOR_LIGHT_F); 
					$this->fan_colors['U'] = array( 'bgd' => $FILL_COLOR_LIGHT_U, 'txt' => $TEXT_COLOR_LIGHT_U); 
		
				break;
				case '4':
					//black/white				
					 $TEXT_COLOR_LIGHT_M = array(0, 0, 255);
					 $FILL_COLOR_LIGHT_M = array(255, 255, 255); //white
					 
					 $TEXT_COLOR_LIGHT_F = array(255, 0, 0);
					 $FILL_COLOR_LIGHT_F = array(255, 255, 255); //lightpink
					
					 $TEXT_COLOR_LIGHT_U = array(0, 0, 0);
					 $FILL_COLOR_LIGHT_U = array(255, 255, 255);
					 
					$this->fan_colors['M'] = array( 'bgd' => $FILL_COLOR_LIGHT_M, 'txt' => $TEXT_COLOR_LIGHT_M); 
					$this->fan_colors['F'] = array( 'bgd' => $FILL_COLOR_LIGHT_F, 'txt' => $TEXT_COLOR_LIGHT_F); 
					$this->fan_colors['U'] = array( 'bgd' => $FILL_COLOR_LIGHT_U, 'txt' => $TEXT_COLOR_LIGHT_U); 
		
				break;
				
			}	
		}
			
		//set background
		if (($pref = Auth::user()->getPreference('FAN_CHART_BGD' )) == null){
			$this->bgd = $this->getSetting('FAN_CHART_BGD');
		} else {
			$this->bgd = $pref;
		}	
		
	return true;	
	}
	
	/*
	*	background image 
	*	saved in images xx.png
	*/
	private function setBackgroundImage(PdfPageController $pdf, $img){		
		$pageDim = $pdf->getPageDimensions();
		$w  = $pageDim["CropBox"]["urx"] - $pageDim["CropBox"]["llx"];
		$h = $pageDim["CropBox"]["ury"] - $pageDim["CropBox"]["lly"];
		if (!file_exists($img)){
			$img = '';
		}
		// background image
		$html = '<div >
				<img src="' . $img .'" style="width:' . $w .'pt;
					height:' . $h .'pt;
				">
				</div>
		';
		$PDF_MARGIN_LEFT = $pdf->getMargins()['left'];
		$PDF_MARGIN_TOP = $pdf->getMargins()['top'];
		$PDF_MARGIN_RIGHT = $pdf->getMargins()['right'];
		$pdf->setXY($PDF_MARGIN_LEFT, $PDF_MARGIN_TOP*4);
		$pdf->writeHTML($html);
	return true;
	}

	/*
	*	color of lines between sectors
	*
	*/
	private function setLineColor(PdfPageController $pdf, $colorSet){
		if ($colorSet == 4){
			$pdf->SetColorArray('draw', $this->DRAW_COLOR_DARK);
		} else {
			$pdf->SetColorArray('draw', $this->DRAW_COLOR_LIGHT);
		}
	return true;	
	}
	
	/*
	* 	write $text on the right page  Gen = 1
	*/
	private function printTextCharbyChar(PdfPageController $pdf, $text, $x, $y, $size){
		
		$sl = (int)(strlen($text)/2);
		$x = $this->centres['1']['x'] - ($sl + 1)*$size*0.5;
		for ($i = 0; $i < strlen($text) ; $i++ ){
			if ($i <= $sl){
				$page = 1 ;
				$pdf->SetPage($page);	
				$pdf->setFontSize($size*0.5);
				$pdf->Text($x, $y, ($text[$i]));
			} else {
					$page = 2;
					$pdf->SetPage($page);
					$pdf->setFontSize($size*0.5);
					$pdf->Text($x - $this->centres['1']['x'] + $size*1.5, $y, $text[$i]);
			}			
				
				$x += $pdf->getFontSize();
		}
	return true;	
	}

	/*
	*	write on the right page for Gen = 1
	*/

		private function arcWriteGen1(PdfPageController $pdf, $text, $cx, $cy, $r, $size, $angle){
		//$text = utf8_decode($text);
		$r *= 0.9;
		$arc = 0.8 * 3.1415 * ($r - $size)*$angle/180;
		 $nbrLettres = strlen($text);
  $size = $r / 5;
  $pdf->SetFontSize($size);
  $len = $pdf->GetStringWidth($text) ;	
	if ($len > 0.8*$arc){
		$sizet = $size*0.8*$arc/$len;
		$pdf->SetFontSize($sizet);
		$len *= $sizet / $size;
	} 
	
	$step = $angle*0.8/($nbrLettres*1.8);
	if ($step > 10){
		 $step = 8;
	}
	if ($len < 0.6*$arc){
		$step *= 1.3;	
	}  
	 //************
	$pdf->SetPage(2);
	$size2 = $pdf->GetFontSize();
	  
	//***************   
	  $phy = (90 - ($step * $nbrLettres*1.8) / 2) ;
	 $angle = $phy; 
  for ($i = 0; $i < $nbrLettres; $i++){
    $angle +=  $step*1.2;
	if (($angle + $step) <= 85){
		$page = 1;
		$pdf->SetPage($page);
		$pdf->SetFontSize($size2);
	} else {
		if ( (-90 + $angle + $step)  <  8){
									$angle += 14;
									}
		$page = 2;
		$pdf->SetPage($page);
		$cx = $this->centres['2']['x'];
	}

    $angleRAD = $angle * 3.1415/180;
	
    $x = $cx - cos($angleRAD)*$r;
    $y = $cy - sin($angleRAD)*$r;
		$pdf->startTransform();
			$pdf->Rotate(90 - $angle, $x, $y);
			$pdf->Text($x, $y, $text[$i]);
		$pdf->stopTransform();
	}

	return true;	
	}

}