<?php
/**
 * webtrees: online genealogy
 * Copyright (C) 2015 webtrees development team
 * Copyright (C) 2015 JustCarmen
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *---------------------------------------------------------------------
 *   V 1.0.1 for "arbre" V-1-7-2 for Webtrees 1.7.2
*   André DUMAS August 2015
*   http://www.adumas.1s.fr/wt
*
*	V-1-1 for Arbre V-1-7-6-1  WT 1.7.6  25 JUN 2016
*	V-1-2 for Arbre V-1-7-8-1  WT 1.7.8  20 NOV 2016
*	V-1-3 for Arbre V-1-7-9-1  WT 1.7.9  15 JAN 2017
*	V-1-4 for Arbre V-1-7-9-2  WT 1.7.9  15 FEB 2017
*	V-1-5 for Arbre V-1-7-9-4  WT 1.7.9  15 MAR 2017
*		option multipages
*		Pie-sectors
 */
namespace Andre\WebtreesAddOn\Arbre\Template;

use Fisharebest\Webtrees\File;
use Fisharebest\Webtrees\Filter;
use Fisharebest\Webtrees\I18N;
use Fisharebest\Webtrees\Auth;
use Fisharebest\Webtrees\Individual;
use Andre\WebtreesAddOn\Arbre\ArbreClass;
use Andre\WebtreesAddOn\Arbre\ArbreModule;

use Andre\WebtreesAddOn\Arbre\Template\PdfPageController;
use Andre\WebtreesAddOn\Arbre\Template\myPDFTools;


use TCPDF;
use TCPDF_STATIC;

class PdfNewTreeTemplatePie_1 extends ArbreModule {
	
	const PdfTemplate_CUSTOM_VERSION  = '2-0';
	const PdfTemplate_CUSTOM_VERSION_DATE  = '2017-09-20';	
	const PdfTemplate_CUSTOM_VERSION_AUTHOR  = 'André DUMAS FRANCE';
	
	
	var $ltxtref;
	// default : half circle
	var $startAngle; // 0
	var $stopAngle ; /// 180
	
	//colors and backgrounds
	var $fan_colors;
	var $bgd;
	
		
	public function pageBody($format, $type, $colorSet , $ftype, $multipages ) {
		global $WT_TREE;
		
		$this->initColors($colorSet);
		$path_bgd = WT_MODULES_DIR . $this->getName() .'images/bgd/';
		
	 $DRAW_COLOR_LIGHT = array(255, 255, 255);
	 $DRAW_COLOR_DARK = array(90, 90, 90);
	
		switch ($ftype) {
			case 'fan180' :
				$this->startAngle = 0; // 0
				$this->stopAngle  = 180; /// 180
			break;
			case 'fan270' :
				$this->startAngle = -45; // 270°
				$this->stopAngle  = 225; 
			break;
			case 'fan360' :
				$this->startAngle = -90; // 360°
				$this->stopAngle  = 270; 
			break;
			default:
			break;
		}
		
		// on lit le tableau de valeurs
			
			$temp = WT_DATA_DIR . $this->getSetting('ARBRE_CACHE_DIR');			
				$f = DIRECTORY_SEPARATOR . 'arbre.txt' ;						
				$cache = $temp . $f;				
				//Commence à 1 TODO
				$data = unserialize(file_get_contents($cache));
				// nbre de generations
				$gen = count($data);

				////
						$person = Individual::getInstance($data[1][0]["id"], $WT_TREE);		
			$titre = I18N::translate('Fan chart ') . '  ' . $person->getAllNames()[$person->getPrimaryName()]['fullNN'] ;	
			$header = '<div style="width:100%;height:5%;">'.$WT_TREE->getTitle(). '     :     ' . $titre .'</div>';	
			$footer = '<div style="width:100%;">'
						. 'Module "arbre" for Webtrees @AD 2015-2016-2017'.'</div>';
// ---------------------------------------------------------

if (!is_array($format)){
$PDF_PAGE_FORMAT = $format;
} else {
	$PDF_PAGE_FORMAT = 'A4';
}
$PDF_PAGE_ORIENTATION = 'L'; //L pour paysage

/////////////
$pdf = new PdfPageController($PDF_PAGE_FORMAT, $PDF_PAGE_ORIENTATION);
$tool = new myPDFTools();

//-------------------------------------------------------------

// set auto page breaks
$pdf->SetAutoPageBreak(FALSE, PDF_MARGIN_BOTTOM);

//---------------------------------------------------------------
// add a page
$pdf->AddPage();  


$twidth  = $pdf->getPageWidth();
$theight = $pdf->getPageHeight(); 
/////////////
//Cercle:
//Centre $xc, $yc
// X Y axis
$window = $this->stopAngle - $this->startAngle;
	if ($window == 180){
		$xc = $twidth / 2 - $twidth*0.007;
		$yc = $theight - $theight*0.05; //bottom of the page - 10
		$r = 0.9 * $twidth/2/$gen;
	} else {
		$xc = $twidth / 2 - $twidth*0.007;
		$yc = $theight / 2 + $theight*0.01; //
		$r = 0.9 * $theight/2/$gen;
	}

///////////////
// background-image 
$img = $path_bgd . $this->bgd;
$this->setBackgroundImage($pdf, $img);

$pdf->header = $header;
$pdf->Header();

// set default monospaced font
// set font
$size = 4;
$pdf->SetFont('dejavu sans', 'BI', $size);
// set margins
$pdf->SetMargins(PDF_MARGIN_LEFT, PDF_MARGIN_TOP, PDF_MARGIN_RIGHT);
$pdf->SetTopMargin('1');

//------------Boucle

$start = $this->startAngle;
///////////////////////////////////////////////////////////////////
$maxCharLn = 14; // nbr max chars/line
//strref
$this->ltxtref = str_repeat('X', $maxCharLn);
$size = $r / $maxCharLn; // 15 chars max
$pdf->SetFont('dejavu sans', 'BI', $size);
////////////////////////////////////////////////////////////////////
if ($colorSet == 4){
		$pdf->SetColorArray('draw', $DRAW_COLOR_DARK);
} else {
	$pdf->SetColorArray('draw', $DRAW_COLOR_LIGHT);
}

	for ($j = $gen; $j > 0; $j--){ 
	$ids = pow(2, $j - 1);
	$inc = $window / $ids;
	$r1 = $j * $r;
	$deb = $this->startAngle;
	$fin = 0;

	for ($i = 0; $i < $ids  ; $i++){
		$fin = $deb + $inc -0.01;
		//******** 
				$person = Individual::getInstance($data[$j][$i]["id"], $WT_TREE);
			if ($person){
				$lifeInt = $tool->getMyLifeSpan($person);
				switch ($sex = $person->getSex()) {
				case 'M':
					$tcolor = $this->fan_colors['M']['txt'];;
					$color = $this->fan_colors['M']['bgd'];
					$pdf->SetTextColorArray($tcolor);
					
					$pdf->SetColorArray('fill',$color);
				break;	
				case 'F':
					$tcolor = $this->fan_colors['F']['txt'];;
					$color = $this->fan_colors['F']['bgd'];
					$pdf->SetTextColorArray($tcolor);
					$pdf->SetColorArray('fill',$color);
				break;	
				default:
					$tcolor = $this->fan_colors['U']['txt'];;
					$color = $this->fan_colors['U']['bgd'];
					$pdf->SetTextColorArray($tcolor);
					$pdf->SetColorArray('fill',$color);
				break;	
			}
				$text = utf8_encode($tool->getMyShortName($person, $maxCharLn));
			} else {
				$text = '? ?';
				$lifeInt = '( ? - ? )';
				$tcolor = $this->fan_colors['U']['txt'];;
					$color = $this->fan_colors['U']['bgd'];
					$pdf->SetTextColorArray($tcolor);
					$pdf->SetColorArray('fill',$color);
			}
						
		//****************
		
		$pdf->PieSector($xc, $yc, $r1, $deb, $fin, 'FD', true, 180);
		
	//font size
	/*	$size = $r*0.9/$maxCharLn;
		if ($r1*$inc*3.1415/180 < 4*$size){$size *= 0.9;}
		$pdf->SetFontSize($size);
	*/		
	switch ($j) {
		case '1':			
		$person = Individual::getInstance($data[1][0]["id"], $WT_TREE);
		$lifeInt = $tool->getMyLifeSpan($person);
		$media = $person->findHighlightedMedia();
		$link = WT_BASE_URL . 'individual.php?pid=' . $data[1][0]["id"] . '&ged=' . $WT_TREE->getName();
		
		if (($media != null) && ($media->fileExists('thumb')) ) {
			$imgType = strtoupper($media->extension());
			if ($imgType == 'PNG' || $imgType == 'JPG' || $imgType == 'JPEG'){
				$imgPath = $media->getServerFilename('thumb');
			} 
		} 
			//Title	
				switch ($sex ) {
						case 'M':
							$pdf->SetTextColor(0, 0, 255);
							$pdf->SetFillColor(176, 213, 230);
							if (($media == null) || !($media->fileExists('thumb')) ){
								$imgType = 'PNG';
								$imgData = (img_Class::getImage('male'));
							}
						break;	
						case 'F':
							$pdf->SetTextColor(255, 0, 0);
							$pdf->SetFillColor(254, 182, 193);
							if (($media == null) || !($media->fileExists('thumb'))  ){
								$imgData = (img_Class::getImage('female'));
								$imgType = 'PNG';
							}
						break;	
						default:
							$pdf->SetTextColor(0, 0, 0);
							$imgData = (img_Class::getImage('unknown'));
							break;	
				}
					
		$imgW = ($theight / 8);
		if (($media == null) || !($media->fileExists('thumb')) ){
			$html = '<img  style="width:500pt;"
				src="'.$imgData . '" />';

			$pdf->writeHTMLCell(	$w= $imgW,
										$h = $imgW,
										$x= $twidth*0.08,
										$y = $theight*0.08,
										$html,
										$border = 0,
										$ln = 0,
										$fill = false,
										$reseth = true,
										$align ='',
										$autopadding = true 
								) ;		
		} else {
		$pdf->Image($imgPath, $twidth*0.08, $theight*0.08, $imgW, '', $imgType, '', 'T', false, 300, '', false, false, 1, false, false, false);	
		}
	if ($ftype == 'fan180'){
		$tsize = $theight / 40;
		$pdf->SetFont('dejavu sans', 'B', $tsize);
		$name = $person->getAllNames()[$person->getPrimaryName()]['fullNN'] ;
		$xn = $xc - strlen($name)* $tsize / 3;
			$text = $name . '  ' . $lifeInt;
			$pdf->SetX($twidth*0.25);
			$pdf->SetY($theight*0.1);
			$pdf->setCellPadding(15);
			$pdf->setCellMargins($left= $twidth*0.20);						
				$pdf->Cell($twidth*0.65, 0, $text, 1, 1, 'C', 1,  $link, 4);
		$pdf->SetFont('dejavu sans', 'B', $size);
		$pdf->setCellMargins($left= 0);
		$pdf->setCellPadding(0);
	} else {
		$tsize = $theight / 70;
		$pdf->SetFont('dejavu sans', 'B', $tsize);
		$name = $person->getAllNames()[$person->getPrimaryName()]['fullNN'] ;
		$xn = $twidth - strlen($name)* $tsize / 3;
			$text = $name ;
			$pdf->SetX($twidth*0.8);
			$pdf->SetY($theight*0.1);
			$pdf->setCellPadding(15);
			$pdf->setCellMargins($left= $twidth*0.75);						
				$pdf->Cell($twidth*0.2, 0, $text, 1, 1, 'C', 1,  $link, 4);
				$h = $pdf->getCellHeight($tsize);
				$pdf->setCellMargins($left= $twidth*0.75, $top = 0);
				$pdf->SetY($theight*0.1 + $h);
				$pdf->Cell($twidth*0.2, 0, $lifeInt, 1, 1, 'C', 1,  $link, 4);
				
		$pdf->SetFont('dejavu sans', 'B', $size);
		$pdf->setCellMargins($left= 0);
		$pdf->setCellPadding(0);
	}	
		
		//First gen
		//$text = explode(' ', $tool->getMyTabNames($person, $maxCharLn*2));
		$text = $tool->getMyTabNames($person, $maxCharLn*2);
		$inc = $window;
		
		$size = 15;
		$size = $r / 5;
		$pdf->SetFont('dejavu sans', 'B', $size); 
		
		if ( ($sex == 'M')){ //dark				 
				 $pdf->SetColorArray('text',$this->fan_colors['M']['txt']);
		} else {
			$pdf->SetColorArray('text',$this->fan_colors['F']['txt']);
		}

			$this->arcWrite($pdf, $text[0], $xc, $yc, $r, $size, $window );
			$pdf->SetFontSize($size*0.5);		
			$lfn = $pdf->GetStringWidth($text[1]) ;	
			$lli = $pdf->GetStringWidth($lifeInt) ;
												
			$delta = (1.8*$r - $lfn)/2; 
			$deltai = (1.8*$r - $lli)/2; 
			if ( !($ftype == 'fan360') ){
			$pdf->Text($xc - $r*0.9 + $delta, $yc - 0.5 * $r, $text[1]);
			$pdf->Text($xc - $r*0.9 + $deltai, $yc - 0.3 * $r, $lifeInt);
			} else {
				$pdf->Text($xc - $r*0.9 + $delta, $yc - 0.1 * $r, $text[1]);
				$pdf->SetFontSize($size*0.5);
				$deltai = (1.7*$r - $lli*0.7)/2; 
				$pdf->Text($xc - $r*0.9 + $deltai, $yc + 0.2 * $r, $lifeInt);
			}			
			$pdf->SetFont('dejavu sans', 'B', $size); 
	 

		break;
		case '2':
			if ($person){
				$t = $tool->getMyTabNames($person, $maxCharLn);
				$text = ArbreClass::delAccents($t[1]) . '  ' . ArbreClass::delAccents($t[0]);
			} else {
				$text = ' ?  ? ';
			}
			$this->arcWrite1($pdf, $text, $xc, $yc, $r1, $maxCharLn, $inc, $i, $j);
			$h = 0.55;
			$this->writeRotLine($pdf, $xc, $yc, $deb, $inc, $r, $r1, $lifeInt, $h);			
		break;
		case   	'3':
		case   	'4':
		case 	'5':
			if ($person){
				$text = $tool->getMyTabNames($person, 100);
			} else {
				$text = array(' ? ', ' ? ');
			}
			$this->arcWrite1($pdf, ArbreClass::delAccents($text[0]), $xc, $yc, $r1, $maxCharLn, $inc, $i, $j);
			$h = 0.4;
			$fontpc = 0.9;
			$this->writeRotLine($pdf, $xc, $yc, $deb, $inc, $r, $r1, ArbreClass::delAccents($text[1]), $h, $fontpc);
			$h = 0.6;
			$fontpc = 0.6;
			$this->writeRotLine($pdf, $xc, $yc, $deb, $inc, $r, $r1, $lifeInt, $h, $fontpc);							
		break;
		case '6':
		case '7':
			if ($person){
				$name = $tool->getMyShortName2($person, $maxCharLn*3);
			} else {
				$name = ' ?   ? ';
			}
			$t = explode(' ', $name);
			$name = ArbreClass::delAccents($t[0]). ' ' . ArbreClass::delAccents($t[1]); 
			
			$this->writeRadialData($pdf, $name, $lifeInt,$r1,  $r, $deb, $inc, $xc, $yc, $maxCharLn );
		break;
		default:
			$pdf->SetFontSize($size*0.8);
			if ($person){
				$name = $tool->getMyShortName2($person, $maxCharLn*3);
			} else {
				$name = ' ?   ? ';
			}
			$t = explode(' ', $name);
			$name = ArbreClass::delAccents($t[0]). ' ' . ArbreClass::delAccents($t[1]); 
			$this->writeRadialData($pdf, $name, $lifeInt,$r1,  $r, $deb, $inc, $xc, $yc, $maxCharLn );
		break;
	}
	$deb +=  $inc;
		
	}
}
//-------------
$pdf->footer = $footer;
$pdf->Footer();
			
			$pdf->Output('Fan-chart' . '.pdf', 'I');
					 
	}
	
	
	/*
	*	write first sector->root
	*
	*/

	private function arcWrite(PdfPageController $pdf, $text, $cx, $cy, $r, $size, $angle){
		$text = ArbreClass::delAccents($text);
		$r *= 0.9;
		$arc = 0.8 * 3.1415 * ($r - $size)*$angle/180;
		 $nbrLettres = strlen($text);
		$size = $r / 5;
		$pdf->SetFontSize($size);
		$len = $pdf->GetStringWidth($text) ;	
	
	if ($len > 0.8*$arc){
		$sizet = $size*0.8*$arc/$len;
		$pdf->SetFontSize($sizet);
		$len *= $sizet / $size;
	} 
	
	$step = $angle*0.8/($nbrLettres);
	if ($step > 10){
		 $step = 10;
	}
	if ($len < 0.6*$arc){
		$step *= 1.3;	
	}  
	  
	  $phy = (180 - ($step * $nbrLettres)) / 2;
	  
		for ($i = 0; $i < $nbrLettres; $i++){
			$angle = $phy + $i * $step*1.2;
			$angleRAD = $angle * 3.1415/180;
			
			$x = $cx - cos($angleRAD)*$r;
			$y = $cy - sin($angleRAD)*$r;
				$pdf->startTransform();
					$pdf->Rotate(90 - $angle, $x, $y);
					$pdf->Text($x, $y, $text[$i]);
				$pdf->stopTransform();
		}

	return true;	
	}

	/*
	*	write  sector-> gen > 1
	*
	*/
	private function arcWrite1(PdfPageController $pdf, $text, $cx, $cy, $r, $maxChars, $inc, $id, $gen){
		
		$nbrLettres = strlen($text);
		$l_arc = 3.1415*$r*$inc/180;

		$size = $pdf->GetFontSize();
		$size = $r / 5 / (2*$gen);
		if ($gen > 2){$size = $size /1.5;}
		$lname = 	$pdf->GetStringWidth($text) ;
		
		if ($lname < 0.6*$l_arc){
			$size *= 1.8;
			$pdf->SetFontSize($size);
		}
		
		$step = $inc*0.85/($nbrLettres*1.8);
		  if ($step > 10){
			  $step = 10;
		  }
	  
	  $step *= 1.3;
	  $phy = $this->startAngle + $id*$inc + ($inc - ($step * $nbrLettres)) / 2;
	  $r -= 1*$size;
			for ($i = 0; $i < $nbrLettres; $i++){
				$angle = $phy + $i * $step;
				$angleRAD = $angle * 3.1415/180;
			
				$x = $cx - cos($angleRAD)*$r;
				$y = $cy - sin($angleRAD)*$r;
				$pdf->startTransform();
					$pdf->Rotate(90 - $angle, $x, $y);
					$pdf->Text($x, $y, $text[$i]);
				$pdf->stopTransform();
			}

	return true;	
	}

	/*
	*
	*	$tpos position in %r of the line < 1
	*	$fsize size in fractio of init size font
	*/
	private function writeRotLine(PdfPageController $pdf, $xc, $yc, $deb, $inc, $r, $r1, $text, $tpos = 0.5, $fsize = 0.8){
		$lifeInt = utf8_decode($text);
		$size = $pdf->getFontSizePt();
			$pdf->SetFontSize($size*$fsize);	
			$larc = ($r1 - $r*$tpos)*$inc*3.1415/180;
			
			$ltxt = $pdf->GetStringWidth 	( 	$lifeInt) ;		
			$dt = ($larc - $ltxt)/2;
			$x = $xc - ($r1 - $r*$tpos)*cos(($deb)*3.1415/180) + 1*$dt*cos((90 - $deb - $inc/4)*3.1415/180);
			$y = $yc - ($r1 - $r*$tpos)*sin(($deb)*3.1415/180) - 1*$dt*sin((90 - $deb - $inc/4)*3.1415/180);
			$pdf->StartTransform();
			$pdf->Rotate( 90 - $deb - $inc/2,$x ,$y);
				$pdf->Text($x, $y, $lifeInt);
			$pdf->StopTransform();
	}
	
		
	/*
	*	$text array($name $fname, $lifeInt)
	*
	*/
	
	private function writeRadialData(PdfPageController $pdf, $name, $lifeInt,$r1 ,$r, $deb, $inc, $xc, $yc, $maxChars ){
		$rot = $inc/2 + $deb;
		$rotrad = $rot*3.1415/180;
		$alpha = 1;
		$name = utf8_decode($name);
		
		// font
		$l_arc = $r*$inc*3.1415/180;
		
		// behind gen 7
		if ($inc < 3){  // 3 - 180/64
			$alpha = 0;
			$name .= ' ' . $lifeInt;
			
				if (strlen($name) > $maxChars ){
					$size = $pdf->GetFontSize()*$maxChars/strlen($name);
					$pdf->SetFontSize($size);
				}
		}
		
		$ln = 	$pdf->GetStringWidth 	( 	$name) ;	
		$lfi = 	$pdf->GetStringWidth 	( 	$lifeInt) ;											
		
		
		//centrage
		$delta = ($r - $ln)/2;
		$deltai = ($r - $lfi)/2;
		$size = $pdf->GetFontSize();		
			if ($rot < 90){			
				//point haut gauche->$deb	
				$x = $xc - $r1*cos($rotrad);
				$y = $yc - $r1*sin($rotrad);
				//offset  name $size
				$xn = $x + $alpha*$size*cos(3.1415/2 - $rotrad) + $delta*cos($rotrad);
				$yn = $y - $alpha*$size*sin(3.1415/2 - $rotrad) + $delta*sin($rotrad);
				
				$xli =  $x - $alpha*$size*1*cos(3.1415/2 - $rotrad) + $deltai*cos($rotrad);
				$yli = $y + $alpha*$size*1*sin(3.1415/2 - $rotrad) + $deltai*sin($rotrad);
		
				$rot =  -$rot;
			// Start Transformation
				$pdf->StartTransform();		
				$pdf->Rotate($rot, $xn, $yn);
				$pdf->Text($xn, $yn , $name);
				$pdf->StopTransform();
				if ($alpha > 0){
				$pdf->StartTransform();
				$pdf->Rotate( $rot, $xli, $yli);
				$pdf->Text($xli, $yli , $lifeInt);
				// Stop Transformation
				$pdf->StopTransform();
				}
			} else {
				$x = $xc - ($r1 - $r)*cos($rotrad);
				$y = $yc - ($r1 - $r)*sin($rotrad);
				//offset  name $size
				$xn = $x + (-$alpha*$size*1.5)*cos(3.1415/2 - $rotrad) + $delta*cos(3.1415 - $rotrad);
				$yn = $y - (-$alpha*$size*1.5)*sin(3.1415/2 - $rotrad) - $delta*sin(3.1415 - $rotrad);
				
				$xli =  $x + ($alpha*$size*1.3)*1*cos(3.1415/2 - $rotrad) + $deltai*cos(3.1415 - $rotrad);
				$yli =  $y - ($alpha*$size*1.3)*1*sin(3.1415/2 - $rotrad) - $deltai*sin(3.1415 - $rotrad);
					
				$rot =  180- $rot + $inc/2;
				
			// Start Transformation
				$pdf->StartTransform();		
				$pdf->Rotate($rot, $xn, $yn);
				$pdf->Text($xn, $yn , $name);
				$pdf->StopTransform();
				if ($alpha > 0){
				$pdf->StartTransform();
				$pdf->Rotate( $rot, $xli, $yli);
				$pdf->Text($xli, $yli , $lifeInt);
				// Stop Transformation
				$pdf->StopTransform();
				}				
			}			
	return true;	
	}
	
	/*
	*	Init colors for sectors and text
	*
	*/
	private function initColors($colorSet){	
		$sexs = array('M', 'F', 'U');
		$this->fan_colors = array();
		foreach ($sexs as $sex){
			switch ($colorSet){
				case '1'://default			
					$this->fan_colors[$sex] = unserialize($this->getSetting('FAN_CHART_COLORS_' . $sex));
				break;
				case '2':
				if (Auth::user()->getPreference('FAN_CHART_COLORS_' . $sex ) ){
					$this->fan_colors[$sex] = unserialize(Auth::user()->getPreference('FAN_CHART_COLORS_' . $sex ));
				} else {
					$this->fan_colors[$sex] = unserialize($this->getSetting('FAN_CHART_COLORS_' . $sex));
				}
				break;
				case '3':
						//classic				
					 $TEXT_COLOR_LIGHT_M = array(0, 0, 255);
					 $FILL_COLOR_LIGHT_M = array(173, 216, 230); //lightblue
					 $TEXT_COLOR_LIGHT_F = array(0, 0, 255);
					 $FILL_COLOR_LIGHT_F = array(254, 182, 193); //lightpink
					
					 $TEXT_COLOR_LIGHT_U = array(0, 0, 0);
					 $FILL_COLOR_LIGHT_U = array(180, 180, 180);
					 
					$this->fan_colors['M'] = array( 'bgd' => $FILL_COLOR_LIGHT_M, 'txt' => $TEXT_COLOR_LIGHT_M); 
					$this->fan_colors['F'] = array( 'bgd' => $FILL_COLOR_LIGHT_F, 'txt' => $TEXT_COLOR_LIGHT_F); 
					$this->fan_colors['U'] = array( 'bgd' => $FILL_COLOR_LIGHT_U, 'txt' => $TEXT_COLOR_LIGHT_U); 
		
				break;
				case '4':
					//black/white				
					 $TEXT_COLOR_LIGHT_M = array(0, 0, 255);
					 $FILL_COLOR_LIGHT_M = array(255, 255, 255); //white
					 
					 $TEXT_COLOR_LIGHT_F = array(255, 0, 0);
					 $FILL_COLOR_LIGHT_F = array(255, 255, 255); //lightpink
					
					 $TEXT_COLOR_LIGHT_U = array(0, 0, 0);
					 $FILL_COLOR_LIGHT_U = array(255, 255, 255);
					 
					$this->fan_colors['M'] = array( 'bgd' => $FILL_COLOR_LIGHT_M, 'txt' => $TEXT_COLOR_LIGHT_M); 
					$this->fan_colors['F'] = array( 'bgd' => $FILL_COLOR_LIGHT_F, 'txt' => $TEXT_COLOR_LIGHT_F); 
					$this->fan_colors['U'] = array( 'bgd' => $FILL_COLOR_LIGHT_U, 'txt' => $TEXT_COLOR_LIGHT_U); 
		
				break;
				
			}	
		}
			
		//set background
		if (($pref = Auth::user()->getPreference('FAN_CHART_BGD' )) == null){
			$this->bgd = $this->getSetting('FAN_CHART_BGD');
		} else {
			$this->bgd = $pref;
		}	
		
	return true;	
	}
	
	/*
	*	background image 
	*	saved in images xx.png
	*/
	private function setBackgroundImage(PdfPageController $pdf, $img){		
		$pageDim = $pdf->getPageDimensions();
		$w  = $pageDim["CropBox"]["urx"] - $pageDim["CropBox"]["llx"];
		$h = $pageDim["CropBox"]["ury"] - $pageDim["CropBox"]["lly"];
		if (!file_exists($img)){
			$img = '';
		}
		// background image
		$html = '<div >
				<img src="' . $img .'" style="width:' . $w .'pt;
					height:' . $h .'pt;
				">
				</div>
		';
		$PDF_MARGIN_LEFT = $pdf->getMargins()['left'];
		$PDF_MARGIN_TOP = $pdf->getMargins()['top'];
		$PDF_MARGIN_RIGHT = $pdf->getMargins()['right'];
		$pdf->setXY($PDF_MARGIN_LEFT, $PDF_MARGIN_TOP*4);
		$pdf->writeHTML($html);
	return true;
	}
	
}