<?php
/**
 * webtrees: online genealogy
 * Copyright (C) 2015 webtrees development team
 * Copyright (C) 2015 JustCarmen
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *---------------------------------------------------------------------
 *   V 1.0.1 for "arbre" V-1-7-2 for Webtrees 1.7.2
*   André DUMAS August 2015
*   http://www.adumas.1s.fr/wt
*
*	V-1-1 for Arbre V-1-7-6-1  WT 1.7.6  25 JUN 2016
*	V-1-2 for Arbre V-1-7-8-1  WT 1.7.8  20 NOV 2016
*	V-1-3 for Arbre V-1-7-9-1  WT 1.7.9  15 JAN 2017
*	V-1-4 for Arbre V-1-7-9-2  WT 1.7.9  15 FEB 2017
*		Tree with Rounded Rect
 */
namespace Andre\WebtreesAddOn\Arbre\Template;

use Fisharebest\Webtrees\File;
use Fisharebest\Webtrees\Filter;
use Fisharebest\Webtrees\Date;
use Fisharebest\Webtrees\Auth;
use Fisharebest\Webtrees\I18N;
use Fisharebest\Webtrees\Fact;
use Fisharebest\Webtrees\Family;
use Fisharebest\Webtrees\GedcomRecord;
use Fisharebest\Webtrees\Individual;
use Andre\WebtreesAddOn\Arbre\ArbreClass;
use Andre\WebtreesAddOn\Arbre\ArbreModule;

use Andre\WebtreesAddOn\Arbre\Template\PdfPageController;
use Andre\WebtreesAddOn\Arbre\Template\myPDFTools;
use Andre\WebtreesAddOn\Arbre\familyTools;

use TCPDF;
use TCPDF_STATIC;
use TCPDF_FONTS;

class PdfNewTreeTemplateRRectM3P_1 extends ArbreModule {
	
	const PdfTemplate_CUSTOM_VERSION  = '2-0';
	const PdfTemplate_CUSTOM_DATE  = '2017-09-15';
	const PdfTemplate_CUSTOM_AUTHOR  = 'André DUMAS FRANCE';
	
	
	var $root;
	var $fontname;
	//colors and backgrounds
	var $tree_colors;
	var $bgd_center;
	var $bgd_lat;
	

	
	public function pageBody($format, $type = 'fan') {
		global $WT_TREE;
		
		// define background Image 
		$this->initColors();
		$bgd_img = WT_MODULES_DIR . $this->getName() . '/images/bgd/bgd1.png';
		$bgd_TL_img = WT_MODULES_DIR . $this->getName() . '/images/bgd/bgd_TL.png';
		$bgd_P_img = WT_MODULES_DIR . $this->getName() . '/images/bgd/arbre_3.png';
		$bgd_L_filig_WT = WT_MODULES_DIR . $this->getName() . '/images/bgd/filigrane_WT.png';
		$bgd_filig_WT = WT_MODULES_DIR . $this->getName() . '/images/bgd/bgd_filigrane_WT.png';
		$bgd_filig_site = WT_MODULES_DIR . $this->getName() . '/images/bgd/bgd_filigrane_site.png';

		$img_c = WT_MODULES_DIR . $this->getName() . '/images/bgd/' . $this->bgd_center;
		$img_l = WT_MODULES_DIR . $this->getName() . '/images/bgd/' . $this->bgd_lat;
		
		// on lit le tableau de valeurs
			$temp = WT_DATA_DIR . ArbreModule::getSetting('ARBRE_CACHE_DIR');			
			$f = DIRECTORY_SEPARATOR . ArbreModule::getSetting('ARBRE_CACHE_FILE') ;
			$titre = I18N::translate('Personal booklet');			
			$cache = $temp . $f;
				
			//Commence à 1 TODO
			$data = unserialize(file_get_contents($cache));				
			$gen = count($data);
				
			$header = '<div style="width:100%;height:5%;">'.$WT_TREE->getTitle(). '     :     ' . $titre .'</div>';	
			$footer = '<div style="width:100%;">'
						. 'Module "arbre" for Webtrees @AD 2015-2016-2017 '.'    ' . '</div>';
						
//-----------------------------------------------------------------						
// convert TTF font to TCPDF format and store it on the fonts folder
if (file_exists(WT_MODULES_DIR .$this->getName() .'/fonts/Billy Argel Font___.ttf')){
	$this->fontname = TCPDF_FONTS::addTTFfont(WT_MODULES_DIR .$this->getName() .'/fonts/Billy Argel Font___.ttf',' TrueTypeUnicode', '', 96);
}	

// ---------------------------------------------------------
//Determine format   Ax or Letter Legal - P or L
	switch ($format) {
		case '1' : // 3 pages  -> A4/P
			$PDF_PAGE_FORMAT = 'A4';
			$PDF_PAGE_ORIENTATION = 'P';
			$PDF_PAGE_M_FORMAT = 'A4';
			$PDF_PAGE_M_ORIENTATION = 'P';
		break;
		case '3' : // 3 pages  ->A4/L + A4/P + A4/L
			$PDF_PAGE_FORMAT = 'A4';
			$PDF_PAGE_ORIENTATION = 'L';
			$PDF_PAGE_M_FORMAT = 'A4';
			$PDF_PAGE_M_ORIENTATION = 'P';
		break;
		case '2' : // 3 pages  ->A3/L + A4/P + A3/L
			$PDF_PAGE_FORMAT = 'A3';
			$PDF_PAGE_ORIENTATION = 'L';
			$PDF_PAGE_M_FORMAT = 'A4';
			$PDF_PAGE_M_ORIENTATION = 'P';
		break;
		case '4' : // 3 pages  ->Letter/P 
			$PDF_PAGE_FORMAT = 'Letter';
			$PDF_PAGE_ORIENTATION = 'P';
			$PDF_PAGE_M_FORMAT = 'A4';
			$PDF_PAGE_M_ORIENTATION = 'P';
		break;
		case '5' : // 3 pages  ->Legal/L  + Letter/P + Legal/L 
			$PDF_PAGE_FORMAT = 'Legal';
			$PDF_PAGE_ORIENTATION = 'L';
			$PDF_PAGE_M_FORMAT = 'Letter';
			$PDF_PAGE_M_ORIENTATION = 'P';
		break;
	}

$pdf = new PdfPageController($PDF_PAGE_FORMAT, $PDF_PAGE_ORIENTATION);
$tool = new myPDFTools();
// set font
$size = 4;
$pdf->SetFont('dejavu sans', 'BI', $size);
	
//-------------------------------------------------------------

// set auto page breaks
$pdf->SetAutoPageBreak(FALSE, PDF_MARGIN_BOTTOM);

//---------------------------------------------------------------
// add a page
$pdf->AddPage();  //page 1 - left
$twidth  = $pdf->getPageWidth();
$theight = $pdf->getPageHeight();;
//Bottom
$pbottom = $pdf->getPageHeight() * 0.95;//utile
$pleft = 0;
///////////////
$pdf->header = $header;
$pdf->footer = $footer;
$pdf->Header();
// X Y axis
$color = array(255, 12, 55);

//  second page
//Father's tree on page 1
//mother's tree on page 3
$pdf->addPage($PDF_PAGE_M_ORIENTATION, $PDF_PAGE_M_FORMAT);
$this->setBackgroundImage($pdf, $this->bgd_center);

//page 3
$pdf->addPage($PDF_PAGE_ORIENTATION, $PDF_PAGE_FORMAT);
$this->setBackgroundImage($pdf, $this->bgd_lat);

$pdf->SetDrawColorArray(0, 0, 255);
$pdf->SetLineWidth(1);
$x1 = 0;
$y1 = 0;
$x2 = 0;
$y2 = $theight;
$pdf->Line($x1, $y1, $x2, $y2);

//------------Boucle

$start = $twidth * 0.05;
$cheight = 0.95*$theight / (($gen + 1)*1.4);
$maxCharLn = 20; // nbr max chars/line
$deltah = 0.7*$cheight;

//default
$size = 10;
$pdf->SetFont('dejavu sans', 'B', $size);
$lpage = $twidth * 0.9; //largeur utile
$cw = 0; //largeur cellule
 //hauteur cellule

$cy = $pbottom;
$ch = $cheight * 0.98;
$xp2 = 0;$yp2 = 0;
$imgType = '';
$imgPath = '';

for ($j = 0; $j < $gen ; $j++){
	$ids = pow(2, $j);
	$cdeltaw = $lpage / $ids; //width each cell 
	$cx = $start;	
	$cy -= $ch*1.5;
	if ($j == 0) {
		$this->connectBlock0($pdf, $cx, $cy, $cdeltaw, $ch);
	}
	for ($i = 0; $i < $ids; $i++){
		$person = Individual::getInstance($data[$j + 1][$i]["id"], $WT_TREE);
		if ($person){
			$lifeInt = $tool->getMyLifeSpan($person);
			if ($j == 0){
				$names = $person->getAllNames()[$person->getPrimaryName()]['fullNN'] ;
				$this->root = $names . '  ' . $lifeInt;
				$this->mainPage($pdf, $person);
			} elseif ($j > 5){
				$names = $tool->getMyShortName2($person, $maxCharLn);
				$names .= ' ' . $lifeInt;
				$lifeInt = '';
			} else {
				$names = $tool->getMyShortName($person, $maxCharLn);
			}
			
				$media = $person->findHighlightedMedia();
				switch ($sex = $person->getSex()) {
							case 'M':
								$pdf->SetTextColor(0, 0, 255);
								if (($media == null) || !($media->fileExists('thumb')) ){
									$imgPath = '';
									$imgType = 'PNG';
								}
							break;	
							case 'F':
								$pdf->SetTextColor(0, 0, 0);
								if (($media == null) || !($media->fileExists('thumb'))  ){
									$imgPath = '';
									$imgType = 'PNG';
								}
							break;	
							default:
								$pdf->SetTextColor(0, 0, 0);
								$imgPath = '';
								$imgType = 'PNG';
							break;	
					}

		} else {
			$names = '?  ?';
			$lifeInt = '(? - ?)';
			$media = null;
			$pdf->SetTextColor(0, 0, 0);
			$imgPath = '';
			$imgType = 'PNG';
			$sex = 'U';
			if ($j > 5){
				$names = '? ? ' . $lifeInt;
				$lifeInt = '';
			}
		}
			
			if (($media != null) && ($media->fileExists('thumb')) ) {
				$imgType = strtoupper($media->extension());
				if ($imgType == 'PNG' || $imgType == 'JPG' || $imgType == 'JPEG'){
					$imgPath = $media->getServerFilename('thumb');
				} 
			} 
							
					$cw = $lpage / ($ids);
				if (($i < $ids / 2 ) && ($j > 0)) {
					$pdf->SetPage(1);	
					////									
					if ($i == 0)  {
						$cx = $start +  $cw / 2;
					} else {
						$cx += $cw;
					}
					if ($j < $gen -1){
						$this->connectBlocks($pdf, $cx, $cy, $cw, $ch);
					}
				////
				} else {
					$pdf->SetPage(3);
					
					if (($i - $ids/2) == 0)  {
						$cx = $start + $cw / 2;
					} else {
						$cx += $cw;
					}
					if (($j < $gen -1) && ($j != 0)){
						$this->connectBlocks($pdf, $cx, $cy, $cw, $ch);
					}	
				}
			if (($j) != 0)  {
				$text = $names ;	
				$this->cellId($pdf, $cx, $cy, $ch, $cw, $names, $lifeInt, $imgPath , $imgType, $sex, $j + 1 );
			}
			$cx += $cw;
	}
}
	
//-------------
$pdf->SetPage(1);
$pdf->Footer();
	$header_2 = '<div style="width:100%;height:5%;">'.' '. '         ' . 				$this->root .'</div>';		
	$footer_2 = '<div style="width:100%;border-top:0px solid black;">' .
						'<table><tr><td>' .
					 	' Booklet of ' . $this->root . 
						'</td><td style="width:50%;"></td><td> <span > Date:  ' . Date("Y-m-d") . '</span>' .
						'<span>   @AD 2017</span>' .
						'</td></tr></table></div>';
$pdf->SetPage(3);
$pdf->footer = $footer_2;
$pdf->Footer();
$pdf->header = $header_2;
$pdf->Header();

			$pdf->Output('Tree_MP' . '.pdf', 'I');
									 
	}
	
	/*
	* conect -> 3 lines
	* Vertical above and under each one, from middle, height = 0.25*ch
	*/				
	private function connectBlocks(PdfPageController $pdf, $cx, $cy, $cw, $ch){
		$pdf->SetDrawColor(241, 81, 255);
		$pdf->SetLineWidth(2);
		$x1 = $cx + $cw/2;
		$x2 = $x1;
		$y1 = $cy;
		$y2 = $y1 - 0.25*$ch;
		
			$pdf->Line($x1, $y1, $x2, $y2);//up
												
			$pdf->Line($x1 - $cw/2, $y2, $x1 + $cw/2, $y2); //horizontal sup
			$pdf->Line($x1 - $cw/2, $y2, $x1 - $cw/2, $y2 - 0.25*$ch);
			$pdf->Line($x1 + $cw/2, $y2, $x1 + $cw/2, $y2 - 0.25*$ch);
		$x1 += $cw*2;	
		$pdf->SetLineWidth(1);
	}
	
	private function connectBlock0(PdfPageController $pdf, $cx, $cy, $cw, $ch){
		$pdf->SetDrawColor(241, 81, 255);
		$pdf->SetLineWidth(3);
		$pdf->SetPage(1);
		$x1 = $cx + $cw/2;
		$x2 = $x1;
		$y1 = $cy - 0.5*$ch;
		$y2 = $y1 + $ch;
		
			$pdf->Line($x1, $y1, $x2, $y2);//down									
			$pdf->Line($x1 , $y2, $x1 + $cw/2, $y2); //horizontal right
			
			$pdf->SetPage(3);
		$x1 = 0;
		$x2 = $cw/2;
		$y1 = $cy - 0.5*$ch;
		$y2 = $y1 + $ch;
		
			$pdf->Line($x2, $y1, $x2, $y2);//down									
			$pdf->Line($x1 , $y2, $x1 + $cw/2, $y2); //horizontal right
			
		//$x1 += $cw*2;
		$pdf->SetPage(1);
		$pdf->SetDrawColor(255, 255, 255);
		$pdf->SetLineWidth(1);
	}
	
	
	/*
	*	background image 
	*	saved in images xx.png
	*/
	private function setBackgroundImage(PdfPageController $pdf, $img){		
		
		$w  = $pdf->getPageWidth();
		$h = $pdf->getPageHeight();
		if (!file_exists($img)){
			//$img = '';
			return;
			
		}
		// background image
		$html = '<div >
				<img src="' . $img .'" style="width:' . $w .'pt;
					height:' . $h .'pt;
				">
				</div>
		';
		$PDF_MARGIN_LEFT = $pdf->getMargins()['left'];
		$PDF_MARGIN_TOP = $pdf->getMargins()['top'];
		$PDF_MARGIN_RIGHT = $pdf->getMargins()['right'];
		$pdf->setXY($PDF_MARGIN_LEFT, $PDF_MARGIN_TOP*4);
		$pdf->writeHTML($html);
	return true;
	}

	private function cellId(PdfPageController $pdf, $cx, $cy, $ch, $cw, $name, $lifeInt, $imgPath = '', $imgType = '', $sex, $gen ){
		
		// Rounded rectangle
					
		//image
		$imgH = $ch * 0.8 /$gen;
		$imgx = $cx +  $ch * 0.1;
		$imgy = $cy  + ($ch * 0.1) ;

		$imgData = '';
		switch ($sex) {
			case 'M':
				$tcolor = $this->tree_colors['M']['txt'];;
				$color = $this->tree_colors['M']['bgd'];
				if ($imgType == 'PNG'){
					$imgData = (img_Class::getImage('male'));
				}
			break;
			case 'F':
				$tcolor = $this->tree_colors['F']['txt'];
				$color = $this->tree_colors['F']['bgd'];
				if ($imgType == 'PNG'){
					$imgData = (img_Class::getImage('female'));
				}
			break;
			default:
				$tcolor = $this->tree_colors['U']['txt'];;
				$color = $this->tree_colors['U']['bgd'];
				if ($imgType == 'PNG'){
					$imgData = (img_Class::getImage('unknown'));
				}	
			break;	
		}
		if ($gen <5) {$r = 3.5;} else {$r = 1;} //radius RRect 
		$pdf->SetLineStyle(array('width' => 0.2, 'cap' => 'butt', 'join' => 'miter', 'dash' => 0, 'color' => array(0, 0, 0)));
		$pdf->RoundedRect($cx, $cy , $cw, $ch, $r, '1111', 'DF', '' , $color);
		$pdf->setFontSize($cw / 10);
		if ($gen <= 4){
			if (($imgPath != '') ){
				$pdf->Image($imgPath, $imgx, $imgy, '', $imgH, $imgType, '', 'T', false, 300, '', false, false, 1, false, false, false);
			} else {
				if ($imgData != '') {
					$html = '<img  style="width:500pt;"
					src="'.$imgData . '" />';
				} else {
						$html = '';					
				}
			
				$pdf->writeHTMLCell(	$w= $imgH,
										$h = $imgH,
										$x= $imgx,
										$y = $imgy,
										$html,
										$border = 0,
										$ln = 0,
										$fill = false,
										$reseth = true,
										$align ='',
										$autopadding = true 
								) ;
			}
		}
		
		
		switch ($gen) {
			case '1':
				//$size = $cw/40;
				$size = $ch / 5;
				$pdf->setFontSize($size);
				$ny   = $cy + $ch/2 - $size;
				$liy   = $cy + $ch/2 + 0.1*$size;
				$cnx = $cx +  $cw/2;
				$lname = $lfi = 	$pdf->GetStringWidth 	( 	$name) ;	
				$lfi = 	$pdf->GetStringWidth 	( 	$lifeInt) ;									
				$nx    = $cnx - $lname/2;
				$lix   = $cnx - $lfi / 2;	
				$pdf->SetTextColorArray($tcolor);				
				$pdf->Text($nx, $ny, $name);
				$pdf->setFontSize($size*0.8);	
				$pdf->Text($lix, $liy, $lifeInt);
			break;
			case '2':
			case '3':
				$size = $cw / 25;
				$pdf->setFontSize($size);
				$ny   = $cy + $ch/2 - $size;
				$liy   = $cy + $ch/2 + 0.5*$size;
				$cnx = $cx +  $cw/2;
				$lname = $lfi = 	$pdf->GetStringWidth 	( 	$name) ;	
				$lfi = 	$pdf->GetStringWidth 	( 	$lifeInt) ;	
				$nx    = $cnx - $lname/2;
				$lix   = $cnx - $lfi / 4;	
				$pdf->SetTextColorArray($tcolor);				
				$pdf->Text($nx, $ny, $name);
				$pdf->setFontSize($size*0.8);		
				$pdf->Text($lix, $liy, $lifeInt);
				
			break;
			case '4':
				$size = $cw / 20;
				$pdf->setFontSize($size);
				$ny   = $cy + $ch/2 - $size;
				$liy   = $cy + $ch/2 + $size;
				$cnx = $cx +  $cw/2 ;
				$lname = $lfi = 	$pdf->GetStringWidth 	( 	$name,
												$fontname = '',
												$fontstyle = '',
												$fontsize = 0,
												$getarray = false 
												) ;	
				$lfi = 	$pdf->GetStringWidth 	( 	$lifeInt,
												$fontname = '',
												$fontstyle = '',
												$fontsize = 0,
												$getarray = false 
												) ;	
				$nx    = $cnx - $lname / 2;
				$lix   = $cnx - $lfi / 2;			
				$pdf->SetTextColorArray($tcolor);
				$pdf->Text($nx, $ny, $name);
				$pdf->setFontSize($size*0.8);	
				$pdf->Text($lix, $liy, $lifeInt);
			break;
			case '5':
			case '6':
				$size = $ch / 15;
				$pdf->setFontSize($size);
				$lname = $lfi = 	$pdf->GetStringWidth 	( 	$name) ;	
				$lfi = 	$pdf->GetStringWidth 	( 	$lifeInt) ;	
				
				$ny   = $cy + $ch/2 - $lname / 2; 
				$liy   = $cy + $ch/2 - $lfi / 2; 
				$cnx = $cx +  $cw/2;
				$nx    = $cnx + $size*1;
				$lix   = $cnx - 0.3*$size;
				$pdf->startTransform();
				$pdf->Rotate(-90, $nx, $ny);
				$pdf->SetTextColorArray($tcolor);
				$pdf->Text($nx  , $ny , $name);
				$pdf->stopTransform();
				$pdf->startTransform();
				$pdf->Rotate(-90, $lix, $liy);
					$pdf->setFontSize($size*0.8);		
				$pdf->Text($lix , $liy , $lifeInt);
				$pdf->stopTransform();
			break;
			default:  // gen > 6 only one line				
		$size = $ch / 15;
				$pdf->setFontSize($size);
				$lname = $lfi = 	$pdf->GetStringWidth 	( 	$name) ;	
				
				if ($lname > $ch*0.9){
					$size = $pdf->GetFontSize()*$ch*0.8/$lname;
					$pdf->SetFontSize($size);
				}
				if ($size > 0.8*$cw){
					$size = 0.6*$cw;
					$pdf->SetFontSize($size);
				}
				$lname = 	$pdf->GetStringWidth 	( 	$name) ;
				$ny   = $cy + $ch/2 - $lname / 2; 
				
				$cnx = $cx +  $cw*1.57/2;
				$nx    = $cnx ;
				
				$pdf->SetTextColorArray($tcolor);
				$pdf->startTransform();
				$pdf->Rotate(-90, $nx, $ny);
				$pdf->Text($nx  , $ny , $name);
				$pdf->stopTransform();
				
			break;	
		}
		
		
		}

	private function mainPage(PdfPageController $pdf, $person){
		global $WT_TREE;
		
		$tool = new myPDFTools();
		$ftool = new familyTools();
		
		$lightblue = array(173, 216, 230);
			$lightpink = array(254, 182, 193);
			$lightgrey = array(211, 211, 211);
		
		$pdf->SetPage(2);
		
		$sex = $person->getSex();
		$names = $person->getAllNames()[$person->getPrimaryName()]['fullNN'] ;
				$this->root = $names . '  ' . $tool->getMyLifeSpan($person);
				$imgPath = '';
		$media = $person->findHighlightedMedia();
		if (($media != null) && ($media->fileExists('thumb')) ) {
				$imgType = strtoupper($media->extension());
				if ($imgType == 'PNG' || $imgType == 'JPG' || $imgType == 'JPEG'){
					$imgPath = $media->getServerFilename('thumb');
				} 
		} else {
			$imgType = 'PNG';
		}
		$imgData = '';
		switch ($person->getSex() ) {
			case 'M':
				//$color = $lightblue;
				$color = $this->tree_colors['M']['bgd'];
				if ($imgType == 'PNG'){
					$imgData = (img_Class::getImage('male'));
				}
			break;
			case 'F':
				//$color = $lightpink;
				$color = $this->tree_colors['F']['bgd'];
				if ($imgType == 'PNG'){
					$imgData = (img_Class::getImage('female'));
				}
			break;
			default:
				//$color = $lightgrey;
				$color = $this->tree_colors['U']['bgd'];
				if ($imgType == 'PNG'){
					$imgData = (img_Class::getImage('unknown'));
				}	
			break;	
		}
		$pdf->SetPage(2);
		$l = $pdf->getPageWidth();
		$imgH = $l / 4;
		$delta = $l	/ 2 - $imgH/2;
		$imgx = $delta;
		$imgy = $pdf->getPageHeight() *0.1;
		
		if (($imgPath != '') ){
				$pdf->Image($imgPath, $imgx, $imgy, '', $imgH, $imgType, '', 'T', false, 300, '', false, false, 1, false, false, false);
		} elseif ($imgData != '') {
				$html = '<img  style="width:500pt;"
						src="'.$imgData . '" />';
				$pdf->writeHTMLCell(	$w= $imgH,
										$h = $imgH,
										$x= $imgx,
										$y = $imgy,
										$html,
										$border = 0,
										$ln = 0,
										$fill = false,
										$reseth = true,
										$align ='',
										$autopadding = true 
								) ;		
		}
		
	
		//Details
		
		
		// name
		
		$cx = 0.1*$l;
		$ch = $pdf->getPageHeight() / 8;
		$cy = $pdf->getPageHeight() - $ch*1.5;		
		$r = 4;
		$cw = $l* 0.8;
		$lenMax = 5*$cw/$ch;
		$name = $tool->getMyShortName($person, $lenMax);
		
		$size = $ch / 2.5;
		
		
		$pdf->SetFont($this->fontname, '', 14, '', false);
		$pdf->SetFontSize($size);
		$pdf->RoundedRect($cx, $cy , $cw, $ch, $r, '1111', 'DF', '' , $color);
		// use the font ttf
		$ln = $pdf->getStringWidth($name);

		$pdf->Text($cx + $cw/2 -$ln/2, $cy + $ch/2 - $size, $name);
				
		//return to original font	
		$pdf->SetFont('dejavu sans', 'BI', $size);	
		$pdf->setFontSize($size / 3);
		$lifeInt = $tool->getMyLifeSpan($person);
		$lfi = $pdf->getStringWidth($lifeInt);
		$pdf->Text($cx + $cw/2 -$lfi/2, $cy + $ch/2 + $size*0.3, $lifeInt);
		
		//Details
		$pdf->SetFont('dejavu sans', 'I', $size/3);
		$tx = $pdf->getPageWidth() / 5;
		$ty = $imgH + $pdf->getPageHeight() / 7;
		$interligne = $size*0.5;
		$birth = $names . ' ' ;
		
		if ($person->getBirthDate()->isOK() ){
			$birth .= I18N::translate('was born on ') . strip_tags($person->getBirthDate()->display());
		} else {
			$birth .= I18N::translate('was born on') . ' ' . I18N::translate('an unknown date');
		}	
		$pbirth =   I18N::translate(' at ') . $person->getBirthPlace();
		$pdf->Text($tx, $ty, $birth);
		$ty += $interligne;
		$pdf->Text($tx, $ty , $pbirth);
		if ($sex == 'M'){
			if ($person->getDeathDate()->isOK() ){
				$death  = I18N::translate('He died on') . ' ' . strip_tags($person->getDeathDate()->display());
			} else {
				$death  = I18N::translate('He died on ') . ' ' . I18N::translate('an unknown date');
			}
		} else {
			if ($person->getDeathDate()->isOK() ){
				$death  = I18N::translate('She died on') . ' ' . strip_tags($person->getDeathDate()->display());
			} else {
				$death  = I18N::translate('She died on ') . ' ' . I18N::translate('an unknown date');
			}	
		}
		$pdeath = I18N::translate(' at ') . $person->getDeathPlace();
		$ty += $interligne;
		$pdf->Text($tx, $ty, $death);
		$ty += $interligne;
		$pdf->Text($tx, $ty, $pdeath);
		//Occupation
		
		$ty += $interligne;
		$jobs = $person->getFacts('OCCU'); 
		if (!empty($jobs) ){
			if ($sex == 'M'){
				$text = I18N::translate('He worked as') . ' ';
			} else {
				$text = I18N::translate('She worked as') . ' ';
			}
			$text1 = '';
			$text2 = '';
			$text = '';
			foreach ($jobs as $job){
				$text .= $job->getValue();
				$text1  = I18N::translate(' at ') . ' ' . $job->getPlace()->getGedcomName() . ' ';
				$text2 = '';
				if (count($jobs) > 1) {
					$text2 = ' '. I18N::ttranslate('puis') . ' ';
				} 
			}
		
		
		$ty += $interligne;		
		$pdf->Text($tx, $ty, $text);
		$ty += $interligne;		
		$pdf->Text($tx, $ty, $text1);
		$ty += $interligne;		
		$pdf->Text($tx, $ty, $text2);
		}
		// Marriage & children
		$numberOfSpouses = $ftool->gotMarried($person);
		if ( $numberOfSpouses == 0){
			if ($sex == 'M'){
				$text = I18N::translate('He was never married');
			} else {				 
					$text = I18N::translate('She was never married');
			}			
		} else {
			if ($sex == 'M'){
				$text = I18N::translate('He was married ' ) . $numberOfSpouses . I18N::translate(' times');
			} else {
				$text = I18N::translate('She was married ' ) . $numberOfSpouses . I18N::translate(' times');
			}	
 		}
		
		$ty += $interligne;
		$ty += $interligne;
		$pdf->Text($tx, $ty, $text);
		
		$nkids = $person->getNumberOfChildren();
		$ctext = $nkids;
		switch($nkids ){
			case '0':
				$text = I18N::translate('and got no'). I18N::translate('child.');
			break;
			case '1':
				$text = I18N::translate('and got ') . $nkids . ' ' . I18N::translate('child.');
			break;
			default:
			$text = I18N::translate('and got ') . $nkids . ' ' .  I18N::translate('children.');
			break;
		}
		
		$ty += $interligne;		
		$pdf->Text($tx, $ty, $text);
			
		
	}
	
	private function familiesPage(){
		// families and children
		
	}

	/*
	*	Init colors for sectors and text
	*
	*/
	private function initColors(){	
			$sexs = array('M', 'F', 'U');
			$this->tree_colors = array();
			foreach ($sexs as $sex){	
				$this->tree_colors[$sex] = unserialize($this->getSetting('TREE_COLORS_' . $sex));
			}
				
			$this->bgd_lat = $this->getSetting('BOOKLET_BGD_LAT');
			$this->bgd_center = $this->getSetting('BOOKLET_BGD_CENTER');
						
	return true;	
	}
	
}