<?php
/**
 * webtrees: online genealogy
 * Copyright (C) 2015 webtrees development team
 * Copyright (C) 2015 JustCarmen
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *---------------------------------------------------------------------
 *   V 1.0.1 for "arbre" V-1-7-2 for Webtrees 1.7.2
*   André DUMAS August 2015
*   http://www.adumas.1s.fr/wt
*
*	V-1-1 for Arbre V-1-7-6-1  WT 1.7.6  25 JUN 2016
*	V-1-2 for Arbre V-1-7-8-1  WT 1.7.8  20 NOV 2016
*	V-1-3 for Arbre V-1-7-9-1  WT 1.7.9  15 JAN 2017
*	V-1-4 for Arbre V-1-7-9-2  WT 1.7.9  15 FEB 2017
*	V-1-5 for Arbre V-1-7-9-9b  WT 1.7.9  08 APR 2018
*		Tree with Rounded Rect
 */
namespace Andre\WebtreesAddOn\Arbre\Template;

use Fisharebest\Webtrees\File;
use Fisharebest\Webtrees\Auth;
use Fisharebest\Webtrees\Filter;
use Fisharebest\Webtrees\I18N;
use Fisharebest\Webtrees\Individual;
use Andre\WebtreesAddOn\Arbre\ArbreClass;
use Andre\WebtreesAddOn\Arbre\ArbreModule;

use Andre\WebtreesAddOn\Arbre\Template\PdfPageController;
use Andre\WebtreesAddOn\Arbre\Template\myPDFTools;

use TCPDF;
use TCPDF_STATIC;

class PdfNewTreeTemplateRRect_1 extends ArbreModule {
	
	const PdfTemplate_CUSTOM_VERSION  = '1-4 2017-02-15 AD';
	
	
	var $tree_colors;
	var $bgd;
		
	public function pageBody($format, $colorSet, $type = 'fan') {
		global $WT_TREE;
		
		$tool = new myPDFTools();
		
		$this->initColors($colorSet);
		// on lit le tableau de valeurs
			$temp = WT_DATA_DIR . $this->getSetting('ARBRE_CACHE_DIR');			
			$f = DIRECTORY_SEPARATOR . 'arbre.txt' ;						
			$cache = $temp . $f;				
			//Commence à 1 TODO
			$data = unserialize(file_get_contents($cache));
			
			$person = Individual::getInstance($data[1][0]["id"], $WT_TREE);		
			$titre = I18N::translate('Tree chart ') . '  ' . $person->getAllNames()[$person->getPrimaryName()]['fullNN']  ;
			
		$pdf = new PdfPageController($format, 'L');	

		$pdf->header = $WT_TREE->getTitle() . ':   ' . $titre;
		// set auto page breaks
		$pdf->SetAutoPageBreak(FALSE, PDF_MARGIN_BOTTOM);

		//---------------------------------------------------------------
		// add a page
		$pdf->AddPage();  

		$pdf->Header();
		// X Y axis
		$color = array(255, 12, 55);

//------------Boucle
$gen = count($data);
$start = $pdf->getPageWidth() * 0.05;

$deltah = 4;
$cheight = ($pdf->getPageHeight() - $deltah*($gen -1)) / ($gen + 1);
$maxCharLn = 20; // nbr max chars/line

//default
$size = 10;
$pdf->SetFont('dejavu sans', 'BI', $size);
$chart_width = $pdf->getPageWidth() * 0.9;
$separator_cells = 4;

for ($j = 0; $j < $gen ; $j++){
	$ids = pow(2, $j);
	$cdeltaw = ($chart_width  - $separator_cells*($ids-1))/ $ids; //width each cell 
	
	for ($i = 0; $i < $ids; $i++){
		$person = Individual::getInstance($data[$j + 1][$i]["id"], $WT_TREE);
		if ($person){
			$lifeInt = $tool->getMyLifeSpan($person);
			if ($j == 0){
				$names = $person->getAllNames()[$person->getPrimaryName()]['fullNN'] ;
			} elseif ($j > 5){
				$names = $tool->getMyShortName2($person, $maxCharLn);
				$names .= ' ' . $lifeInt;
				$lifeInt = '';
			} else {
				$names = $tool->getMyShortName($person, $maxCharLn);
			}
			
			$media = $person->findHighlightedMedia();
						switch ($sex = $person->getSex()) {
							case 'M':
								$pdf->SetTextColorArray($this->tree_colors['M']['txt']);
								if (($media == null) || !($media->fileExists('thumb')) ){
									$imgPath = '';
									$imgType = 'PNG';
								}
							break;	
							case 'F':
								$pdf->SetTextColorArray($this->tree_colors['F']['txt']);
								if (($media == null) || !($media->fileExists('thumb'))  ){
									$imgPath = '';
									$imgType = 'PNG';
								}
							break;	
							default:
								$pdf->SetTextColorArray($this->tree_colors['U']['txt']);
								$imgPath = '';
								$imgType = 'PNG';
							break;	
					}

		} else {
			$names = '?  ?';
			$lifeInt = '(? - ?)';
			$media = null;
			$pdf->SetTextColorArray($this->tree_colors['U']['txt']);
			$imgPath = '';
			$imgType = 'PNG';
			$sex = 'U';
			if ($j > 5){
				$names = '? ? ' . $lifeInt;
				$lifeInt = '';
			}
		}

			
		if (($media != null) && ($media->fileExists('thumb')) ) {
				$imgType = strtoupper($media->extension());
				if ($imgType == 'PNG' || $imgType == 'JPG' || $imgType == 'JPEG'){
					$imgPath = $media->getServerFilename('thumb');
				} 
			} 
							
				$cw = $cdeltaw;				
			
				if ($i == 0){
					$cx = $start  ;
				} else {
					$cx = $start + $i * ($cdeltaw  + $separator_cells);
				}
				$ch = $cheight ;
				$dec = 0;
				if ($j == 0){
					$cy =  $pdf->pbottom - $dec  - $ch ;//- $deltah;	
				} else {
					$cy = $pdf->pbottom -$dec - $ch*($j + 1) - $deltah*$j;
				}				
				$text = $names ;	
		
			$this->cellId($pdf, $cx, $cy, $ch, $cw, $names, $lifeInt, $imgPath , $imgType, $sex, $j + 1 );
	}
}
	
//-------------
$pdf->Footer();
			
			$pdf->Output('Fan-chart' . '.pdf', 'I');
				
			// remove the temporary files
			//File::delete($cache);					 
	}
	
	
	
	private function arcWrite($text, $r, $size, $angle){
		$arc = 0.8 * 2 * 3.1415 * ($r - $size);
		
	}
	
	/* len($text) < $max see MyShortName
	*  add spaces -> center text 
	*	param 	$max: max chars /line ->  fix size 
	*			$r   rayon
	*	return text with 15 chars centered
	*/

	/*	
	*    rect cells 
	*
	*/
	private function cellId(PdfPageController $pdf, $cx, $cy, $ch, $cw, $name, $lifeInt, $imgPath = '', $imgType = '', $sex, $gen ){
		
		// Rounded rectangle		
			$lightblue = array(173, 216, 230);
			$lightpink = array(254, 182, 193);
			$lightgrey = array(211, 211, 211);
		//image
		$imgH = $ch * 0.8 /$gen;
		$imgx = $cx +  $ch * 0.1;
		$imgy = $cy  + ($ch * 0.1) ;

		switch ($sex) {
			case 'M':
				$color = $this->tree_colors['M']['bgd'];
				$tcolor = $this->tree_colors['M']['txt'];
				$imgData = (img_Class::getImage('male'));
			break;
			case 'F':
				$color = $this->tree_colors['F']['bgd'];
				$tcolor = $this->tree_colors['F']['txt'];
				$imgData = (img_Class::getImage('female'));
			break;
			default:
				$color = $this->tree_colors['U']['bgd'];
				$tcolor = $this->tree_colors['U']['txt'];
				$imgData = (img_Class::getImage('unknown'));
			break;
			
		}
		
		$pdf->SetLineStyle(array('width' => 0.2, 'cap' => 'butt', 'join' => 'miter', 'dash' => 0, 'color' => array(0, 0, 0)));
		if ($gen < 7){
			$pdf->RoundedRect($cx, $cy , $cw, $ch, 1.50, '1111', 'DF', '' , $color);
		} else {
			$pdf->RoundedRect($cx, $cy , $cw, $ch, 1.50, '0000', 'DF', '' , $color);
		}
		$pdf->setFontSize($cw / 10);
		$pdf->SetTextColorArray($tcolor);
		if ($gen <= 4){
			if ($imgPath != ''){
				$pdf->Image($imgPath, $imgx, $imgy, '', $imgH, $imgType, '', 'T', false, 300, '', false, false, 1, false, false, false);
			} else {
				$html = '<img  style="width:500pt;"
				src="'.$imgData . '" />';

			$pdf->writeHTMLCell(	$w= $imgH,
									$h = $imgH,
									$x= $imgx,
									$y = $imgy,
									$html,
									$border = 0,
									$ln = 0,
									$fill = false,
									$reseth = true,
									$align ='',
									$autopadding = true 
								) ;	
			}
		}
		switch ($gen) {
			case '1':
				$size = $cw/40;
				$pdf->setFontSize($size);
				$ny   = $cy + $ch/2 - $size;
				$liy   = $cy + $ch/2 + 0.1*$size;
				$cnx = $cx +  $cw/2;
				$lname = $lfi = 	$pdf->GetStringWidth 	( 	$name) ;	
				$lfi = 	$pdf->GetStringWidth 	( 	$lifeInt) ;									
				$nx    = $cnx - $lname/2;
				$lix   = $cnx - $lfi / 4;					
				$pdf->Text($nx, $ny, $name);
				$pdf->setFontSize($size*0.8);		
				$pdf->Text($lix, $liy, $lifeInt);
			break;
			case '2':
			case '3':
				$size = $cw / 25;
				$pdf->setFontSize($size);
				$ny   = $cy + $ch/2 - $size;
				$liy   = $cy + $ch/2 + 0.5*$size;
				$cnx = $cx +  $cw/2;
				$lname = $lfi = 	$pdf->GetStringWidth 	( 	$name,
												$fontname = '',
												$fontstyle = '',
												$fontsize = 0,
												$getarray = false 
												) ;	
				$lfi = 	$pdf->GetStringWidth 	( 	$lifeInt,
												$fontname = '',
												$fontstyle = '',
												$fontsize = 0,
												$getarray = false 
												) ;	
				$nx    = $cnx - $lname/2;
				$lix   = $cnx - $lfi / 4;					
				$pdf->Text($nx, $ny, $name);
				$pdf->setFontSize($size*0.8);		
				$pdf->Text($lix, $liy, $lifeInt);
			break;
			case '4':
				$size = $cw / 20;
				$pdf->setFontSize($size);
				$ny   = $cy + $ch/2 - $size;
				$liy   = $cy + $ch/2 + $size;
				$cnx = $cx +  $cw/2 ;
				$lname = $lfi = 	$pdf->GetStringWidth 	( 	$name,
												$fontname = '',
												$fontstyle = '',
												$fontsize = 0,
												$getarray = false 
												) ;	
				$lfi = 	$pdf->GetStringWidth 	( 	$lifeInt,
												$fontname = '',
												$fontstyle = '',
												$fontsize = 0,
												$getarray = false 
												) ;	
				$nx    = $cnx - $lname / 2;
				$lix   = $cnx - $lfi / 2;					
				$pdf->Text($nx, $ny, $name);
				$pdf->setFontSize($size*0.8);		
				$pdf->Text($lix, $liy, $lifeInt);
			break;
			case '5':  //2 lines and cell rotated
			case '6':
				$size = $ch / 15;
				$pdf->setFontSize($size);
				$lname = $lfi = 	$pdf->GetStringWidth 	( 	$name) ;	
				$lfi = 	$pdf->GetStringWidth 	( 	$lifeInt) ;	
				
				$ny   = $cy + $ch/2 - $lname / 2; 
				$liy   = $cy + $ch/2 - $lfi / 2; 
				$cnx = $cx +  $cw/2;
				$nx    = $cnx + $cw/4;
				$lix   = $cnx - $cw/8;
				$pdf->startTransform();
				$pdf->Rotate(-90, $nx, $ny);
				$pdf->Text($nx  , $ny , $name);
				$pdf->stopTransform();
				$pdf->startTransform();
				$pdf->Rotate(-90, $lix, $liy);
					$pdf->setFontSize($size*0.8);		
				$pdf->Text($lix , $liy , $lifeInt);
				$pdf->stopTransform();
			break;
			default:  // gen > 6 only one line				
		$size = $ch / 15;
				$pdf->setFontSize($size);
				$lname = $lfi = 	$pdf->GetStringWidth 	( 	$name) ;	
				
				if ($lname > $ch*0.9){
					$size = $pdf->GetFontSize()*$ch*0.8/$lname;
					$pdf->SetFontSize($size);
				}
				if ($size > 0.8*$cw){
					$size = 0.6*$cw;
					$pdf->SetFontSize($size);
				}
				$lname = 	$pdf->GetStringWidth 	( 	$name) ;
				$ny   = $cy + $ch/2 - $lname / 2; 
				
				$cnx = $cx +  $cw*1.5/2;
				$nx    = $cnx ;
				
				$pdf->startTransform();
				$pdf->Rotate(-90, $nx, $ny);
				$pdf->Text($nx  , $ny , $name);
				$pdf->stopTransform();
				
			break;	
		}				
	}

		/*
	*	Init colors for sectors and text
	*
	*/
	private function initColors($colorSet){	
		$sexs = array('M', 'F', 'U');
		$this->tree_colors = array();
		foreach ($sexs as $sex){
			switch ($colorSet){
				case '1'://default		
					
					$this->tree_colors[$sex] = unserialize($this->getSetting('TREE_COLORS_' . $sex));
				break;
				case '2':
				if (Auth::user()->getPreference('TREE_COLORS_' . $sex ) ) {
					$this->tree_colors[$sex] = unserialize(Auth::user()->getPreference('TREE_COLORS_' . $sex ));
				} else {
					$this->tree_colors[$sex] = unserialize($this->getSetting('TREE_COLORS_' . $sex));
				}
				break;
					case '3':
						//classic				
					 $TEXT_COLOR_LIGHT_M = array(0, 0, 255);
					 $FILL_COLOR_LIGHT_M = array(173, 216, 230); //lightblue
					 $TEXT_COLOR_LIGHT_F = array(0, 0, 255);
					 $FILL_COLOR_LIGHT_F = array(254, 182, 193); //lightpink
					
					 $TEXT_COLOR_LIGHT_U = array(0, 0, 0);
					 $FILL_COLOR_LIGHT_U = array(180, 180, 180);
					 
					$this->tree_colors['M'] = array( 'bgd' => $FILL_COLOR_LIGHT_M, 'txt' => $TEXT_COLOR_LIGHT_M); 
					$this->tree_colors['F'] = array( 'bgd' => $FILL_COLOR_LIGHT_F, 'txt' => $TEXT_COLOR_LIGHT_F); 
					$this->tree_colors['U'] = array( 'bgd' => $FILL_COLOR_LIGHT_U, 'txt' => $TEXT_COLOR_LIGHT_U); 
		
				break;
				case '4':
					//black/white				
					 $TEXT_COLOR_LIGHT_M = array(0, 0, 255);
					 $FILL_COLOR_LIGHT_M = array(255, 255, 255); //white
					 
					 $TEXT_COLOR_LIGHT_F = array(255, 0, 0);
					 $FILL_COLOR_LIGHT_F = array(255, 255, 255); //lightpink
					
					 $TEXT_COLOR_LIGHT_U = array(0, 0, 0);
					 $FILL_COLOR_LIGHT_U = array(255, 255, 255);
					 
					$this->tree_colors['M'] = array( 'bgd' => $FILL_COLOR_LIGHT_M, 'txt' => $TEXT_COLOR_LIGHT_M); 
					$this->tree_colors['F'] = array( 'bgd' => $FILL_COLOR_LIGHT_F, 'txt' => $TEXT_COLOR_LIGHT_F); 
					$this->tree_colors['U'] = array( 'bgd' => $FILL_COLOR_LIGHT_U, 'txt' => $TEXT_COLOR_LIGHT_U); 
		
				break;
				
			}	
		}
		if ($colorSet == 3){
			//classic				
					 $TEXT_COLOR_LIGHT_M = array(0, 0, 255);
					 $FILL_COLOR_LIGHT_M = array(173, 216, 230); //lightblue
					 $TEXT_COLOR_LIGHT_F = array(0, 0, 255);
					 $FILL_COLOR_LIGHT_F = array(254, 182, 193); //lightpink
					
					 $TEXT_COLOR_LIGHT_U = array(0, 0, 0);
					 $FILL_COLOR_LIGHT_U = array(180, 180, 180);
					 
					$this->tree_colors['M'] = array( 'bgd' => $FILL_COLOR_LIGHT_M, 'txt' => $TEXT_COLOR_LIGHT_M); 
					$this->tree_colors['F'] = array( 'bgd' => $FILL_COLOR_LIGHT_F, 'txt' => $TEXT_COLOR_LIGHT_F); 
					$this->tree_colors['U'] = array( 'bgd' => $FILL_COLOR_LIGHT_U, 'txt' => $TEXT_COLOR_LIGHT_U); 
		}
		//set background
		if (($pref = Auth::user()->getPreference('TREE_BGD' )) == null){
			$this->bgd = $this->getSetting('TREE_BGD');
		} else {
			$this->bgd = $pref;
		}	
		
	return true;	
	}

	
}