<?php
/**
 * webtrees: online genealogy
 * Copyright (C) 2015 webtrees development team
 * Copyright (C) 2015 JustCarmen
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *---------------------------------------------------------------------
 *   V 1.0.1 for "arbre" V-1-7-2 for Webtrees 1.7.2
*   André DUMAS August 2015
*   http://www.adumas.1s.fr/wt
 */
namespace Andre\WebtreesAddOn\Arbre\Template;

use Fisharebest\Webtrees\File;
use Fisharebest\Webtrees\Auth;
use Fisharebest\Webtrees\Filter;
use Fisharebest\Webtrees\I18N;
use Fisharebest\Webtrees\Individual;
use Andre\WebtreesAddOn\Arbre\ArbreClass;
use Andre\WebtreesAddOn\Arbre\ArbreModule;


use TCPDF;
use TCPDF_STATIC;

class PdfPageController extends TCPDF {
	
	const PDF_CUSTOM_VERSION  = '1-0 2017-09-15 AD';
	const DEFAULT_FORMAT = 'A4';
	const DEFAULT_UNIT = 'pt';
	const AUTHOR = 'André DUMAS pour Webtrees';
	const KEYWORDS = 'PDF, Webtrees, module, arbre, ascendance';
	const CREATOR = 'André DUMAS FRANCE';
	const DEFAULT_TOP_MARGIN = '10';
	
	public $title = '';
	public $header;
	public $footer;
	private $PAGE_FORMAT;
	private $PAGE_UNIT ;
	private $PAGE_ORIENTATION ;
	
	
	public $pbottom;
	public $ptop;
	public $pwidth;
	public $pheight;
	
	public function __construct($format, $orientation = 'L') {
		global $WT_TREE;
		
			//$this->header = '<div style="width:100%;height:5%;border-bottom:0px solid black;">';//.$WT_TREE->getTitle(). '     :     ' . $titre .'</div>';	
			$this->footer =  '<div style="width:100%;">'
						. 'Module "arbre" for Webtrees @AD 2015-2016-2017 '.'   - ' . Date("Y-m-d") . '</div>';
				
			//format
			if (!is_array($format)){
				$this->PAGE_FORMAT = $format;
			} else {
				$this->PAGE_FORMAT =  self::DEFAULT_FORMAT;
			}

			$this->PAGE_UNIT = self::DEFAULT_UNIT;
			$this->PAGE_ORIENTATION = $orientation; //L pour paysage

			parent::__construct($this->PAGE_ORIENTATION, $this->PAGE_UNIT, $this->PAGE_FORMAT, true, 'UTF-8', false);
			
			// set document information
			$this->SetCreator(self::CREATOR);
			$this->SetAuthor(self::AUTHOR);
			$this->SetSubject('Webtrees module');
			$this->SetKeywords(self::KEYWORDS);

			// set default monospaced font
			$this->SetDefaultMonospacedFont(PDF_FONT_MONOSPACED);


			// set margins
			$this->SetMargins(PDF_MARGIN_LEFT, PDF_MARGIN_TOP, PDF_MARGIN_RIGHT);
			$this->SetTopMargin(self::DEFAULT_TOP_MARGIN);

			$this->setPrintFooter(false);
			$this->setPrintHeader(false);
			
			// set image scale factor
			$this->setImageScale(PDF_IMAGE_SCALE_RATIO);
	
			$this->pwidth = $this->GetPageWidth();
			$this->pheight = $this->GetPageHeight();
		/////////////
				//Bottom
				$this->pbottom =  $this->pheight * 0.965;
				$this->pleft = 0;

	}
	
	
	

	
	    //Page header) 
		public function Header() {
		$h = $this->getPageHeight();
		$this->SetY($h*0.02);
		$o_size = $this->getFontSize();
        // Set font
		$ls = $this->getStringWidth($this->header);
		$size = $this->getFontSize();
		if ($ls > 0){
			$size = $size/(2.6*$ls/($this->getPageWidth()) );
			$this->SetFont('dejavu sans', 'BI', $size) ;         // Title   
		}
		// $this->setCellPadding(5);
     $this->writeHTMLCell(
							$w = 0, 
							$ht = 0,
							$x = '', 
							$y = '', 
							$this->header, 
							$border = 'B', 
							$ln = 1, 
							$fill = false, 
							$reseth = true, 
							$align = 'C', 
														$autopadding = true
							);
				$this->setFontSize($o_size);			
	}

    // Page footer
    public function Footer() {
		$h = $this->getPageHeight();
        // Position at 15 mm from bottom
        $this->SetY(-$h*0.025);
 
		 // Set font
		$o_size = $this->getFontSize(); 
		$ls = $this->getStringWidth($this->footer);
		$size = $this->getFontSize();
		if ($ls > 0){		
			$size = $size/(2.5*$ls/($this->getPageWidth()) );
			$this->SetFont('dejavu sans', 'BI', $size) ;      
		}
        $this->writeHTMLCell(
							$width = 0, 
							$height = 0, 
							$x = '',
							$y = '',
							$html = $this->footer, 
							$border = 'T',
							$ln = 0,
							$fill = false,
							$reseth = true,
							$align = '',
							$autopadding = true 
						);
		$this->setFontSize($o_size);				
    }
	


	
}