<?php
/**
 * webtrees: online genealogy
 * Copyright (C) 2015 webtrees development team
 * Copyright (C) 2015 JustCarmen
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *---------------------------------------------------------------------
 *   V 1.0.1 for "arbre" V-1-7-2 for Webtrees 1.7.2
*   André DUMAS August 2015
*   http://www.adumas.1s.fr/wt
*
*	V-1-1 for Arbre V-1-7-6-1  WT 1.7.6  25 JUN 2016
*	V-1-2 for Arbre V-1-7-8-1  WT 1.7.8  20 NOV 2016
*	V-1-3 for Arbre V-1-7-9-1  WT 1.7.9  15 JAN 2017
*			print agna/cogna trees
 */
namespace Andre\WebtreesAddOn\Arbre\Template;

use Fisharebest\Webtrees\File;
use Fisharebest\Webtrees\Filter;
use Fisharebest\Webtrees\I18N;
use Fisharebest\Webtrees\Auth;
use Fisharebest\Webtrees\Individual;
use Fisharebest\Webtrees\Family;
use Andre\WebtreesAddOn\Arbre\ArbreClass;
use Andre\WebtreesAddOn\Arbre\ArbreModule;

use Andre\WebtreesAddOn\Arbre\Template\PdfPageController;
use Andre\WebtreesAddOn\Arbre\Template\myPDFTools;
use TCPDF;

class PdfTemplate_1 extends ArbreModule {
	
	const PdfTemplate_CUSTOM_VERSION  = '1-3';
	var $tree_colors = array();	
		
	public function pageBody($format, $type = 'main', $colorType) {
		global $WT_TREE;
		
		$this->initColors($colorType);
			$temp = WT_DATA_DIR . ArbreModule::getSetting('ARBRE_CACHE_DIR');
			switch ($type) {
				case 'main':
					$f = DIRECTORY_SEPARATOR . ArbreModule::getSetting('ARBRE_CACHE_FILE') ;
					$titre = 'Arbre d\'ascendance';
				break;
				case 'cogna':
					$f = DIRECTORY_SEPARATOR . $type . '_' . ArbreModule::getSetting('ARBRE_CACHE_FILE') ;
					$titre = 'Arbre cognatique';
				break;
				case 'agna':
					$f = DIRECTORY_SEPARATOR . $type . '_' . ArbreModule::getSetting('ARBRE_CACHE_FILE') ;
					$titre = 'Arbre agnatique';
				break;
				default:
				break;
			}	
			$cache = $temp . $f;
			
		if (is_dir(WT_DATA_DIR) && is_readable($cache)) {
			$t 		= file_get_contents($cache);
			$s 		= unserialize($t);
			$nlin 	= $s['NbreLn'];
			$ncol 	= $s['NbreCol'];
			$ttype 	= $s['type'];
			$root = $s['root'];
			$rows = $s['rows'];
			$data 	= $s['data'];  //array[li][ col]
		
		$person = Individual::getInstance($root, $WT_TREE);
		$titre = $person->getAllNames()[$person->getPrimaryName()]['fullNN']  ;
			
			$header = '<div style="width:100%;">'.$WT_TREE->getTitle(). '     :     ' . $titre .'</div>';	
			$footer = '<div style="width:100%;b">'
						. 'Module "arbre" for Webtrees @AD 2015-2016'.'</div>';
// ---------------------------------------------------------
$PDF_PAGE_FORMAT = $format;
$PDF_PAGE_ORIENTATION = 'L'; //L pour paysage

$pdf = new PdfPageController($PDF_PAGE_FORMAT, PDF_PAGE_ORIENTATION);
$tool = new myPDFTools();

// set auto page breaks
$pdf->SetAutoPageBreak(false, PDF_MARGIN_BOTTOM);

$twidth = $pdf->getPageWidth() * 0.9;
$theight = $pdf->getPageHeight() * 0.8;

$deltacol = 4;
$deltalin = 4;

$lmargin = $twidth * 0.05;
$tmargin = $theight * 0.1;;

$cwidth = ($twidth - ($ncol - 1)*$deltacol ) / $ncol;
$cheight = ($theight - ($nlin - 1)*$deltalin ) / $nlin;

$fsize = $pdf->getFontSize();
$text = 'Gen:  XX';
$fsize = $this->getSizeOfFont($text, $cwidth, $pdf->getFontSize(), $pdf, 0.5);

	$lightblue = array(173, 216, 230);
	$lightpink = array(254, 182, 193);
	$lightgrey = array(211, 211, 211);
	$black = array(0, 0, 0);
//---------------------------------------------------------------
// add a page
$pdf->AddPage();  
$pdf->header = $header;
$pdf->Header();

	// Title of page
	$titre = 'Arbre cognatique  ' . $titre . '  ' . $tool->getMyLifeSpan($person);
		$nx = $twidth/2 - $pdf->getStringWidth($titre) / 2;
		$ny = $theight * 0.06;
		$pdf->Text($nx, $ny, $titre);

	//Boucle...
	for ($c = 0; $c < $ncol ; $c++){
		$cy = $tmargin;
		$tcheight  = $cheight;
		$offset = $cheight + $deltalin;
		
		for ($l = 0; $l < $nlin; $l++){
			$text = '';
			if (($l == 0) || ($l == 4) || ($l == 2) ) {
				$pdf->setFontSize($fsize);
				$pdf->SetTextColorArray($black);
				$text = $data[$l][$c];
				$tcheight  = $cheight/2 ;
					if ($l == 0){					
						$pdf->setFontSize($this->getSizeOfFont($text, $cwidth, $pdf->getFontSize(), $pdf, 0.6));
						$offset = 0;
					}
			}		
			if (($l == 1) || ($l == 5) || ($l == 3) ) {
					$offset = $cheight/2 + $deltalin;;
			}
				
				$cx =  $lmargin + $c * ($cwidth + $deltacol);
				$cy +=  $offset ;
				
				
				$pdf->setFontSize($fsize);
				$name = '';
				$lifeInt = '';
				$surname = '';
				$givname = '';
				$color = $lightgrey;
				$pdf->SetTextColorArray($black);
				
				if ( $l < $rows[$c] ){
					$person = Individual::getInstance($data[$l][$c], $WT_TREE);
					if ($person){
						$surname = $tool->getMyTabNames($person)[0];
						$givname = $tool->getMyTabNames($person)[1];
						$lifeInt = $tool->getMyLifeSpan($person);	
							//colors													
							switch ($sex = $person->getSex()) {
							case 'M':
								$pdf->SetTextColorArray($this->tree_colors['M']['txt']);
								$color = $this->tree_colors['M']['bgd'];
								
							break;	
							case 'F':
								$pdf->SetTextColorArray($this->tree_colors['F']['txt']);
								$color = $this->tree_colors['F']['bgd'];
							break;	
							default:
								$pdf->SetTextColorArray($this->tree_colors['U']['txt']);
								$color = $this->tree_colors['U']['bgd'];
							break;	
					}

					/////
					} 
				
				
				$pdf->RoundedRect($cx, $cy , $cwidth, $tcheight, 1.50, '1111', 'DF', '' , $color);
				
				if (($l == 0) || ($l == 4) || ($l == 2) ){
					$pdf->setFontSize($fsize);
					$nx = $cx + $cwidth/2 -$pdf->getStringWidth($text) /2;
					$ny =  $cy + $tcheight /8 + $pdf->getFontSize()/4;
					$pdf->Text($nx, $ny, $text);
				}
				
				$lfsize = $this->getSizeOfFont('(1900-1999)', $cwidth, $pdf->getFontSize(), $pdf, 0.6);						
				$pdf->setFontSize($lfsize);
				
				//surname			
				$nx = $cx + $cwidth/2 -$pdf->getStringWidth($surname) /2;
				$ny =  $cy + $tcheight /11 + $pdf->getFontSize()/4;				
				$pdf->Text($nx, $ny, $surname);
				
				$pdf->setFontSize($lfsize*0.7);
				//givn 
				$nx = $cx + $cwidth/2 -$pdf->getStringWidth($givname) /2;
				$ny =  $cy + $tcheight /3 + $pdf->getFontSize()/2;				
				$pdf->Text($nx, $ny, $givname);
				
				//lifeSpan
				$nx = $cx + $cwidth/2 -$pdf->getStringWidth($lifeInt) /2;
				$ny =  $cy + $tcheight /1.8 + $pdf->getFontSize()/2;				
				$pdf->Text($nx, $ny, $lifeInt);
								
				}
		}
		
	}

$pdf->footer = $footer;
$pdf->Footer();
		
			$pdf->Output('Ascendance' . '.pdf', 'I');
		
	} 
	}

	
	private function getSizeOfFont($text, $w, $size, $pdf, $lmax){
		
		$tw = $pdf->getStringWidth($text);
		if ( $tw/$w > $lmax) {
			return $size/(($tw/$w)/$lmax);
			
		} else {
			return $size;
		}
	}
	
	/*
	*	Init colors for sectors and text
	*
	*/
	private function initColors($colorSet){	
		$sexs = array('M', 'F', 'U');
		$this->tree_colors = array();
		foreach ($sexs as $sex){
			switch ($colorSet){
				case '1'://default		
					
					$this->tree_colors[$sex] = unserialize($this->getSetting('TREE_COLORS_' . $sex));
				break;
				case '2':
				if (Auth::user()->getPreference('TREE_COLORS_' . $sex ) ) {
					$this->tree_colors[$sex] = unserialize(Auth::user()->getPreference('TREE_COLORS_' . $sex ));
				} else {
					$this->tree_colors[$sex] = unserialize($this->getSetting('TREE_COLORS_' . $sex));
				}
				break;
					case '3':
						//classic				
					 $TEXT_COLOR_LIGHT_M = array(0, 0, 255);
					 $FILL_COLOR_LIGHT_M = array(173, 216, 230); //lightblue
					 $TEXT_COLOR_LIGHT_F = array(0, 0, 255);
					 $FILL_COLOR_LIGHT_F = array(254, 182, 193); //lightpink
					
					 $TEXT_COLOR_LIGHT_U = array(0, 0, 0);
					 $FILL_COLOR_LIGHT_U = array(180, 180, 180);
					 
					$this->tree_colors['M'] = array( 'bgd' => $FILL_COLOR_LIGHT_M, 'txt' => $TEXT_COLOR_LIGHT_M); 
					$this->tree_colors['F'] = array( 'bgd' => $FILL_COLOR_LIGHT_F, 'txt' => $TEXT_COLOR_LIGHT_F); 
					$this->tree_colors['U'] = array( 'bgd' => $FILL_COLOR_LIGHT_U, 'txt' => $TEXT_COLOR_LIGHT_U); 
		
				break;
				case '4':
					//black/white				
					 $TEXT_COLOR_LIGHT_M = array(0, 0, 255);
					 $FILL_COLOR_LIGHT_M = array(255, 255, 255); //white
					 
					 $TEXT_COLOR_LIGHT_F = array(255, 0, 0);
					 $FILL_COLOR_LIGHT_F = array(255, 255, 255); //lightpink
					
					 $TEXT_COLOR_LIGHT_U = array(0, 0, 0);
					 $FILL_COLOR_LIGHT_U = array(255, 255, 255);
					 
					$this->tree_colors['M'] = array( 'bgd' => $FILL_COLOR_LIGHT_M, 'txt' => $TEXT_COLOR_LIGHT_M); 
					$this->tree_colors['F'] = array( 'bgd' => $FILL_COLOR_LIGHT_F, 'txt' => $TEXT_COLOR_LIGHT_F); 
					$this->tree_colors['U'] = array( 'bgd' => $FILL_COLOR_LIGHT_U, 'txt' => $TEXT_COLOR_LIGHT_U); 
		
				break;
				
			}	
		}
		if ($colorSet == 3){
			//classic				
					 $TEXT_COLOR_LIGHT_M = array(0, 0, 255);
					 $FILL_COLOR_LIGHT_M = array(173, 216, 230); //lightblue
					 $TEXT_COLOR_LIGHT_F = array(0, 0, 255);
					 $FILL_COLOR_LIGHT_F = array(254, 182, 193); //lightpink
					
					 $TEXT_COLOR_LIGHT_U = array(0, 0, 0);
					 $FILL_COLOR_LIGHT_U = array(180, 180, 180);
					 
					$this->tree_colors['M'] = array( 'bgd' => $FILL_COLOR_LIGHT_M, 'txt' => $TEXT_COLOR_LIGHT_M); 
					$this->tree_colors['F'] = array( 'bgd' => $FILL_COLOR_LIGHT_F, 'txt' => $TEXT_COLOR_LIGHT_F); 
					$this->tree_colors['U'] = array( 'bgd' => $FILL_COLOR_LIGHT_U, 'txt' => $TEXT_COLOR_LIGHT_U); 
		}
		//set background
		if (($pref = Auth::user()->getPreference('TREE_BGD' )) == null){
			$this->bgd = $this->getSetting('TREE_BGD');
		} else {
			$this->bgd = $pref;
		}	
		
	return true;	
	}

	
}