<?php
// Classes and libraries for module system
//
// webtrees: Web based Family History software
// Copyright (C) 2012 webtrees development team.
// Derived from PhpGedView
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
//--------------------------------------------------------
/*
* http://www.adumas.1s.fr/wt/   
* AD  July 2015
*============================================
*
* 	From the admin page see -> Documentation / changes
*	or readme from the download page http://www.adumas.1s.fr/download
*	See on my site and on facebook: https://www.facebook.com/Modules-Webtrees-1690823611178607/
*/ 

namespace Andre\WebtreesAddOn\Arbre;

use Composer\Autoload\ClassLoader;
use Fisharebest\Webtrees\Controller\BaseController;
use Fisharebest\Webtrees\Controller\PageController;
use Fisharebest\Webtrees\Filter;
use Fisharebest\Webtrees\I18N;
use Fisharebest\Webtrees\Log;
use Fisharebest\Webtrees\Module\AbstractModule;
use Fisharebest\Webtrees\Module\ModuleMenuInterface;
use Fisharebest\Webtrees\Module\ModuleTabInterface;
use Fisharebest\Webtrees\Module\ModuleConfigInterface;
use Fisharebest\Webtrees\Individual;
use Fisharebest\Webtrees\Menu;
use Fisharebest\Webtrees\Auth;
use Fisharebest\Webtrees\Database;
use Fisharebest\Webtrees\Functions;
use Fisharebest\Webtrees\Functions\FunctionsPrint;

use Fisharebest\Webtrees\Session;

use Andre\WebtreesAddOn\Arbre\Template\PdfTemplate_1;
use Andre\WebtreesAddOn\Arbre\Template\PdfNewTreeTemplate_1;
use Andre\WebtreesAddOn\Arbre\Template\PdfImgTemplate_1;
use Andre\WebtreesAddOn\Arbre\Template\PdfNewTreeTemplateRRect_1;
use Andre\WebtreesAddOn\Arbre\Template\PdfNewTreeTemplatePie_1;
use Andre\WebtreesAddOn\Arbre\Template\PdfNewTreeTemplatePieMP2_1;
use Andre\WebtreesAddOn\Arbre\Template\AdminTemplate;

use Andre\WebtreesAddOn\Arbre\Template\PdfNewTreeTemplateRRectMP_1;
use Andre\WebtreesAddOn\Arbre\Template\PdfNewTreeTemplateRRectM3P_1;


use Andre\WebtreesAddOn\Arbre\Template\PdfPageController;


class ArbreModule extends AbstractModule implements  ModuleConfigInterface, ModuleTabInterface {

		/* Cache for arbre */
		CONST CUSTOM_VERSION  = '1-7-9-9b';
		CONST CUSTOM_VERSION_DATE  = '2018-04-07';
		CONST CUSTOM_VERSION_AUTHOR = 'André DUMAS FRANCE';
		
		const CUSTOM_WEBSITE  = 'http://www.adumas.1s.fr/wt';
		const CUSTOM_DOWNLOAD = 'http://www.adumas.1s.fr/download';

	// How to update the database schema for this module
	const SCHEMA_TARGET_VERSION			= 2;
	const SCHEMA_SETTING_NAME			= 'ARBRE_SCHEMA_VERSION';
	const SCHEMA_MIGRATION_PREFIX		= '\Andre\WebtreesAddOn\Arbre\Schema';
						
			var $NumberOfGenerations ;  //Nombre max de generations affichées
			var $directory;
			var $myfan;
			var $myarbre;
			
	public function __construct() {
		parent::__construct(__DIR__);

		// register the namespaces
		$loader = new ClassLoader();
		$loader->addPsr4('Andre\\WebtreesAddOn\\Arbre\\', WT_MODULES_DIR . $this->getName() . '/app');
		$loader->register();
	
		//initialise le nombre max de geneartions affichées
		$this->NumberOfGenerations = 5;
		$this->directory = WT_MODULES_DIR . $this->getName();
						
		}

	/**
	 * Get the module class.
	 * JustCarmen Thanks (-_+)
	 * Class functions are called with $this inside the source directory.
	 */
	private function module() {
		return new ArbreClass;
	}
	
	// Extend Module
	// = = = = = = = = = = = = = = = =
	public function getTitle(){ 
		$title = 'Arbre ';
		return $title; /* I18N: Name of a module/sidebar I18N::translate('Arbre:   '); */
		
		return $title;
	}

	// = = = = = = = = = = = = = = = =
	public function getDescription(){
		return I18N::translate('A tab showing an individual’s tree.');
	}

	// = = = = = = = = = = = = = = = =
	public function defaultTabOrder(){
		return 20;
	}

		/** {@inheritdoc} */
	public function hasTabContent() {
		return true;
	}

	/** {@inheritdoc} */
	public function isGrayedOut() {
		return false;
	}

		/** {@inheritdoc} */
	public function getPreLoadContent() {
		return 
		'<script>
			if (document.createStyleSheet) {
				document.createStyleSheet("' . $this->css() . '"); // For Internet Explorer
			} else {
				var newSheet=document.createElement("link");
    		newSheet.setAttribute("rel","stylesheet");
    		newSheet.setAttribute("type","text/css");
   			newSheet.setAttribute("href","' . $this->css() . '");
		    document.getElementsByTagName("head")[0].appendChild(newSheet);
			}
			</script>
			<script>
			if (document.createStyleSheet) {
				document.createStyleSheet("' . $this->csscp() . '"); // For Internet Explorer
			} else {
				var newSheet=document.createElement("link");
    		newSheet.setAttribute("rel","stylesheet");
    		newSheet.setAttribute("type","text/css");
   			newSheet.setAttribute("href","' . $this->csscp() . '");
		    document.getElementsByTagName("head")[0].appendChild(newSheet);
			}
			</script>
		';
	}

		//*********************
		/** {@inheritdoc} */
	public function getConfigLink() {
		return 'module.php?mod=' . $this->getName() . '&amp;mod_action=admin_config';
	}	
		
	//=============================
	public function canLoadAjax() {
		return true;
	}

			
			
	// = = = = = = = = = = = = = = = =
	public function getTabContent() {
		global $controller;		
		/* update schema */
		Database::updateSchema(self::SCHEMA_MIGRATION_PREFIX, self::SCHEMA_SETTING_NAME, self::SCHEMA_TARGET_VERSION);
	
		//verif conf
		$dir = $this->getSetting('ARBRE_CACHE_DIR');
		$file = $this->getSetting('ARBRE_CACHE_FILE');
/*			if (!$dir){
				$dir = 'arbre_cache';
				$this->setSetting('ARBRE_CACHE_DIR', $dir);
			} 
			if (!$file){
				$file = 'arbre.txt';
				$this->setSetting('ARBRE_CACHE_FILE', $file);
			} 
*/			//for deskTab and update
				if (!$this->getSetting('CUSTOM_VERSION')  || ( $this->getSetting('CUSTOM_VERSION') != self::CUSTOM_VERSION )  ){
					$this->setSetting('CUSTOM_VERSION', self::CUSTOM_VERSION);
				}
	
		   $js = '	var ModuleDir			= "' . $this->directory . '";'.'
					var ModuleName			= "' . $this->getName() . '";
					var root 				= "' . Filter::get('pid') . '";					
					';

					$this->module()->addCache();
		$root = Filter::get('pid');  
		$person = $this->module()->getPerson($root);
		$max ='5';
			
		$out = '';			
			ob_start();	
	echo '<div id="myArbre">
				 <div id"arbre-script" >
					<script>'.$js.'</script>
					<script src="' . $this->directory . '/colorPicker/colorPicker.js"></script>
					<script src="' . $this->directory . '/js/tab.js"></script>
				</div>';
			echo '
				<div id="conteneur">';
			echo '<body>'.
				'<div id="arbre5g">'.
				$this->module()->getTabHeader($root).
				$this->module()->getTableAscsss($max, $root); 
				/*
				*  data ready -> save fan chart
				*
				*/
				if (!($fctype = $this->getSetting('ARBRE_LAST_FAN')) ){
					$fctype = '2'; //270° default
					$this->setSetting('ARBRE_LAST_FAN', $fctype);
				} 
				$myfc = new myFanChartClass();
				
				$myfc->getOutput(ArbreClass::getMonArbre(), $fctype); //eventail secteurs
				
			$text ='plus de 5 générations';
			
			echo '</div>';
			echo '<br><br><div id="agnatic">				
				<table><tr><td>' ;
				if (Auth::check()){					
					echo '<span style="margin-left:25px;"><a id="agna-pdf" style="margin-left:25px;" href="#" title="' . I18N::translate('Create a pdf') . '">';
				} else {
					echo '<span style="margin-left:25px;">';
					echo  '<a id="" style="margin-left:2px;"  title="' . I18N::translate('Create a pdf if you are a member: subscribe for an account')  . '">';
				}
				echo '	<i class="icon-mime-application-pdf"></i></a></span>'
				. '</td><td>
					<span class="titreArbre">' .I18N::translate("Agnatic tree: ") . 
							'<a href="'.$person->getHtmlUrl().'">'.
							$person->getFullName().'
										</a>'.$person->getLifeSpan().'</span>
				</td></tr></table>				
				</div>';
			
			
			echo '<br><br><div id="cognatic">
				
				<table><tr><td>' ;
				if (Auth::check()){					
					echo '<span style="margin-left:25px;"><a id="cogna-pdf" style="margin-left:25px;" href="#" title="' . I18N::translate('Create a pdf') . '">';
				} else {
					echo '<span style="margin-left:25px;">';
					echo  '<a id="" style="margin-left:2px;"  title="' . I18N::translate('Create a pdf if you are a member: subscribe for an account')  . '">';
				}
				echo '	<i class="icon-mime-application-pdf"></i></a></span>'
				. '</td><td>
					<span class="titreArbre">' .I18N::translate("Cognatic tree: ") . 
							'<a href="'.$person->getHtmlUrl().'">'.
							$person->getFullName().'
										</a>'.$person->getLifeSpan().'</span>
				</td></tr></table>
					
				</div>';
						
			$cache_filename = WT_DATA_DIR . 'arbre_cache' . DIRECTORY_SEPARATOR . 'fan_arbre.png';
		
			echo  '<div id="myFanChart" >' . 
						'<div class="tvertical"><span>André DUMAS:  ' . WT_BASE_URL . '    Fan chart for WT 1.7.9    V-' . self::CUSTOM_VERSION. '</span></div>' . 
						
				'<div class="img_fc">
						
						
						<table style="border:0px;"><tr><td>' .
						'';
				if (Auth::check()){					
						echo '<span style="width:30px; float:left;padding-top:15px;">
							<a id="fan-pdf" style="margin-left:25px;" href="#" title="' . 
							I18N::translate('Create a pdf of this fan') . '">';
				} else {
					echo 	'<span style="width:30px; float:left;padding-top:15px;">' .
							'<a id="" style="margin-left:25px;"  title="' . 
							I18N::translate('Create a pdf if you are a member: subscribe for an account')  . '">';
				}
						
						echo '	<i class="icon-mime-application-pdf"></i></a></span>
								<span>    </span></td><td>' ;
						
					echo		'<span class="titre_fc">' . $person->getFullName() 
								 . ' (' . $person->getLifeSpan() . ')</span></td><td>' ;
				if (Auth::check()){					
						echo '<span style="width:30px; padding-top:15px;">
							<a id="pie-pdf" style="margin-left:25px;" href="#" 
											title="' . I18N::translate('Create a pdf (180°,5 -> 11 gen )') . '">';
				} else {
					echo 	'<span style="width:30px;padding-top:15px;">' .
							'<a id="" style="margin-left:25px;"  title="' . I18N::translate('Create a pdf if you are a member: subscribe for an account')  . '">';
				}
						
						echo '	<i class="icon-mime-application-pdf"></i></a></span>
								<span>    </span> </td></tr>
								<tr>
								<td style="text-align:center;">
									<input type="submit" class="fan180" value="180°"/>
								</td>
								<td style="text-align:center;">
									<input type="submit" class="fan270" value="270°"/>
								</td>
								<td style="text-align:center;">
									<input type="submit" class="fan360" value="360°"/>
								</td>
								</tr>
								</table>' ;
								
						echo ' ' . 
							'<img 
								src="module.php?mod=' . $this->getName() . '&amp;mod_action=print_fan' . '"' .  
									'style="padding:5px;margin:5px 2%;" ' .
									' alt="Fan chart: ' . strip_tags($person->getFullName()) . '"' .
									' title="' . strip_tags($person->getFullName()) . '"' .
									'>' . 
				'</div>' ; //img_fc
						
								
			echo		'</div>
						</div >' ;	
			
			
			echo	'</body>';
					
					
			
			
			echo		'<div id="dialog-confirm" title="'. I18N::translate("Generate a PDF").'" 				style="display:none">
						<p>'.I18N::translate("The pdf contains only visible generation blocks.").'</p>
						<span id="#arbre-control-output" style="display:none">'.$text.'</span>
							<form id="#arbre-nbre-generation" >
								<select id="#arbre-output" class="selm">
									<option value="0">Choississez une option</option>
									<option  value="5">5' . I18N::translate('generations') . ' :Format A4</option>
									<option value="6">5 ' . I18N::translate('generations') . ' :Format A3</option>
									<option value="7">6 ' . I18N::translate('generations') . ' :Format A3</option>
									<option value="8">7 ' . I18N::translate('generations') . ' :Format A3</option>
								</select>
							</form>
						</div>
				
				<div id="popup" class="popup_block">
				
				<div id="tabs1">
					<ul>
						<li><a href="#tabs-2">' . I18N::translate('Children') . '</a></li>
						<li><a href="#tabs-5">' . I18N::translate('Siblings') . '</a></li>
						<li><a href="#tabs-3">' . I18N::translate('Uncles & aunts') . '</a></li>
						<li><a href="#tabs-4">' . I18N::translate('Cousins') . '</a></li>			
					</ul>
					 <div id="tabs-2">            
					 </div>
					 <div id="tabs-5">
					 </div>
					 <div id="tabs-3">            
					 </div>
					 <div id="tabs-4">           
					 </div>
							
				</div>			
			</div>'; //myArbre
				
		return  ob_get_clean();
	} // fin getSidebarContent

	
	
	// = = = = = = = = = = = = = = = =
	public function getTabAjaxContent(){
		return true;
	}

	// = = = = = = = = = = = = = = = =
	public function modAction($mod_action) {
		global $WT_TREE, $pid;
		switch ($mod_action) {
			
			case  'show_pdf' : 
				$format = Filter::get('format');
				$type = Filter::get('type');
				$gen = Filter::get('gen');
				$colorType = Filter::get('colorType');
				switch ($type ){
					case 'fan':
					$template = new PdfImgTemplate_1;
					echo $template->pageBody($format, $type);
					break;
					
					case 'arbre':
					$root = Filter::get('pid');
					$colorType = Filter::get('colorType');
					$this->module()->createTableOfGenerations($gen, $root);
					$template = new PdfNewTreeTemplateRRect_1; // rect cells
					echo $template->pageBody($format, $colorType,  $type);
					break;
					case 'pie':
					$root = Filter::get('pid');
					$colorType = Filter::get('colorType');
					$ftype = Filter::get('ftype');
					$mp = Filter::getBool('multipages', 'false');
					$this->module()->createTableOfGenerations($gen, $root);
					if (!$mp){
						$template = new PdfNewTreeTemplatePie_1; // Pie sectors
					} else {
						$template = new PdfNewTreeTemplatePieMP2_1;
					}
					echo $template->pageBody($format, $type, $colorType, $ftype, $mp);
					break;
					case 'pie2p':
					$root = Filter::get('pid');
					$colorType = Filter::get('colorType');
					$ftype = Filter::get('ftype');
					$this->module()->createTableOfGenerations($gen, $root);
					$template = new PdfNewTreeTemplatePieMP2; // Pie sectors
					echo $template->pageBody($format, $type, $colorType, $ftype);
					break;
					case 'booklet':
						$root = Filter::get('pid');
						$colorType = Filter::get('colorType');
						$ftype = Filter::get('ftype');
						$btype = Filter::get('btype');
						
						$this->module()->createTableOfGenerations($gen, $root);
							switch ($btype) {
								case '2P':
									$template = new PdfNewTreeTemplateRRectMP_1; // Pie sectors
								break;
								case '3P':
									$template = new PdfNewTreeTemplateRRectM3P_1; // Pie sectors
								break;
							}
							
							echo $template->pageBody($format, $type, $colorType, $ftype);
					break;
					default:
					   header('Location: individual.php?pid=I6');
					break;
				}
			break;
			case 'showspecificpdf': // cogna agna trees				
				$type = Filter::get('type');
				$format = Filter::get('format');
				$colorType = Filter::get('colorType');
				
					$template = new PdfTemplate_1;
					echo $template->pageBody($format, $type, $colorType);
			break;
			
			case  'pdf_direct' : 		    
				$format = trim(Filter::get('format'));
				$root = Filter::get('root');	 
				$gen = trim(Filter::get('gen'));	
				$this->module()->createTableOfGenerations($gen, $root);
			break;	
				
			case 'createfan':
				$fctype = Filter::get('fctype');
				$this->setSetting('ARBRE_LAST_FAN', $fctype);
				$pid = Filter::get('pid'); 
				$cache = WT_DATA_DIR . $this->getSetting('ARBRE_CACHE_DIR') . DIRECTORY_SEPARATOR . 'data_' . $this->getSetting('ARBRE_CACHE_FILE');
				
				$myfc = new myFanChartClass();
				$tab = unserialize(file_get_contents($cache));
				$myfc->getOutput($tab,  $fctype);
				
				header('Location: individual.php?pid=' . $pid . '&ged=' . $WT_TREE->getName() );
			break;

			case 'display_cogna' :
			$root = Filter::post('root');
				echo $this->module()->specificTree(familyTools::getAgnaticPedigre($root, 'C'), 'cogna', $root);
					
			break;
			case 'display_agna' :
			$root = Filter::post('root');
				echo $this->module()->specificTree(familyTools::getAgnaticPedigre($root, 'A'), 'agna', $root);
					
			break;
			case 'savecache':
				$type = Filter::post('type');
				$data = Filter::post('cache');
				$this->module()->pushToCache($data, $type);
			break;
		
			case 'display_fct1': // lignes et arc de cercles 
				$fcgen = Filter::get('gen');
				$fcformat = Filter::get('format');
				$pid = Filter::get('pid'); 
				$colorType = Filter::get('colorType');
				$this->module()->createTableOfGenerations($fcgen, $pid);								
				$myfc = new PdfNewTreeTemplate_1();
				$fctype = 'fan';
				$myfc->pageBody($fcformat,  $fctype, $colorType);				
			break;	
						case 'print_fan':
				$mimetype = 'image/png';
				$cache_filename = WT_DATA_DIR . $this->getSetting('ARBRE_CACHE_DIR') . DIRECTORY_SEPARATOR . 'fan_arbre.png';
				$filesize = filesize($cache_filename);
				// send headers for the image
				header('Content-Type: ' . $mimetype);
				header('Content-Disposition: filename="' . basename($cache_filename) . '"');
				header('Content-Length: ' . $filesize);

				// Some servers disable fpassthru() and readfile()
				if (function_exists('readfile')) {
					readfile($cache_filename);
				} else {
					$fp = fopen($cache_filename, 'rb');
					if (function_exists('fpassthru')) {
						fpassthru($fp);
					} else {
						while (!feof($fp)) {
							echo fread($fp, 65536);
						}
					}
					fclose($fp);
				}
			
			break;
			
			//Family
			case 'details' :
				$sosa = Filter::get('sosa');
				$gen = Filter::get('gen');
				$pid = Filter::get('pid');		
				echo familyTools::listingAllChildren($pid);
			break;
			
			
			
			case 'parents' : //Give brothers & sisters
				$sosa = Filter::get('sosa');
				$gen = Filter::get('gen');
				$pid = Filter::get('pid');
				echo familyTools::listingSiblings($pid);
				
			break;
			case 'Gdparents' :
				$sosa = Filter::get('sosa');
				$gen = Filter::get('gen');
				$pid = Filter::get('pid');
				echo familyTools::listingUncles($pid);
				
			break;
			case 'cousins' :
				$sosa = Filter::get('sosa');
				$gen = Filter::get('gen');
				$pid = Filter::get('pid');
				echo familyTools::listingCousins($pid);
				
			break;
			case 'mp-pdf':
				$this->module()->getColorPick();
			break;
			case 'user_colors':
				$this->updateColorsUserSetting();
			break;
			case 'admin_config':
				/* update schema */				
				Database::updateSchema(self::SCHEMA_MIGRATION_PREFIX, self::SCHEMA_SETTING_NAME, self::SCHEMA_TARGET_VERSION);

				//for deskTab and update
				if (!$this->getSetting('CUSTOM_VERSION')  || ( $this->getSetting('CUSTOM_VERSION') != self::CUSTOM_VERSION )  ){
					$this->setSetting('CUSTOM_VERSION', self::CUSTOM_VERSION);
				}
				$template = new AdminTemplate;
				return $template->pageContent();
			break;
			case 'check_version':
				$template = new AdminTemplate;
				$template->forceUpdate();
			break;
			
			case 'saveconfbgk':
				$im = Filter::get('im');
				$conf = Filter::get('conf');
				$type = Filter::get('type');
				switch ($type){
					case 'fc':
						$prefix = 'FAN_CHART_BGD';					
					break;
					case 'b2p':
						$prefix = 'TREE_BGD';
					break;
					case 'b3pl':
						$prefix = 'TREE_BGD_LAT';
						$prefix1 = 'BOOKLET_BGD_LAT';
					break;
					default:
						//b3pc
						$prefix = 'TREE_BGD_C';
						$prefix1 = 'BOOKLET_BGD_CENTER';
					break;
				}
				
				
				if ($conf == 1){
					$this->setSetting($prefix, $im);
				} else {
					Auth::user()->setPreference($prefix, $im);
				}
				header('Location:module.php?mod=' . $this->getName() .'&mod_action=admin_config');
				
			break;
			
			case 'saveconfcolors':
				$cbgd = json_decode(Filter::post('bgd'));
				$ctxt = json_decode(Filter::post('txt'));
				$conf = Filter::post('conf');
				$tg = Filter::post('tg');
				$gender = $tg[1];
				$type = $tg[0];
				
				//table colors
					$colors = 	array($gender => 	array(
													'bgd' => $cbgd,
													'txt' => $ctxt
												)
								);
				$scolors = serialize($colors[$gender]);
				
				switch ($type) {
					case 'F':
						$prefix = 'FAN_CHART_COLORS_' . $gender;
					break;
					case 'T':
						$prefix = 'TREE_COLORS_' . $gender;
					break;
				}
			
				if ($conf == 1){ //default values -> module_setting
					$this->setSetting($prefix, $scolors);
				} else { // my own conf -> user_setting
					Auth::user()->setPreference($prefix, $scolors);
				}
				
			break;
			
			
			/////////
		default:
echo '<br> Erreur dans la fonction modAction : mod_action = ' . $mod_action . '<br>'; break;
//			header('HTTP/1.0 404 Not Found');
		}
	}

	

/**
	 * @return string
	 */
	public function css() {
		return WT_STATIC_URL . WT_MODULES_DIR . $this->getName() . '/css/arbre.css';
	}
	
	public function csscp() {
		return WT_STATIC_URL . WT_MODULES_DIR . $this->getName() . '/colorpicker/colorPicker.css';
	}
	

	/**
	 * @return string
	 */
	public function js() {
		return WT_STATIC_URL . WT_MODULES_DIR . $this->getName() . '/js/';
	}
	public function jscp() {
		return WT_STATIC_URL . WT_MODULES_DIR . $this->getName() . '/colorpicker/colorPicker.js';
	}

	/*
	*	bgd fan, etc -> bgdL for booklets
	*	$bgdC main page fr booklet 3P
	*	OK on conf page
	*/
	private function updateColorsUserSetting() {	
	$userId = Auth::Id();
	$uid = Auth::user();
	
		$what = Filter::post('what'); // Colors for Fan / Trees
		
		$Mbgd = Filter::post('cBgdMale');
		$Fbgd = Filter::post('cBgdFemale');
		$Ubgd = Filter::post('cBgdUnknown');
		$Mtxt = Filter::post('cTxtMale');
		$Ftxt = Filter::post('cTxtFemale');
		$Utxt = Filter::post('cTxtUnknown');
		
		
	//Initialise colors for fan chart: ARBRE_FAN_COLORS
	$lightblue = array(173, 216, 230);
	$lightpink = array(254, 182, 193);
	$lightgrey = array(211, 211, 211);
	$white = array(255, 255, 255);
	$beigeclair = array(245, 245, 220);
	$greyp = array(206, 206, 206);
	$blue_duck = array(4, 139, 154);
	$honey = array(218, 179, 10);
	$silver = array(206, 206, 206);
	$black = array(0, 0, 0);
	
	
	$fan_background_image  = 'bgd_filigrane_WT.png';
	$tree_background_image = 'bgd_filigrane_WT.png';
	
	$fan_colors = array('M' => array('bgd'=>$Mbgd, 'txt'=>$Mtxt ),
						'F' => array('bgd'=>$Fbgd, 'txt'=>$Ftxt ),
						'U' => array('bgd'=>$Ubgd, 'txt'=>$Utxt )
						);
					
	//colors for trees with rectangles
		
	$rec_colors = array('M' => array('bgd'=>$Mbgd, 'txt'=>$Mtxt ),
						'F' => array('bgd'=>$Fbgd, 'txt'=>$Ftxt ),
						'U' => array('bgd'=>$Ubgd, 'txt'=>$Utxt )
						);
	
		
	$booklet_background_center = 'bgd_filigrane_WT.png';
	$booklet_background_lat = 'bgd_filigrane_WT.png';
	
	if ($what == 'Fan') {
			$prefix = 'FAN_CHART_';
			$data = $fan_colors;
		} else {
			$prefix = 'TREE_';
			$data = $rec_colors;
		}
	

		//update user's preferences
		//one raw/sex because  $setting_value too short (255 chars)
		foreach ($data as $key => $cdata){
			$uid->setPreference($prefix . 'COLORS_' . $key, serialize($cdata));
		}
			
		Log::addConfigurationLog('Arbre: user\'s colors values');
		
		// fin creation 
			
	}
	
	/*
	*	backgrounds for: 	Fan charts, trees-> same  (one page)  	BGD
	*						Booklets 2P  one bgd					BGD2P
	*						Booklet  3P   	bgd->center = main		BGD3pC BGD3PL
	*						what = BGD/BGD2P/BGD3PC/BGD3PL				bgd->lateral = right & left sides 
	*/
	private function updateBgdsUserSetting() {	
	$userId = Auth::Id();
	$uid = Auth::user();
	
		$what = Filter::post('what'); // Colors for Fan / Trees
				
	//Initialise colors for fan chart: ARBRE_FAN_COLORS
	$lightblue = array(173, 216, 230);
	$lightpink = array(254, 182, 193);
	$lightgrey = array(211, 211, 211);
	$white = array(255, 255, 255);
	$beigeclair = array(245, 245, 220);
	$greyp = array(206, 206, 206);
	$blue_duck = array(4, 139, 154);
	$honey = array(218, 179, 10);
	$silver = array(206, 206, 206);
	$black = array(0, 0, 0);
	
	
	$fan_background_image  = 'bgd_filigrane_WT.png';
	$tree_background_image = 'bgd_filigrane_WT.png';
			
	$booklet_background_center = 'bgd_filigrane_WT.png';
	$booklet_background_lat = 'bgd_filigrane_WT.png';
	
	switch ($what) {

		case 'BGD';
			$setting_name = 'TREE_BACKGROUND';
			$data = Filter::post('bgd');
		break;
		case 'BGD2P';
			$setting_name = 'BOOKLET_2P_BACKGROUND';
			$data = Filter::post('bgd2p');
		break;
		case 'BGD3PC';
			$setting_name = 'BOOKLET_3PC_BACKGROUND';
			$data = Filter::post('bgd3pc');
		break;
		case 'BGD3PL';
			$setting_name = 'BOOKLET_3PL_BACKGROUND';
			$data = Filter::post('bgd3pl');
		break;
		}
	
		//update user's preferences
		$uid->setPreference($setting_name, $data);
	
		Log::addConfigurationLog('Arbre: user\'s backgrounds values');
		
		// fin creation 			
	}


	/*
	*
	*
	*/
	private function mySettingsYN(){
		
	}
	
	

}	

return new ArbreModule;
